
var TmaChannel = (function() {

	function TmaChannel(channel) {

		// call super class constructor
		TmaChannel._super.constructor.apply(this);

		this.chnum = 0;
		this.chtype = 0;
		this.satindex = 0;
		this.logoid = "";
		this.videostreamtype = 0;
		this.flagdelete = 0;
		this.flagcas = 0;
		this.flaglock = 0;
		this.flagskip = 0;
		this.tuner = 0;
		this.transponderindex = 0;
		this.serviceid = 0;
		this.pmtpid = 0;
		this.pcrpid = 0;
		this.videopid = 0;
		this.audiopid = 0;
		this.lcn = 0;
		this.audiostreamtype = 0;
		this.servicename = "";
		this.providername = "";
		this.namelock = 0;
		this.flags2 = 1;
		this.unknown2 = "";

		this.logo = null;

		this.LCNorNR	= 0;
		this.networkid 	= 0;
		this.tsid		= 0;
		this.isAnalogue = false;

		this.isCurrent = false;
		this.recordings = new JKarray();

		this.timers = new JKarray();
		this.timers.sortBy('starttime');

		this.__p.epgSlices = {};
		this.__p.epgEventIds = {};

		this.epgEvents = new JKarray();
		this.epgEvents.sortBy('starttime_local');

		this.transponder = null;

		// init
		if (channel) {
			channel.LCNorNR		= channel.lcn || (channel.chnum + 1);
			channel.networkid 	= channel.logoid ? parseInt( channel.logoid.substr(4, 4), 16 ) : 0;
			channel.tsid		= channel.logoid ? parseInt( channel.logoid.substr(8, 4), 16 ) : 0;
			channel.isAnalogue	= channel.logoid.substring(0,14) == "ffffffffffffff";
		}
		this.init(channel);

	}

	JK.inherit(TmaChannel, JKobject);

	TmaChannel.TYPE_TV		=  0;
	TmaChannel.TYPE_RADIO	=  1;
	TmaChannel.TYPE_ANALOGUE= -1;

	TmaChannel.prototype.videoStreamTypeStr = function() {
		if (this.videostreamtype == 27) {
			return 'H.264';
		} else {
			return 'Mpeg2';
		}
	};

	TmaChannel.prototype.audioStreamTypeStr = function() {
		switch (this.audiostreamtype) {
			case   3: return 'Mpeg';
			case   4: return 'Mpeg2';
			case 129: return 'AC3';
			default : return '';
		}
	};

	TmaChannel.prototype.favPrev = function() {
		var index = tma.channels.favChannels.indexOf(this);
		if (index == -1) {
			return null;
		}
		if (index === 0) {
			// this is the first channel in the fav-list => previous one is the last
			index = tma.channels.favChannels.length;
		}
		return tma.channels.favChannels[index - 1];
	};

	TmaChannel.prototype.favNext = function() {
		var index = tma.channels.favChannels.indexOf(this);
		if (index == -1) {
			return null;
		}
		if (index === tma.channels.favChannels.length - 1) {
			// this is the last channel in the fav-list => next one is the first
			index = -1;
		}
		return tma.channels.favChannels[index + 1];
	};

	/**
	 * returns a time slice
	 *
	 * @param {Number}	sliceNr
	 * @returns {TimeSlice}
	 */
	TmaChannel.prototype.epgSlice = function(sliceNr) {

		if (!this.__p.epgSlices[sliceNr]) {
			this.__p.epgSlices[sliceNr] = new TmaTimeSlice(sliceNr);
		}

		return this.__p.epgSlices[sliceNr];

	};

	/**
	 * load EPG events of a time slice
	 *
	 * @param {Date}	starttime	date string will be converted
	 * @param {Date}	endtime		date string will be converted (optional, if not present: endtime = starttime)
	 * @returns {jQueryDeferred}
	 */
	TmaChannel.prototype.getEvents = function(starttime, endtime) {
		var self = this;

		function fillEpgEvents(epg) {
			if ( epg ) {
				for( i = 0; i < epg.event.length; i++ ) {
					var event = epg.event[i];
					if ( !self.__p.epgEventIds[event.eventid] ) {
						self.__p.epgEventIds[event.eventid] = event;
						self.epgEvents.add( event );
					}
				}
			}
		}

		function addToPipe(pipe, sliceNr) {

			var slice = self.epgSlice( sliceNr );
			return pipe.then(function() {
				return slice.load( self ).done( fillEpgEvents );
			});

		}

		if ( !(starttime instanceof Date) ) {
			starttime = new Date(starttime);
		}
		endtime = endtime || starttime;
		if ( !(endtime instanceof Date) ) {
			endtime = new Date(endtime);
		}

		// get visible time slices
		var fromSliceNr = parseInt( starttime / (TmaEpgEvents.timeSliceLength * 60000) );
		var toSliceNr	= parseInt( endtime	  / (TmaEpgEvents.timeSliceLength * 60000) );

		var pipestart = $.Deferred();
		var pipe = pipestart;
		var i;

		// add all missing EPG events
		for( var sliceNr = fromSliceNr; sliceNr <= toSliceNr; sliceNr++ ) {
			pipe = addToPipe(pipe, sliceNr);
		}

		pipestart.resolve();

		var deferred = $.Deferred();
		pipe.promise().done(function() {

			deferred.resolve(
					self.epgEvents.filter(function(event) {

						var result =
							((starttime	>= event.startDate()) && (starttime	<= event.endDate() )) ||
							((endtime	>= event.startDate()) && (endtime	<= event.endDate() )) ||
							((starttime	<= event.startDate()) && (endtime	>= event.endDate() ));

						return result;

					})
			);

		});

		return deferred.promise();

	};

	TmaChannel.mixin({

		haslogo: function() {
			"watch: {{logo}}";
			return (this.logo !== null);
		},

	});

	return TmaChannel;

})();