/**
 * encapsulates json response object of channel list
 */

var TmaChannels = (function() {

	function TmaChannels() {

		// call super class constructor
		TmaChannels._super.constructor.apply(this);

		// empty default data
		this.count				= 0;
		this.timestamp			= '';
		this.content			= 'channels';
		this.current_chtype		= 0;
		this.current_chnum		= 0;
		this.current_logoid		= '';
		this.favourites			= 0;
		this.fav_name			= 'undefined';
		this.fav_name_hex		= '';

		this.channels			= [];

		this.favChannels		= [];

		// init
		this.init();

		this.__p.logoidIndex = {};

		/**
		 * updates favChannels on fav_name change
		 */
		this.on('change', 'fav_name', function (event, favGroup, self) {

			self.getFavChannels(self.favChannels, favGroup);

		});

		this.channels.on(
				'add',
				'*',
				/** @param {TmaChannel} channel */
				function(e, channel, self) {

					/** @returns {TmaTransponder} */
					var transponder = tma.transponders.find(
							/** @param {TmaTransponder} transp */
							function(transp) {
								return transp.transponder_num == channel.transponderindex;
							}
					);
					if ( !transponder ) {
						transponder = new TmaTransponder( {
							transponder_num: channel.transponderindex,
						} );
						tma.transponders.push( transponder );
					}
					transponder.channels.push( channel );
					channel.transponder = transponder;

				}
		);

	}
	JK.inherit(TmaChannels, JKobject);

	TmaChannels.prototype.extend = function(channels) {

		// extract channel list
		var channellist = [];
		if (channels.channels) {
			channellist = channels.channels;
			delete channels.channels;
		}

		// call inherited extend
		this.constructor._super.extend.call(this, channels);

		// convert channels and add them
		for (var i = 0; i < channellist.length; i++) {
			var channel = channellist[i];
			// remove DVB control chars (for WebControl < 1.41m)
			channel.servicename = channel.servicename.replace(/[\x86\x87]/g, '');
			channel.providername = channel.providername.replace(/[\x86\x87]/g, '');
			this.__p.logoidIndex[channel.logoid] = this.channels.length;
			this.channels.push(new TmaChannel(channel));
		}

	};

	/**
	 * get a channel object through channelid
	 *
	 * @param {String}	channelid (logoid)
	 * @returns {TmaChannel} or null, if not found
	 */
	TmaChannels.prototype.channel = function(channelid) {

		if ( this.__p.logoidIndex[channelid] !== undefined ) {
			return this.channels[this.__p.logoidIndex[channelid]];
		} else {
			return null;
		}

	};

	/**
	 * get a channel object through chtype/chnum
	 *
	 * @param {String}	channelid (logoid)
	 * @returns {TmaChannel} or null, if not found
	 */
	TmaChannels.prototype.channelByTypeNum = function(chtype, chnum) {

		return tma.channels.channels.find(
				/** @param {TmaChannel} channel */
				function(channel) {
					return (channel.chtype == chtype) && (channel.chnum == chnum);
				}
		);

	};

	/**
	 * Get a listing of all channels from the PVR
	 *
	 * @returns jQueryDeferred
	 */
	TmaChannels.prototype.load = function () {

		var self = this;

		var params = {
				'function':	'channel',
				action:		'get!'
		};

		return $.getJSON('/api', params).done(function(data) {

			self.extend(data);

		}).fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in TmaChannels.load()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
				}
			});

		});

	};

	/**
	 * fills tgtArray with channels of a favourites group
	 *
	 * @param {JKarray}		tgtArray
	 * @param {String}		favGroup	if empty, return all channels
	 * @param {Function}	filterCb	optional filter callback function( {TmaChannel} )
	 */
	TmaChannels.prototype.getFavChannels = function (tgtArray, favGroup, filterCb) {

		var typeFilter,
			any = !favGroup,
			i = 0,
			len = 0;

		tgtArray.length = 0;

		switch (favGroup) {
			case '__analogue'	: typeFilter = TmaChannel.TYPE_ANALOGUE;break;
			case '__tv'			: typeFilter = TmaChannel.TYPE_TV;		break;
			case '__radio'		: typeFilter = TmaChannel.TYPE_RADIO;	break;
		}

		if (any || (typeFilter !== undefined)) {

			for( i = 0, len = this.channels.length; i < len; i++) {
				var channel = this.channels[i];

				if (
						( any || (channel.chtype == typeFilter) ) &&
						( !filterCb || (filterCb(channel) === true) )
				) {
					tgtArray.push(channel);
				}
			}

		} else {
			// return channels of a favourite group

			/** @param {TmaFavouritesGroup} group */
			tma.favourites.eachGroup(function(i, group) {
				if (group.name == favGroup) {
					for( i = 0, len = group.channels.length; i < len; i++) {
						var channel = group.channels[i];

						if ( !filterCb || (filterCb(channel) === true) ) {
							tgtArray.push( channel );
						}
					}
				}
			});
		}

	};

	return TmaChannels;

})();