
var TmaEpgEvents = (function() {

	function TmaEpgEvents( events ) {

		// call super class constructor
		TmaEpgEvents._super.constructor.apply(this);

		// empty default data
		this.timestamp =			'';
		this.content =				'epg_events';
		this.count =				0;
		this.error =				0;
		this.requested_starttime =	'';
		this.requested_endtime =	'';
		this.requested_chtype =		0;
		this.requested_chnum =		0;
		this.requested_logoid = 	'';
		this.requested_epgsource =	'none';
		this.epgsource =			'none';
		this.event =				[];
		/** @returns {TmaChannel} */
		this.channel =				null;

		// init
		this.init(events);

	}

	JK.inherit(TmaEpgEvents, JKobject);

	TmaEpgEvents.prototype.extend = function(events) {

		// extract channel list and fav_name
		var eventlist = [];
		if (events.event && $.isArray(events.event)) {
			eventlist = events.event;
			delete events.event;
		}

		// call inherited extend
		this.constructor._super.extend.call(this, events);

		// convert events and add them
		var day = 99;
		for (var i = 0; i < eventlist.length; i++) {

			var event = new TmaEpgEvent(eventlist[i], events.channel);
			event.timestamp = events.timestamp;
			this.event.push( event );

			// Calculate extra data

			// first event of day?
			event.firstOfDay = (event.startDate().getDate() > day);
			day = event.startDate().getDate();

		}

	};

	/**
	 * loops through all events
	 *
	 * @param {Function}	callback	callback function will be called for each recording with parameters (nr, event)
	 */
	TmaEpgEvents.prototype.eachEvent = function( callback ) {

		for (var i = 0; i < this.event.length; i++ ) {
			if (callback.apply( this.event[i], [i, this.event[i]]) === false) {
				break;
			}
		}
	};

	/**
	 * @param {String} name
	 * @returns {TmaFavouritesGroup}
	 */
	TmaEpgEvents.prototype.getEvent = function( eventid ) {

		for (var i = 0; i < this.event.length; i++) {
			if (this.event[i].eventid == eventid) {
				return this.event[i];
			}
		}

		return false;

	};

	/**
	 * default query length in minutes
	 */
	TmaEpgEvents.timeSliceLength = 360; // 6h => 4 slices per day

	return TmaEpgEvents;

})();


var TmaEpgEvent = (function() {

	function TmaEpgEvent(event, channel) {

		// call super class constructor
		TmaEpgEvent._super.constructor.apply(this);

		// empty default data
		this.eventid			= 0;
		this.datastatus			= 0;
		this.runningstatus		= 0;
		this.starttime_local	= '';
		this.endtime_local		= '';
		this.timezone			= 0;
		this.duration			= 0;
		this.satindex			= 0;
		this.networkid			= 0;
		this.tsid				= 0;
		this.serviceid			= 0;
		this.svctype			= 0;
		this.svcnum				= 0;
		this.logoid				= '';	// epgsearch only
		this.lang				= '';
		this.parental			= 0;
		this.namelength			= 0;
		this.descriptionlength	= 0;
		this.textlength			= 0;
		this.name				= '';
		this.description		= '';
		this.text				= '';
		this.citid				= 0;
		//this.contentidentifier= 0;
		//this.contentidentifier_hex= '';
		//this.genre_text		= '';
		//this.genre_code		= '';
		this.genre_value		= 0;
		this.au_genre			= 0;
		this.sourceflag			= 0;
		this.icechannel			= 0;
		this.parental_text		= '';

		/** @returns {TmaChannel} */
		this.channel			= null;
		/** @returns {TmaReservation} */
		this.reservation		= null;
		/** @returns {TmaGenre} */
		this.genre				= null;

		this.timestamp = '';
		this.isCurrentEvent = false;
		this.currentPercent = 0;

		// init
		if (event && !event.timestamp) {
			event.timestamp = tma.status.timestamp;
		}
		this.init(event);

		// apply channel
		if (event) {
			if (!channel) {
				// if contructor is called without a channel object
				// and the plain event object has a logoid
				if (event.logoid) {
					// then set the channel object
					channel = tma.channels.channel(event.logoid);
				}
			}
			if (channel) {
				this.channel = channel;
			}
		}

		// watch timestamp
		var self = this;
		tma.status.on('change', 'timestamp', function(event, val) {
			self.timestamp = val;
		});

		if (this.channel) {

			// set recording state due to existing timers
			for( var i = 0, len = this.channel.timers.length; i < len; i++) {
				var timer = this.channel.timers[i];
				self.setReservation(timer);
			}

			this.channel.timers.on('add', '*', function(e, timer) {
				// set recording state of this event
				self.setReservation(timer);
			});

			this.channel.timers.on('del', '*', function(e, nil, parent, index, delTimer ) {
				// remove recording state of this event
				if (
						self.reservation &&
						(self.reservation.timer == delTimer)
				) {
					self.reservation = null;
				}
			});

		}

	}

	JK.inherit(TmaEpgEvent, JKobject);

	/**
	 * tests, if any reservation of a timer covers this event
	 *
	 * @param {TmaTimer} timer
	 */
	TmaEpgEvent.prototype.setReservation = function(timer) {

		for( var j = 0, len = timer.reservations.length; j < len; j++) {
			var reservation = timer.reservations[j];

			if (
					(this.starttime_local >= reservation.starttime) &&
					(this.endtime_local   <= reservation.endtime)
			) {
				this.reservation = reservation;
				break;
			}
		}

	};

	/**
	 * extended event text, where newlines are replaced by BR
	 *
	 * @returns {String}
	 */
	TmaEpgEvent.prototype.htmlText = function() {
		return this.text.replace(/(?:\r\n|\r|\n|\x8a)/g, '<br>');
	};

	/**
	 * Date object of 'starttime_local'
	 *
	 * @returns {Date}
	 */
	TmaEpgEvent.prototype.startDate = function() {
		return new Date(this.starttime_local);
	};

	/**
	 * Date object of 'endtime_local'
	 *
	 * @returns {Date}
	 */
	TmaEpgEvent.prototype.endDate = function() {
		return new Date(this.endtime_local);
	};

	/**
	 * load single event
	 */
	TmaEpgEvent.prototype.load = function (channel, pvrTimestamp) {

		if (channel) {
			var self = this;
			channel.getEvents( pvrTimestamp ).done(function(events) {
				if (events.length) {
					// copy relevant data
					self.channel = channel;
					self.extend( events[0] );
				}
			});

		}

	};

	/**
	 * test, if a related object intersects with this event
	 *
	 * @param {Object}	relObject	object with properties "starttime" and "endtime"
	 */
	TmaEpgEvent.prototype.isIntersecting = function( relObject ) {
		return (
				// related object starts before eventstart and ends after event start
				((relObject.starttime <= this.starttime_local) && (relObject.endtime	>  this.starttime_local)) ||
				// or related object starts in event's time span
				((relObject.starttime >= this.starttime_local) && (relObject.starttime	<  this.endtime_local))
		);
	};

	TmaEpgEvent.mixin({

		/**
		 * get the related genre object
		 *
		 * @returns {TmaGenre}
		 */
		setGenre: function() {
			"watch: {{genre_value}}";

			this.genre = tma.genres.genre(this.genre_value);
		},

		durationFmt: function() {
			"watch: {{duration}}";
			return tma.formatDuration(this.duration);
		},

		starttimeFmt: function() {
			"watch: {{starttime_local}}";
			return tma.date("H:i", new Date(this.starttime_local));
		},

		endtimeFmt: function() {
			"watch: {{endtime_local}}";
			return tma.date("H:i", new Date(this.endtime_local));
		},

		updateCurrent: function() {
			"watch: {{timestamp}}";
			this.isCurrentEvent = (this.starttime_local <= this.timestamp) && (this.endtime_local > this.timestamp);
			if ( this.isCurrentEvent ) {
				var timestamp = new Date(this.timestamp);
				this.currentPercent = parseInt((timestamp - this.startDate()) * 100 / (this.endDate() - this.startDate()));
			} else {
				this.currentPercent = 0;
			}
		},

	});

	return TmaEpgEvent;

})();

function TmaTimeSlice(sliceNr) {

	var start	= sliceNr * TmaEpgEvents.timeSliceLength * 60000;
	var end		= start + TmaEpgEvents.timeSliceLength * 60000;

	/**
	 * slice number relative to timstamp 0
	 */
	this.nr = sliceNr;

	/**
	 * query state:
	 * 	0: empty
	 * 	1: loading
	 *  2: ready
	 */
	this.state = 0; // empty

	/**
	 * slice start in PVR date format
	 */
	this.starttime = tma.toPvrDate( start );

	/**
	 * slice end in PVR date format
	 */
	this.endtime = tma.toPvrDate(end);

	/**
	 * api EPG response for this slice
	 *
	 * @returns {TmaEpgEvents}
	 */
	this.epg = new TmaEpgEvents();

	/**
	 * load EPG events of a channels slice
	 *
	 * @param {TmaChannel}	channel
	 */
	this.load = function(channel) {

		var self = this;
		var deferred = $.Deferred();

		if (!this.state) {

			this.state = TmaTimeSlice.STATE_BUSY; // loading
			tma.apiGetEpg(channel, this.starttime, this.endtime).done(function(epg) {
				self.state = TmaTimeSlice.STATE_READY; // loaded
				self.epg.extend(epg);

				deferred.resolve(self.epg);
			}).fail(function() {
				self.state = TmaTimeSlice.STATE_ERROR;
			});

		} else if (this.state == TmaTimeSlice.STATE_READY) {

			// data is available => resolve asynchronously
			setTimeout(function() {
				deferred.resolve();
			}, 1);

		}

		return deferred.promise();

	};

}
TmaTimeSlice.STATE_ERROR = -1;
TmaTimeSlice.STATE_EMPTY =  0;
TmaTimeSlice.STATE_BUSY	 =  1;	// loading
TmaTimeSlice.STATE_READY =  2;
