var TmaFiles = (function() {

	function TmaFiles( files ) {

		// call super class constructor
		TmaFiles._super.constructor.apply(this);

		// empty default data
		this.count		= 0;
		this.timestamp	= "";
		this.content	= "files";
		this.directory	= "";
		this.files		= [];

		/**
		 * drive of current path section (internal: status.drive; external: status.drives[x]
		 */
		this.drive		= null;

		this.init( files );

	}
	JK.inherit(TmaFiles, JKobject);

	TmaFiles.prototype.extend = function(files) {

		var t_start = new Date().getTime();
		$('BODY').css('cursor', 'wait');
		this.files.detach( true );

		// call inherited extend
		this.constructor._super.extend.call(this, files, ['files']);

		// convert files and add them
		if (files.files) {
			for (var i = 0, len = files.files.length; i < len; i++) {
				var file = new TmaFile();
				file.set( files.files[i] );
				this.files.add(file);
			}
		}

		this.files.attach();
		setImmediate(function() {
			var t_end = new Date().getTime();
			console.log('TmaFiles.extend: ' + (t_end - t_start) + ' ms');
			$('BODY').css('cursor', '');
		});

	};

	/**
	 * find file by property value
	 *
	 * @param {String}	key		property
	 * @param			value
	 * @returns {TmaFile} or false
	 */
	TmaFiles.prototype.find = function( key, value ) {

		for (var i = 0, len = this.files.length; i < len; i++ ) {
			if (this.files[i][key] == value) {
				return this.files[i];
			}
		}
		return false;
	};

	/**
	 * return the sort function
	 * (private method)
	 *
	 * @param {String} key			property name of the file object
	 * @param {String} direction	'asc' or 'desc'
	 * @returns {Function}
	 */
	function getSortFn(key, direction) {

		function getProperty(obj, prop) {

			var path = prop.split('.');

			if (path.length > 1) {
				var key = path.shift();
				var subobj = obj[key];
				if ( (subobj) && (typeof subobj == 'object') ) {
					return getProperty(subobj, path.join('.'));
				} else {
					return "";
				}

			} else {
				if ( typeof obj[path[0]] == 'function' ) {
					return obj[path[0]]();
				} else {
					return obj[path[0]];
				}
			}

		}

		var dir = (direction == 'desc') ? -1 : 1;

	    return function(a, b) {

	    	// always compare filetype first
			var aAttr = parseInt(a.attr);
			var bAttr = parseInt(b.attr);

			if (aAttr < bAttr) {
				return dir;
			} else if (aAttr > bAttr) {
				return -dir;
			} else {

				var aVal = getProperty(a, key);
				var bVal = getProperty(b, key);
				if (typeof aVal == 'string') aVal = aVal.toLowerCase();
				if (typeof bVal == 'string') bVal = bVal.toLowerCase();

				if (aVal < bVal) {
					return -dir;
				} else if (aVal > bVal) {
					return dir;
				} else {
					return 0;
				}
			}

	    };
	}

	/**
	 * sort the filelist
	 *
	 * @param {String} key		property name of the file object
	 * @param {String} dir		sort direction: 'asc' or 'desc'
	 */
	TmaFiles.prototype.sort = function(key, dir) {

		this.files.sort( getSortFn( key, dir ) );

	};

	TmaFiles.mixin({

		pathSection: function() {
			"watch: {{directory}}";

			var m = this.directory.match(/^\/mnt\/hd\/([^/]+)/);
			if (m) {
				// standard path, e.g. /mnt/hd/DataFiles
				this.drive = tma.status.drive;
				return m[1];
			} else {
				m = this.directory.match(/^(\/mnt\/[^/]+)/);
				if (m) {
					// external drive, e.g. /mnt/sdb
					for ( var i = 0, len = tma.status.drives.length; i < len; i++ ) {
						var drive = tma.status.drives[i];
						if ( this.directory.indexOf( drive.path.replace(/^\/dev/, '/mnt') ) === 0) {
							this.drive = drive;
							break;
						}
					}
					return m[1];
				}
			}
			return '';
		},

	});

	return TmaFiles;

})();
