/**
 * encapsulates EPG-Genre object of json response
 *
 * @param {TmaGenre}	genre
 */
var TmaGenre = (function() {

	function TmaGenre(genre) {

		// call super class constructor
		TmaGenre._super.constructor.apply(this);

		// empty default data
		this.genre_code		= '';
		this.genre_value	= 0;
		this.genre_text		= '';
		this.text_color		= '';
		this.background		= '';
		this.coloring		= false;

		this.__p.subGenres = [];

		this.init(genre);

		this.setStyle();

		// save color settings
		this.on('change', 'text_color', function(event, val, obj) {
			if ( obj.genre_code ) {
				if ( val !== obj.prefTextColor() ) {
					tma.setSetting( 'gc_txt_' + obj.genre_code, val );
				}
			}
		});

		// save color settings
		this.on('change', 'background', function(event, val, obj) {
			if ( obj.genre_code ) {
				if ( val !== obj.prefBackground() ) {
					tma.setSetting( 'gc_bgr_' + obj.genre_code, val );
				}
			}
		});

		// save 'enable' settings
		this.on('change', 'coloring', function(event, val, obj) {
			if ( obj.genre_code ) {
				if ( val !== obj.prefColoring() ) {
					tma.setSetting( 'gc_ena_' + obj.genre_code, val );
				}
			}
		});

	}

	JK.inherit(TmaGenre, JKobject);

	TmaGenre.prototype.extend = function(genre) {

		// get color settings
		genre.text_color	= TmaGenre.prototype.prefTextColor.call(genre);
		genre.background	= TmaGenre.prototype.prefBackground.call(genre);
		genre.coloring		= TmaGenre.prototype.prefColoring.call(genre);

		// call inherited extend
		this.constructor._super.extend.call(this, genre);

	};

	TmaGenre.prototype.prefTextColor = function() {
		return tma.setting( 'gc_txt_' + this.genre_code, '' );
	};

	TmaGenre.prototype.prefBackground = function() {
		var defaultBgr = '';
		if (tma.genres.returned_source == 'au') {
			switch (this.genre_value) {
				case 0x20:	defaultBgr = '#87caf3'; break;	//AU News
				case 0x30:	defaultBgr = '#fcc45a'; break;	//AU Entertainment
				case 0x40:	defaultBgr = '#f08787'; break;	//AU Sports
				case 0x50:	defaultBgr = '#e8e753'; break;	//AU children's
				case 0x90:	defaultBgr = '#ff22aa'; break;	//AU Educational
				case 0xC0:	defaultBgr = '#ff9999'; break;	//AU Comedy
				case 0xD0:	defaultBgr = '#b8eb58'; break;	//AU Drama
				case 0xE0:	defaultBgr = '#babafe'; break;	//AU Documentary
			}
		} else {
			switch (this.genre_value) {
				case 0x10:	defaultBgr = '#b8eb58'; break;	//ETSI Movie/Drama
				case 0x20:	defaultBgr = '#87caf3'; break;	//ETSI News/Current Affairs
				case 0x30:	defaultBgr = '#fcc45a'; break;	//ETSI Show/Game Show
				case 0x40:	defaultBgr = '#f08787'; break;	//ETSI Sports
				case 0x50:	defaultBgr = '#e8e753'; break;	//ETSI Children's/Youth Programmes
				//case 0x60:	defaultBgr = '#ff9999'; break;	//ETSI Music/Ballet/Dance
				case 0x70:	defaultBgr = '#80f6de'; break;	//ETSI Arts/Culture
				case 0x80:	defaultBgr = '#babafe'; break;	//ETSI Social/Political Issues/Economics
				case 0x90:	defaultBgr = '#ff22aa'; break;	//ETSI Education/Science/Factual Topics
				//case 0xA0:	defaultBgr = '#ff22aa'; break;	//ETSI Leisure Hobbies
			}
		}
		return tma.setting( 'gc_bgr_' + this.genre_code, defaultBgr );
	};

	TmaGenre.prototype.prefColoring = function() {
		return tma.setting( 'gc_ena_' + this.genre_code, true );
	};

	TmaGenre.mixin({

		setStyle: function() {
			"watch: {{coloring}}, {{text_color}}, {{background}}";

			this.removeStyle();

			if (this.coloring) {

				var style =
					'<style type="text/css">' +
						'.genre[genre-value="' + this.genre_value + '"] {' +
							this.styleString() +
						'}' +
					'</style>';

				this.__p.$style = $(style).appendTo('HEAD');

			}

			var subGenres = this.subGenres();
			for( var i = 0; i < subGenres.length; i++) {
				subGenres[i].setStyle();
			}

		},

	});

	/**
	 * returns the style string, if genre-colouring is enabled
	 * if this is a sub-genre, and there are no colours set,
	 * the colours of the parent genre is returned
	 *
	 * @param {Boolean}	coloring	should genre colouring be used?
	 */
	TmaGenre.prototype.style = function( coloring ) {
		var result = '';
		if ( coloring ) {
			result = this.styleString();
		}
		return result;
	};

	TmaGenre.prototype.removeStyle = function() {

		if ( this.__p.$style ) {
			this.__p.$style.remove();
			this.__p.$style = null;
		}

	};

	/**
	 * creates the style string based on defined colours
	 * e.g. color:#000000;background-color:#FF0000;
	 */
	TmaGenre.prototype.styleString = function() {
		var result = '';
		if (this.coloring) {

			if ( ( this.genre_value & 15 ) && !this.text_color && !this.background ) {
				// try parent genre
				var genre = tma.genres.genre(this.genre_value & 240);
				if (genre) {
					result = genre.styleString();
				} else {
					result = 'color:#000;';
				}
			} else {

				if (this.text_color) {
					result += 'color:' + this.text_color + ';';
				} else if (this.background.match(/#[0-9a-fA-F]{6}/)) {
					/* calculate text colour if only background specified*/
					var r = parseInt(this.background.substr(1,2),16);
					var g = parseInt(this.background.substr(3,2),16);
					var b = parseInt(this.background.substr(5,2),16);
					// calculate luminance of the YIQ color space in percent
					var Y = ((r*299)+(g*587)+(b*114))/2550;
					result += 'color:' +( (Y >= 50) ? '#000' : '#fff' ) + ';';
				} else {
					/* Force black if no color specified*/
					result += 'color:#000;';
				}

				if (this.background) {
					result += 'background-color:' + this.background + ';';
				}

			}

		}

		return result;
	};

	/**
	 * get count of sub genres of this genre
	 */
	TmaGenre.prototype.countSubGenres = function() {

		var count = 0;

		if ( (this.genre_value & 0xF) === 0) {

			var genre_value = (this.genre_value & 0xF0) + 1;
			while (genre_value < (this.genre_value & 0xF0) + 16) {

				var genre = tma.genres.genre(genre_value);
				if (genre) {
					if (!genre.genre_text.match(/^Genre [0-9a-fA-F]{2}$/)) {
						count++;
					}
				}

				genre_value++;
			}

		}

		return count;

	};

	/**
	 * get all sub genres of this genre
	 */
	TmaGenre.prototype.subGenres = function() {

		if ( (this.__p.subGenres.length === 0) && (this.genre_value & 0xF) === 0) {

			var genre_value = (this.genre_value & 0xF0) + 1;
			while (genre_value < (this.genre_value & 0xF0) + 16) {

				var genre = tma.genres.genre(genre_value);
				if (genre) {
					if (!genre.genre_text.match(/^Genre [0-9a-fA-F]{2}$/)) {
						this.__p.subGenres.push(genre);
					}
				}

				genre_value++;
			}

		}

		return this.__p.subGenres;

	};

	return TmaGenre;

})();
