/**
 * encapsulates json response object of gere list
 *
 * @param {TmaGenres}	genres
 */
var TmaGenres = (function() {

	function TmaGenres(genres) {
		var i = 0;

		// call super class constructor
		TmaGenres._super.constructor.apply(this);

		// empty default data
		this.count				= 0;
		this.timestamp			= '';
		this.content			= 'epg_genre';
		this.genres				= [];
		this.requested_source	= '';
		this.returned_source	= '';
		this.requested_language	= '';

		// init genre array
		for(i = 0; i < 256; i++) {
			this.genres.push(null);
		}

		this.init(genres);

	}

	JK.inherit(TmaGenres, JKobject);

	TmaGenres.prototype.extend = function(genres) {

		// call inherited extend
		this.constructor._super.extend.call(this, genres, ['genres']);

		// convert genres and add them
		if (genres.genres && $.isArray(genres.genres)) {
			for (var i = 0, len = genres.genres.length; i < len; i++) {

				var genre = new TmaGenre(genres.genres[i]);
				this.genres[genre.genre_value] = genre;
			}
		}

	};

	/**
	 * returns a genre by genre_value or genre_code (hex)
	 *
	 * @param {mixed}	genre_code	String is interpreted as genre_code (hex), number as genre_value
	 * @returns {TmaGenre}	or null
	 */
	TmaGenres.prototype.genre = function( genre_code ) {

		var genre_value = 0;

		if ( typeof genre_code == 'string' ) {
			genre_value = parseInt(genre_code, 16);
		} else {
			genre_value = genre_code;
		}

		if (genre_value >= 0 && genre_value < 256) {
			return this.genres[genre_value];
		}

		return null;

	};

	/**
	 * get all main genres
	 *
	 * @returns {JKarrey}
	 */
	TmaGenres.prototype.getMainGenres = function() {

		var mainGenres = [];

		var genre_value = 16;
		while (genre_value < 256) {

			if (this.genres[genre_value]) {
				if (!this.genres[genre_value].genre_text.match(/^Genre [0-9a-fA-F]{2}$/)) {
					mainGenres.push(this.genres[genre_value]);
				}
			}

			genre_value += 16;
		}

		return mainGenres;

	};

	return TmaGenres;

})();
