
var TmaSearchText = (function() {

	// define search text class
	function TmaSearchText( textCriteria ) {

		// call super class constructor
		TmaSearchText._super.constructor.apply(this);

		this.text = '';
		this.title = true;
		this.desc = true;
		this.ext = true;
		this.points = '50';

		this.init( textCriteria );

	}
	JK.inherit(TmaSearchText, JKobject);

	/**
	 * get the location code of title, desc, ext
	 *
	 * @returns {String}
	 */
	TmaSearchText.prototype.location = function() {

		//Work out the SmartEPG code for the text fields to be searched.
		var locValue = 0;
		if ( this.title ) { locValue += 1; }
		if ( this.desc  ) { locValue += 2; }
		if ( this.ext	) { locValue += 4; }

		switch(locValue) {
			case 1:	//Only search the title
				return "T";
			case 2: //Only search the description
				return "D";
			case 3: //Search the title and description
				return "A";
			case 4: //Only search the extended description
				return "E";
			case 5: //Only search the title and extended description
				return "q";		//"q" for 'this makes no sense but WC and generate this combination', this is not an SE option but an extension for EPGapprentice
			case 6: //Search the descriptions only
				return "S";		//"S" for Synopsis, this is not an SE option but an extension for EPGapprentice
			case 7: //Search everything
				return "B";
			default:
				return "";
		}
	};

	/**
	 * validate text criteria data
	 *
	 * @returns {Number}	error code
	 * 						0: OK
	 * 						1: no search location
	 * 						2: empty search text
	 * 						7: invalid search text
	 */
	TmaSearchText.prototype.validate = function() {

		var errorFlag = 0;
		if ( !this.locationValid() ) {
			errorFlag = 1;
		} else if( !this.text ) {
			errorFlag = 2;
		} else if ( !this.textValid() ){
			errorFlag = 7; // invalid regular expression
		}

		return errorFlag;

	};

	TmaSearchText.mixin({

		textValid: function() {
			"watch: {{text}}";
			if( !this.text ) {
				return false;
			} else {
				var regex = this.text;
				if ( !regex.match(/^\/(.*)\/$/) ) {
					// If text is not encased by slashes, assume that this contains wildcards and convert to regex
					regex = "/" + this.text.replace("?", ".").replace("*", ".*") + "/";
				}

				// test regular expression
				try {
					new RegExp( regex );
					return true;
				} catch (e) {
					return false; // invalid regular expression
				}
			}
		},

		locationValid: function() {
			"watch: {{title}} {{desc}} {{ext}}";
			return this.title || this.desc || this.ext;
		},

	});

	return TmaSearchText;

})();