var TmaTimer = (function() {

	function TmaTimer(timer) {

		// call super class constructor
		TmaTimer._super.constructor.apply(this);

		// empty default data
		this.index			= -1;
		this.tunerindex		= 0;
		this.recmode		= 0;
		this.demuxpath		= 0;
		this.manualrec		= 0;
		this.satindex		= 0;
//		this.servicetype	= 0;
//		this.serviceid		= 0;
		this.lcn			= 0;
		this.logoid			= '';
//		this.chname			= '';
		this.satname		= '';
		this.orgnetid		= 0;
		this.duration		= 0;
		this.filename_hex	= '';
		this.filename		= '';
		this.starttime		= '';
		this.endtime		= '';
		this.pmtid			= 0;
		this.isrec			= 1;
		this.nameset		= 0;
		this.epgmarker		= 0;
		this.eventid1		= 0;
		this.eventid2		= 0;
//		this.serviceindex	= 0;
		this.analoguetype	= 0;
		this.icetv			= 0;

		/**
		 * internal unique id during objects lifecycle
		 */
		this.id				= 0;

		/**
		 * written by TmaTimers.extend
		 */
		this.timestamp		= '';

		this.reservationtype = 0;

		this.reservations = new JKarray();

		/**
		 * data for timer modifications
		 */
		this.modified = false;
		this.modLogoid			= '';
		this.modReservationtype = 0;
		this.modFilename		= '';
		this.modStarttime		= '';
		this.modEndtime			= '';
		this.modDuration		= 0;
		this.modIsrec			= 1;


		// init and apply initial data
		this.init(timer);
		if (timer) {
			this.calcReservations();
			this.modReset();
		}

		this.reservations.sortBy('dist');

		this.destruct = function() {
			while( this.reservations.length > 0) {
				var reservation = this.reservations.pop();
				var ix = tma.timers.reservations.indexOf(reservation);
				if (ix >= 0) {
					tma.timers.reservations.splice(ix, 1);
				}
			}
		};

	}

	JK.inherit(TmaTimer, JKobject);

	TmaTimer.prototype.calcReservations = function() {

		if (
				(this.index >= 0) &&
				this.starttime &&
				this.endtime
		) {

			/**
			 * Valid days for recurring reservation
			 * default: every day is valid
			 */
			var recurringValidDays = [1,1,1,1,1,1,1];

			if (this.reservationtype == 2) {
				// weekend, only sunday and saturday is valid
				recurringValidDays = [1,0,0,0,0,0,1];
			} else if (this.reservationtype == 4) {
				// weekdays, only monday to friday is valid
				recurringValidDays = [0,1,1,1,1,1,0];
			}

			var timer_start = new Date(this.starttime).getTime();
			var dist = 0;
			var day = 0;
			var daysStep = (this.reservationtype == 3) ? 7 : 1; // weekly?
			var reservation;

			var indexFinder = function(reservation) {
				return reservation.dist == dist;
			};

			while (day <= 14 ) {

				var dayOfWeek = parseInt(tma.date('w', timer_start + dist));

				var ix = this.reservations.findIndex( indexFinder );
				if ( recurringValidDays[dayOfWeek] ) {
					if (ix == -1) {
						reservation = new TmaReservation( this, dist, (day > 0) );
						this.reservations.add( reservation );
						tma.timers.reservations.add( reservation );
					}
				} else {
					if (ix >= 0) {
						reservation = this.reservations[ix];
						this.reservations.splice(ix, 1);
						ix = tma.timers.reservations.indexOf(reservation);
						if (ix >= 0) {
							tma.timers.reservations.splice(ix, 1);
						}
					}
				}

				if (!this.reservationtype) {
					// one time
					break;
				}

				// next
				dist		+= daysStep*24*3600000;
				day			+= daysStep;

			}

		}

	};

	TmaTimer.mixin({
		channel: function() {
			"watch: {{logoid}}";
			return tma.channels.channel(this.logoid);
		},

		isnew: function() {
			"watch: {{index}}";
			return this.index < 0;
		},

		/**
		 * Reservation type
		 * 0: One Time
		 * 1: Daily
		 * 2: Weekends
		 * 3: Weekly
		 * 4: Week Days
		 * 5: Intelligent Record
		 *
		 * @param {Number} set	new value
		 */
		setReservationtype: function(set) {
			if (set !== undefined) {
				if (parseInt(set) < 0 || parseInt(set) > 5) {
					throw 'set timer.reservationtype: value out of range';
				}
				this.modReservationtype = parseInt(set);
			}
		},

		starttimeFmt: function() {
			"watch: {{starttime}}";
			return tma.date("H:i", new Date(this.starttime));
		},

		endtimeFmt: function() {
			"watch: {{endtime}}";
			return tma.date("H:i", new Date(this.endtime));
		},

		durationFmt: function() {
			"watch: {{duration}}";
			return tma.formatDuration(this.duration, 1);
		},

		frequencyFmt: function() {
			"watch: {{reservationtype}}";
			if (tma.i18.frequency[this.reservationtype]) {
				return tma.i18.frequency[this.reservationtype].name;
			} else {
				return '???';
			}
		},

		frequencyIcon: function() {
			"watch: {{reservationtype}}";
			return function() {
				switch (this.reservationtype) {
					case 0: return 'ic-onetime';
					case 1: return 'ic-alldays';
					case 2: return 'ic-weekends';
					case 3: return 'ic-weekly';
					case 4: return 'ic-weekdays';
					case 5: return 'ic-search';
				}
				return 'ic-empty';
			};
		},

		/**
		 * file name of this timer (non-UTF Toppy safe)
		 *
		 * @param {Boolean]	removeExt	remove file extension? (default = false)
		 * @returns {String}
		 */
		getFilename: function(removeExt) {
			"watch: {{filename}}";

			/** @returns {String} */
			var name = (tma.config.utftoppy || !this.filename_hex) ?
					this.filename || ''
					: tma.decodeHexname(this.filename_hex);

			if (removeExt) {
				return name.replace(/\.[^.]+$/, '');
			} else {
				return name;
			}

		},

		/**
		 * file name of this timer (non-UTF Toppy safe)
		 * @returns {String}
		 */
		filenameNoExt: function() {
			"watch: {{filename}}";

			/** @returns {String} */
			var name = (tma.config.utftoppy || !this.filename_hex) ?
					this.filename || ''
					: tma.decodeHexname(this.filename_hex);

			return name.replace(/\.[^.]+$/, '');

		},

		/**
		 * recalc reservations on reservationtype change
		 */
		onReservTypeChange: function() {
			"watch: {{reservationtype}}";
			this.calcReservations();
		},

		/**
		 * recalc reservations on index change
		 */
		onIndexChange: function() {
			"watch: {{index}}";
			this.calcReservations();
		},

		/**
		 * adjust endtime on starttime change
		 */
		onModStarttimeChange: function() {
			"watch: {{modStarttime}}";
			if	(this.modStarttime && this.duration)	{
				var start = new Date(this.modStarttime);
				this.modEndtime = tma.toPvrDate( start.getTime() + this.modDuration * 60000 );
			}
		},

		/**
		 * adjust endtime on duration change
		 */
		onModDurationChange: function() {
			"watch: {{modDuration}}";
			if	(this.modStarttime)	{
				var start = new Date(this.modStarttime);
				this.modEndtime = tma.toPvrDate( start.getTime() +  this.modDuration * 60000 );
			}
		},

		/**
		 * adjust duration on endtime change
		 */
		onModEndtimeChange: function() {
			"watch: {{modEndtime}}";
			if	(this.modEndtime && this.modStarttime)	{
				var start = new Date(this.modStarttime);
				var end = new Date(this.modEndtime);
				this.modDuration = parseInt( (end.getTime() - start.getTime()) / 60000 );
			}
		},

		/**
		 * return channel of modLogoid
		 */
		modChannel: function() {
			"watch: {{modLogoid}}";
			return tma.channels.channel(this.modLogoid);
		},

		/**
		 * reset modifications to original values
		 */
		modReset: function() {
			this.modLogoid			= this.logoid;
			this.modReservationtype = this.reservationtype;
			this.modFilename		= this.filenameNoExt();
			this.modStarttime		= this.starttime;
			this.modDuration		= this.duration;
			//this.modEndtime			= this.endtime; // calculated by duration change
			this.modIsrec			= this.isrec;
		},

		/**
		 * update modified status
		 */
		setModified: function() {
			"watch: {{modLogoid}},{{modReservationtype}},{{modStarttime}},{{modDuration}},{{modIsrec}}";
			this.modified =
			(this.modLogoid			!= this.logoid) ||
			(this.modReservationtype!= this.reservationtype) ||
			(this.modFilename		!= this.filenameNoExt()) ||
			(this.modStarttime		!= this.starttime) ||
			(this.modDuration		!= this.duration) ||
			(this.modIsrec			!= this.isrec);
		},

		modStarttimeFmt: function() {
			"watch: {{modStarttime}}";
			return function() {
				return tma.date("H:i", new Date(this.modStarttime));
			};
		},

		modEndtimeFmt: function() {
			"watch: {{modEndtime}}";
			return function() {
				return tma.date("H:i", new Date(this.modEndtime));
			};
		},

		modDurationFmt: function() {
			"watch: {{modDuration}}";
			return function() {
				return tma.formatDuration(this.modDuration, 1);
			};
		},

		modFrequencyFmt: function() {
			"watch: {{modReservationtype}}";
			return function() {
				return tma.i18.frequency[this.modReservationtype].name;
			};
		},

	});

	return TmaTimer;

})();
