/**
 * channels module
 */
tma.registerModule( channelspage );
channelspage.prototype = new tmaModule();
function channelspage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'channelspage';

	this.pageGroup = 'channels';

	this.dependencies = {

			css: [
			      "channels.css"
			      ],

			templates: [
			            "channels.html",
			            "config.html",
			            "channelinfo.html",
			            "morefn.html"
			            ]
	};

	var self = this;

	var $contentTmpl, $configTmpl, $moreFnTmpl;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );
		tma.fn.addFeature( this, 'openChannelInfo' );

		return this.loadLanguage().done(function() {

			// extend TmaChannel
			if (!TmaChannel.prototype.typeStr) {
				TmaChannel.prototype.typeStr = function() {
					switch (this.chtype) {
						case  1: return self.radio		|| 'Radio';
						case -1: return self.analogue	|| 'Analogue';
						default: return self.tv			|| 'TV';
					}
				};
			}

		});

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * this page gets removed => remove databinding templates
	 */
	this.cleanup = function() {

		JK.removeTemplate($contentTmpl);
		JK.removeTemplate($configTmpl);
		JK.removeTemplate($moreFnTmpl);

		$('#timestamp').show();

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		// set channels list
		tma.channels.fav_name = tma.prefs.favGroup;

		this.init().done(function() {

			$('#timestamp').hide();

			$contentTmpl = JK.template(
					self.templates.channels,
					{
						prefs:		prefs,
						gPrefs:		tma.prefs,
						channels:	tma.channels.favChannels,
					});

			//DMC filter AV channels
			$contentTmpl.find('.channel[channelid="ffffffffffffff02"]').hide();
			$contentTmpl.find('.channel[channelid="ffffffffffffff04"]').hide();

			$contentTmpl.appendTo( 'DIV#content' );

			// apply the channelsacross setting
			applyChannelsAcross();

			// add event listener to select channel
			$contentTmpl.on('click', 'DIV.channel .channel-name, DIV.channel .channellogo', function(e) {
				// 'this' is the clicked element
				self.selectCurrentChannel( $(this).parent() );
			});

			// add event listener to open more-fn
			$contentTmpl.on('click', 'DIV.channel-fn > I', function(e) {

				var channel = tma.channels.channel( $(this).parents('[channelid]').attr('channelid') );

				self.displayMoreFn(channel);

			});

			tma.call('addConfigItem', ['MainMenu', self]);

			self.insertConfig();

		});

	};

	/**
	 * creates and displays the "more functions" panel of a channel
	 *
	 * @param {TmaChannel}	channel
	 */
	this.displayMoreFn = function( channel ) {

		function closeMoreFn() {
			JK.removeTemplate($moreFnTmpl);
			tma.call('channelCloseMoreFn');
		}

		// remove previous moreFn's
		$('DIV.morefn').fadeOut(function() {
			closeMoreFn();
		});

		$moreFnTmpl = JK.template(
			this.templates.morefn,
			{
				i18: this.i18,
				channel: channel
			}
		).appendTo( $('.channel[channelid="' + channel.logoid + '"] .channel-fn') )
		// fade in the panel
		.animate({opacity:1}, {
			start: function() {
				var offs = $(this).offset();
				var w = $(this).outerWidth();
				var ww = $(window).width();
				if (offs.left + w >= ww ) {
					$(this).addClass('morefn-right');
				}
			}
		});

		// close morefn
		$moreFnTmpl.on('click', '.morefn-close', function() {
			$(this).parent().fadeOut(function() {
				closeMoreFn();
			});
		});

		// show channel infos
		$moreFnTmpl.on('click', '.info', function() {
			if ( channel ) {
				tma.call('openChannelInfo', [channel]);
			}
			// remove moreFn
			closeMoreFn();
		});

		// let other modules insert their buttons
		tma.call('channelCreateMoreFn', [$moreFnTmpl, channel]);

	};

	/**
	 * selects a channel and marks the element with css class 'current-channel'
	 *
	 * @param {jQueryObject} $channel	channel element with attribute channelid
	 */
	this.selectCurrentChannel = function ($channel) {

		var channel = tma.channels.channel( $channel.attr('channelid') );
		if ( channel ) {
			tma.apiChannelSet( channel.chtype, channel.chnum ).fail(function(xhr, errorType, error) {

				if (errorType == 'tma-error') {
					new jkqAlert({
						title:	tma.i18.error.dialog_title,
						text:	tma.i18.error.channels.set
					}).confirm();
				}

			});
		}

	};

	/**
	 * apply the channelsacross setting
	 */
	var applyChannelsAcross = function() {

		$('DIV.channels').css( { maxWidth: "" } );
		if (prefs.channelsAcross) {
			var $channels = $('DIV.channel');
			if ($channels.length > 1) {
				var left1 = $channels.eq(0).position().left;
				var left2 = $channels.eq(1).position().left;
				var w = parseInt(1 + left2 - left1) * prefs.channelsAcross;
				$('DIV.channels').css( { maxWidth: w } );
			}
		}

	};

	/**
	 * open popup with channel informations
	 *
	 * @param {tmaFeature}	f
	 * @param {TmaChannel}	channel
	 */
	this.openChannelInfo = function(f, channel) {

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			var popup = new jkqWindow();
			popup.width = 400;

			popup.show();
			popup.setTitle( channel.servicename );
			popup.closeOnOutsideClick();

			var $content = JK.template(
					self.templates.channelinfo,
					channel
			).appendTo(popup.container.find('DIV.jkqWindowContent'))

			.find('SPAN[hex]').each(function() {
				// convert values with attribute 'hex' to hex-string
				// toHex(): https://gist.github.com/rubycon/1699103
				var p = parseInt($(this).attr('hex'));
				var value = parseInt($(this).text());
				p = (!p) ? 0 : p;
				var n = (value >>> 0).toString(16);
				while(n.length < p) n = '0' + n;
				$(this).text('0x' + n);
			});

			$(popup).on('close', function() {
				JK.removeTemplate($content);
			});

			popup.adjustPos({
				hAlign:	'right',
				effect:	'slide-rtl'
			});

			// return popup instance
			f.stop(popup);

		});

	};

	this.insertConfig = function () {

		$configTmpl = JK.template(
				this.templates.config,
				{
					i18:			this.i18,
					i18g:			tma.i18,	// global language def.
					prefs:			prefs,
					gPrefs:			tma.prefs,
					favgroups:		tma.favourites.groups,
				}
		).appendTo('DIV#config');

		// change channelsacross setting
		$('DIV#config DIV.cfg-columns *.less').click(function() {

			var columns = prefs.channelsAcross - 1;
			if (columns >= 0) {
				prefs.setChannelsAcross(columns);
				applyChannelsAcross();
			}

		});
		$('DIV#config DIV.cfg-columns *.more').click(function() {

			var columns = prefs.channelsAcross + 1;
			prefs.setChannelsAcross(columns);
			applyChannelsAcross();

		});

		var $fav = $('DIV#config SELECT[name="favourites"]');
		$fav.val( tma.prefs.favGroup );
		$fav.change(function() {

			tma.prefs.setFavGroup( $(this).val() );
			tma.channels.fav_name = $(this).val();

		});

		$('DIV#config INPUT[name="logotext"]')
			.click(function() {

				prefs.setDisplayMode( $('DIV#config INPUT[name="logotext"]:checked').val() );

			})
			.filter('[value="' +  prefs.displayMode + '"]' ).attr('checked', true);

	};

	/* Settings */

	var prefs = JK.model({

			/**
			 * displayMode setting
			 *
			 * @type {Number}		1: display logo
			 * 						2: display name
			 * 						3: display both
			 */
			displayMode: self.localSetting( 'channel_display', 3 ),

			/**
			 * set the displayMode setting
			 *
			 * @param {Number}	set		new displayMode setting
			 * 							1: display logo
			 * 							2: display name
			 * 							3: display both
			 */
			setDisplayMode: function(set) {
				if ((set >= 1) && (set <= 3)) {
					self.setLocalSetting( 'channel_display', set );
					this.displayMode = set;
				}
			},

			/**
			 * channelsacross setting
			 *
			 * @return {Number}
			 */
			channelsAcross: parseInt( self.localSetting( 'channelsacross', 0 ) ),

			/**
			 * set the channelsacross setting
			 *
			 * @param {Number}	set		new channelsacross setting
			 */
			setChannelsAcross: function(set) {
				self.setLocalSetting( 'channelsacross', parseInt(set) );
				this.channelsAcross = parseInt(set);
			},

	});

}
