/**
 * control module
 */
tma.registerModule( controlpage );
controlpage.prototype = new tmaModule();
function controlpage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'controlpage';

	this.pageGroup = 'control';

	var self = this;

	var keyMap = {
			keyOnOff:		"power",
			keyMute:		"mute",
			key1:			"1",
			key2:			"2",
			key3:			"3",
			key4:			"4",
			key5:			"5",
			key6:			"6",
			key7:			"7",
			key8:			"8",
			key9:			"9",
			keyRev:			"recall",
			key0:			"0",
			keyInfo:		"info",
			keyPRGup:		"chup",
			keyPRGdown:		"chdown",
			keyVOLup:		"volup",
			keyVOLdown:		"voldown",
			keyOk:			"ok",
			keyUp:			"up",
			keyDown:		"down",
			keyLeft:		"left",
			keyRight:		"right",
			keyMenu:		"menu",
			keyEpg:			"guide",
			keyFiles:		"playlist",
			keyExit:		"exit",
			keyRed:			"f1",
			keyGreen:		"f2",
			keyYellow:		"f3",
			keyBlue:		"f4",
			keyWhite:		"option",	// ???
			keySubtitle:	"subtitle",
			keyVideotext:	"ttx",
			keyAudio:		"audiotrack",
			keyVF:			"vformat",
			keyAR:			"uhf",
			keyTime:		"sleep",
			keyTvRadio:		"tvradio",
			keyRec:			"rec",
			keyStop:		"stop",
			keySat:			"sat",
			keyM:			"m",		// ???
			keyRewind:		"rew",
			keyPause:		"pause",
			keyPlay:		"play",
			keyForward:		"fwd",
			keyPrev:		"prev",
			keySlow:		"slow",
			keyCut:			"ab",
			keyNext:		"next",
			keyAux:			"tvsat",
			keyFav:			"fav",
			keyNoUse:		"nouse",
	};

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			$( self.templates.content.render(
					{	// data
						screen: tma.PVRscreen
					},
					{	// partials
						remotes: self.templates.remotes
					}
			))
			.appendTo('#content');

			self.showRemote( self.setting( 'remote_type', 'srp2410' ) );

			$.each(keyMap, function(keyClass, keyname) {

				$('#content DIV.remote BUTTON.' + keyClass).attr('key', keyname);

			});

			// event handler: remote buttons

			//http://stackoverflow.com/questions/8503453/click-event-called-twice-on-touchend-in-ipad
			//http://stackoverflow.com/questions/2915833/how-to-check-browser-for-touchstart-support-using-js-jquery
			var startevent	= ('ontouchstart' in document.documentElement) ? 'touchstart' : 'mousedown';
			var stopevent	= ('ontouchstart' in document.documentElement) ? 'touchend'	 : 'mouseup';

			$('#content DIV.remote BUTTON[key]').on(startevent, function() {
				$(this).prop('mousedowntime', new Date().getTime());
			}).on(stopevent, function() {

				var key = $(this).attr('key');
				var deltaT = new Date().getTime() - $(this).prop('mousedowntime');
				if (deltaT > 500) {
					key = 'long-' + key;
				}

				tma.sendremote( key ).done(function() {
					// refresh after 500ms
					window.setTimeout(function() {
						self.refreshPVRscreen();
					}, 500);
				});

			});

			// append the current screen
			$('#pvrscreen').click(function() {
				self.refreshPVRscreen();
			});

			// add config
			tma.call('addConfigItem', ['MainMenu', self]);

			self.insertConfig();

		});

	};

	/**
	 * this page gets removed, so hide the channels config
	 */
	this.cleanup = function() {

	};

	this.refreshPVRscreen = function () {

		$('IMG#pvrscreen').attr( 'src', tma.PVRscreen + "#e" + Date.now() );

	};

	this.showRemote = function ( type ) {

		$('.remote').hide().filter('.' + type ).css({display: 'inline-block'});

	};

	this.insertConfig = function () {

		$( this.templates.cfgremote.render({
			i18: this.i18
		}) ).appendTo('DIV#config');

		// event handler: remote selection
		$('DIV#config SELECT[name="remoteTypes"]').on('change', function() {

			var type = $(this).val();
			self.setSetting( 'remote_type', type );
			self.showRemote( type );

		}).val( self.setting( 'remote_type', 'srp2410' ) );

	};

}
