/**
 * dashboard module
 */
tma.registerModule( dashboard );
dashboard.prototype = new tmaModule();
function dashboard() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'dashboard';

	this.pageGroup = 'control';

	this.dependencies = {

			css: [
				"dashboard.css",
			],

			templates: [
			    "content.html"
			],

	};

	var self = this;

	var $contentTmpl;
	var timerUpdateInterval = null;

	var prevStatusUpdateInterval;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			$('#timestamp').hide();

			// setup channel selector module
			var channelselect = tma.fn.getModule('channelselect');
			channelselect.init().done(function() {

				prevStatusUpdateInterval = tma.status.setRefreshInterval(10);

				// Status mixin (playmode, current file, active recordings, ... formatter)
				self.initStatusFormatters();

				// Timer mixin (textRemaining formatter)
				self.initTimerFormatters();

				// prepare template data (mixed with data, required by channel selector)
				var data = $.extend(
						{
							i18: self.i18,
							status: tma.status,
							timers: tma.timers.timers,
						},
						channelselect.getTemplateData( tma.status, [ "ffffffffffffff00" ] ) // blacklist
				);

				/**
				 * create template
				 */

				$contentTmpl = JK.template(
						self.templates.content,
						data,
						{ channelselect: channelselect.templates.content }	// partials
				).appendTo('DIV#content');


				channelselect.addEventHandler( $contentTmpl, function(channelid) {

					// switch channel
					var ch = tma.channels.channel( channelid );
					if ( ch ) {
						tma.apiChannelSet( ch.chtype, ch.chnum );
					}

				});

				self.addEventHandlerCurrent();
				self.addEventHandlerRecordings();
				self.addEventHandlerTimer();

				tma.timers.timers.sortBy('starttime', 'asc');
				tma.apiGetTimers();

				//Refresh the timers every 60 seconds
				timerUpdateInterval = setInterval(
						function() {
							tma.apiGetTimers();
						},
						60000
				);

			});

		});

	};

	/**
	 * this page gets removed, so remove config class
	 */
	this.cleanup = function() {

		if (timerUpdateInterval) {
			clearInterval(timerUpdateInterval);
			timerUpdateInterval = null;
		}

		// restore status refresh interval
		tma.status.setRefreshInterval(prevStatusUpdateInterval);

		$('#timestamp').show();

	};

	/**
	 * add formating functions for the dashboard template to the TmaStatus class
	 */
	this.initStatusFormatters = function() {

		// Status mixin (playmode, current file, active recordings, ... formatter)
		if (!TmaStatus.prototype.dashboardPlayStatus) {
			TmaStatus.mixin({
				dashboardPlayStatus: function() {
					"watch: {{playmode}}";
					return self.i18.playstatus[this.playmode].name;
				},
				dashboardFilePlayingStr: function() {
					"watch: {{currentblock}}";
					return self.i18.play_recording.format({
						filename:	this.filenameNoExt(),
						played:		tma.formatDuration( ((this.duration * this.currentblock) / this.totalblock).toFixed(0) ),
						duration:	tma.formatDuration(this.duration),
					});
				},
				dashboardActiveRecordingsStr: function() {
					"watch: {{activeRecordings}}";
					if (this.activeRecordings === 0) {
						// No recording in progress
						return self.i18.no_recordings;
					} else if (this.activeRecordings == 1) {
						// One recording in progress
						return this.activeRecordings + ' ' + self.i18.one_recording;
					}
					// Multiple recordings in progress
					return this.activeRecordings + ' ' + self.i18.many_recordings;
				},
			});
		}

	};

	/**
	 * add formating functions for the dashboard template to the TmaTimer class
	 */
	this.initTimerFormatters = function() {

		// Timer mixin (textRemaining formatter)
		if (!TmaTimer.prototype.dashboardTextRemaining) {
			TmaTimer.mixin({
				dashboardTextRemaining: function() {
					"watch: {{starttime}}, {{timestamp}}";
					return function() {
						var	dateNow = new Date(this.timestamp);		//'Now' on the PVR
						var dateOnlyNow = new Date(dateNow);
						dateOnlyNow = dateOnlyNow.setHours(0,0,0,0);		//'Today' on the PVR
						var	dateStart = new Date(this.starttime);
						var dateOnlyStart = new Date(dateStart);
						dateOnlyStart = dateOnlyStart.setHours(0,0,0,0);
						var daysRemaining = (dateOnlyStart - dateOnlyNow) / 60000 / 24 / 60;
						var minsRemaining = (dateStart - dateNow) / 60000;
						var textRemaining;

						if (daysRemaining === 0 || minsRemaining <= 360) {
							//This the timer is due today, or in the next 6 hours

							textRemaining = self.i18.timer_remaining_mins.format({
								remaining: tma.formatDuration(minsRemaining.toFixed(0), 0),
							});

						} else if (daysRemaining == 1) {
							//If the timer is due tomorrow

							textRemaining = self.i18.timer_remaining_tomorrow.format({
								starttime: tma.date( "H:i", this.starttime ),
							});

						} else {
							//If the timer is due after tomorrow

							//TODO - If this is more than 7 days in the future, use the date
							textRemaining = self.i18.timer_remaining_day.format({
								day: tma.i18.days[dateStart.getDay()].name,
								starttime: tma.date( "H:i", this.starttime ),
							});

						}
						return textRemaining;
					};
				},
			});
		}

	};

	/**
	 * current table
	 */
	this.addEventHandlerCurrent = function() {

		// add event listener for playback action keys
		$contentTmpl.on('click', 'BUTTON[play-action]', function() {

			switch ( $(this).attr('play-action') ) {

				case 'stop':
					tma.stopPlayback();
					break;

				case 'pause':
					tma.apiFilesPause();
					break;

				case 'play':
					tma.apiFilesResume();
					break;

			}

		});

		// add event listener for Skip keys
		$contentTmpl.on('click', 'BUTTON.but-skip', function() {

			tma.apiFilesJump( $(this).attr('skip') );

		});

		// add event listener for Transport keys
		$contentTmpl.on('click', 'BUTTON.transport', function() {

			tma.sendremote( $(this).attr('rcu') );

		});

		// add event listener to display EPG
		$contentTmpl.on('click', '.dashboard-clm-name', function(e) {

			tma.call( 'showEvent', [ tma.status.currentEvent() ] );

		});


	};

	/**
	 * recordings table
	 */
	this.addEventHandlerRecordings = function() {

		// stop recording button
		$contentTmpl.on('click', 'BUTTON.rec-stop', function(e) {

			var slot = $(this).parents('TR').attr('slot');
			var rec = tma.status.recording[ slot ];

			if ( rec ) {
				new jkqDialog({
					title:	self.i18.stoprec_dialog_title.format({slot: rec.slot}),
					text:	self.i18.stoprec_dialog_text.format({filename: rec.getFilename(true)}),
					buttons: {
						ok:				 tma.i18.respond_yes,
						jkqDialogCancel: tma.i18.respond_cancel
					}
				}).confirm(function(button) {
					if (button == 'ok') {
						tma.stopRecording(rec.slot);
					}
				});
			}
		});

		// lengthen and shorten buttons
		$contentTmpl.on('click', 'BUTTON.rec-change', function(e) {

			var slot = $(this).parents('TR').attr('slot');
			var rec = tma.status.recording[ slot ];

			var newDuration = rec.duration + parseInt( $(this).attr('delta') );

			//Note: Changing the duration to less than the current recorded time stops the recording.

			//Prevent the user from making the duration shorter then the current elapsed recording time
			if(newDuration > (rec.recordedsec / 60)) {

				tma.apiFilesDuration( slot, newDuration );

			}

		});

		// toggle buttons
		$contentTmpl.on('click', 'BUTTON.rec-morefn', function(e) {
			$(this).parents('.rec-buttons-toggle').toggleClass( 'rec-buttons-visible' );
		});

	};

	/**
	 * event handlers of timer table
	 */
	this.addEventHandlerTimer = function() {

		// add event listener to edit timers
		$contentTmpl.on('click', '.dashboard-timers TD.timer-row', function(e) {

			var timer = tma.timers.find( $(this).parents('TR').attr('timerid') );
			if (timer) {

				tma.call( 'editTimer', [ timer ] );

			}
		});

	};

	// DMCs todos in previous version
	//TODO - When stop on time-shifting, screen does blank and does not return to live TV.
	//TODO - Channel selector: For each channel in the dropdown, show the current EPG entry.
	//TODO - Sometimes no EPG data is returned even when EPG data is on the PVR. TAP problem?  Often isolated to certain channels.
	//TODO - If playing at other than x1 speed, display the speed.
	// recordings
	//TODO - Button to display the INF/INF+ data.
	//TODO - Show logo, % elapsed/remaining, stop recording, extend, start chase-play, get info (inf+ too), reuse module from file list if possible
	//TODO - Red progress bar
	// timers
	//TODO - Calculate how long until the timer triggers and colour-code the line.

}
