/**
 * EPG module
 */
tma.registerModule( epgchannelpage );
epgchannelpage.prototype = new tmaModule();
function epgchannelpage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'epgchannelpage';

	this.pageGroup = 'epg';

	var self = this;

	this.dependencies = {

			css: [
			      "epgch.css"
			      ],

			js: [
			     "../genrecolours/genrecolours.js",
				 "../channelselect/channelselect.js"
			     ],

			templates: [
					     "content.html",
					     "config.html",
					     "head.html",
			]
	};

	var $contentTmpl, $configTmpl, $stickyTmpl;

	/**
	 * current selected channel
	 */
	this.templateData = null;

	/**
	 * total time range ( maxTimeSlices x timeSliceLength )
	 */
	this.maxTimeSlices = 28;	// 7 days

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			// select channels list
			tma.channels.fav_name = tma.prefs.favGroup;

			// load genre list
			tma.apiEpgGenre().done(function() {

				self.showEPG();

				self.showSticky();

				tma.call('addConfigItem', ['MainMenu', self]);
				self.insertConfig();

			});

		});

	};

	/**
	 * this page gets removed, so hide the channels config
	 */
	this.cleanup = function() {

		// remove event handler
		$(window).off('scroll.epgch');

		// remove templates
		JK.removeTemplate( $contentTmpl );
		JK.removeTemplate( $configTmpl );
		JK.removeTemplate( $stickyTmpl );

	};

	this.showSticky = function() {

		// setup channel selector module
		var channelselect = tma.fn.getModule('channelselect');
		channelselect.init().done(function() {

			var blacklist = [  // blacklist
							   "ffffffffffffff00",
							   "ffffffffffffff02",
							   "ffffffffffffff04",
							   ];

			// render template
			$stickyTmpl = JK.template(
					self.templates.head,
					$.extend(
							{data: self.templateData},
							channelselect.getTemplateData( self.templateData, blacklist )
					),
					{ channelselect: channelselect.templates.content }	// partials
			).appendTo( '.sticky-content' );

			$('.epgch').css('padding-top', $stickyTmpl.outerHeight());

			channelselect.addEventHandler( $stickyTmpl, function(channelid) {
				// change channel
				var channel = tma.channels.channel( channelid );
				if (channel) {
					self.templateData.channel = channel;
				}
			});

		});

	};

	/**
	 * load and display an EPG grid
	 *
	 * currently the EPG data is loaded for all channels of the selected favourites group (=> channels page)
	 * the time range is currently fix 3 hours from now
	 */
	this.showEPG = function () {

		// setup content data (channel)
		if (!this.templateData) {
			this.templateData = JK.model({
				channel	: null,
				date	: tma.status.timestamp,
				dateFmt : function() {
					"watch: {{date}}";
					return tma.date(self.i18.list_date_fmt, this.date);
				}
			});

			this.templateData.on('change', 'channel', function(e, channel) {

				if (channel) {
					var starttime = new Date( tma.status.timestamp );
					var minutes = self.maxTimeSlices * TmaEpgEvents.timeSliceLength;
					var endtime = starttime.getTime() + minutes * 60000;

					channel.getEvents( starttime, endtime ).done(function() {

						// scroll to templateData.date
						var rect = $('.sticky-content').get(0).getBoundingClientRect();
						var yoffs = rect.bottom;
						var $events = $('.epgch .epgch-event');
						$events.each(function() {

							var $event = $(this);
							var endtime = $event.attr('endtime');
							if (endtime > self.templateData.date) {
								// scroll to this event
								var top = $event.offset().top;
								$(window).scrollTop( top - yoffs );
//								$event.addClass('jk-mark');
								return false; // break
							}

						});


					});
				}

			});

			// add formatters
			if (!TmaEpgEvent.prototype.epgchDate) {
				TmaEpgEvent.mixin({

					epgchDate		: function() {
						"watch: {{starttime_local}}";
						return tma.date(self.i18.shortdate_fmt, this.starttime_local);
					},

				});
			}

		}

		// render template
		$contentTmpl = JK.template(
				this.templates.content,
				{
					i18:		this.i18,
					prefs:		prefs,
					data:		this.templateData,
				}
		).appendTo( '#content' );

		this.templateData.channel = tma.channels.channel( tma.status.current_logoid );

		// update timers
		tma.timers.load();

		this.createEventHandler();

	};

	this.createEventHandler = function() {

		// show timer form
		$contentTmpl.on('click', '.epgch-evt-reservation', function(e) {

			// prevent opening EPG details
			e.stopPropagation();

			var timer = tma.timers.find( $(this).attr('timerid') );
			if (timer) {

				// hope, that any module handles the 'editTimer' event
				tma.call( 'editTimer', [ timer ] );

			}

		});

		// show event details
		$contentTmpl.on('click', '.epgch-event', function(e) {

			var $event = $(this);

			var eventid = parseInt( $(this).attr('eventid') );
			if ( eventid ) {

				var event = self.templateData.channel.epgEvents.find( function(event) {
					return event.eventid == eventid;
				});

				if ( event ) {

					// check, if any module handles the 'showEvent' event
					// and prevent reopening of the same info
					if (
							(tma.fn.showEvent) &&
							!$event.hasClass('jk-mark')
					) {

						tma.fn.showEvent( event ).done(function(popup) {
							popup.$relatedEvent = $event;
							$(popup).on('close', function() {
								this.$relatedEvent.removeClass('jk-mark');
							});
						});

						$event.addClass('jk-mark');

					}

				}

			}

		});


		$(window).on('scroll.epgch', function() {

			// compute timestamp of current top event
			var rect = $('.sticky-content').get(0).getBoundingClientRect();
			var yoffs = rect.bottom;
			var st = $(window).scrollTop();

			var $events = $('.epgch .epgch-event');
			$events.each(function() {

				var $event = $(this);
				var top = $event.offset().top;

				if (top - st > yoffs) {

//					$events.removeClass('jk-mark');
//					$event.addClass('jk-mark');
					var starttime = $event.attr('starttime');
					var endtime   = $event.attr('endtime');
					if (
							(self.templateData.date < starttime) ||
							(self.templateData.date > endtime)
					) {
						// if top event does not touch current templateData.date,
						// set new current templateData.date
						self.templateData.date = starttime;
					}
					return false; // break
				}

			});

		});

	};

	this.insertConfig = function () {


		//var $config = $( this.templates.config.render( this.i18.config ));

		$configTmpl = JK.template(
				this.templates.config,
				{
					i18:			this.i18.config,
					i18g:			tma.i18,	// global language def.
					prefs:			prefs,
					gPrefs:			tma.prefs,
					favgroups:		tma.favourites.groups,
				}
		).appendTo('DIV#config');

		/* description */
		$configTmpl.find('INPUT[name="description"]').click(function() {

			prefs.setDescrMode( $(this).val() );

		})
		// select
		.filter('[value="' +  prefs.descrMode + '"]' ).prop('checked', true);

		/* text */
		$configTmpl.find('INPUT[name="text"]').click(function() {

			prefs.setTextMode( $(this).val() );

		})
		// select
		.filter('[value="' +  prefs.textMode + '"]' ).prop('checked', true);

		/* text depending on description */
		$configTmpl.find('INPUT[name="textdep"]').click(function() {

			prefs.setTextDepMode( $(this).prop('checked') );

		})
		// select
		.prop( 'checked', prefs.textDepMode );

		/* show/hide genre */
		$configTmpl.find('INPUT[name="genre"]').click(function() {

			prefs.setGenreMode( $(this).val() );

		})
		// select
		.filter('[value="' +  prefs.genreMode + '"]' ).prop('checked', true);

		tma.fn.getModule('genrecolours').createConfig(
				prefs.genreColoring,
				function(isEnabled) { // onEnableChange
					prefs.setGenreColoring(isEnabled);
				}
		).appendTo('DIV#config');

	};

	/* settings */

	var prefs = JK.model({

			/**
			 * description display mode (none|first|*full)
			 *
			 * @type {String}
			 */
			descrMode: self.localSetting('descr', 'full'),

			/**
			 * set the description display mode (none|first|full)
			 *
			 * @param {String}	set
			 */
			setDescrMode: function(set) {
				if (
						(set == 'none') ||
						(set == 'first') ||
						(set == 'full')
				) {
					self.setLocalSetting( 'descr', set );
					this.descrMode = set;
				}
			},

			/**
			 * text display mode (none|*first|full)
			 *
			 * @return {String}
			 */
			textMode: self.localSetting('text', 'first'),

			/**
			 * set the text display mode (none|first|full)
			 *
			 * @param {String}	set
			 */
			setTextMode: function(set) {
				if (
						(set == 'none') ||
						(set == 'first') ||
						(set == 'full')
				) {
					self.setLocalSetting( 'text', set );
					this.textMode = set;
				}
			},

			/**
			 * text-if-no-description display mode (true|*false)
			 *
			 * @type {Boolean}
			 */
			textDepMode: self.localSetting('textdep', false),

			/**
			 * set the text-if-no-description-mode (true|false)
			 *
			 * @param {Boolean}	set
			 */
			setTextDepMode: function(set) {
				if (set !== undefined) {
					self.setLocalSetting( 'textdep', set === true );
					this.textDepMode = set;
				}
			},

			/**
			 * genre display mode (none|*show)
			 *
			 * @return {String}
			 */
			genreMode: self.localSetting('genre', 'show'),

			/**
			 * set the genre display mode (none|show)
			 *
			 * @param {String} set
			 */
			setGenreMode: function(set) {
				if (
						(set == 'none') ||
						(set == 'show')
				) {
					self.setLocalSetting( 'genre', set );
					this.genreMode = set;
				}
			},

			/**
			 * genre-coloring (*on|off)
			 *
			 * @return {Boolean}
			 */
			genreColoring: parseInt( self.localSetting( 'genre-coloring', 1 ) ) == 1,

			/**
			 * set the genre-coloring (on/off)
			 *
			 * @param {Boolean}	set		new genre-coloring setting
			 */
			setGenreColoring: function(set) {
				self.setLocalSetting( 'genre-coloring', (set) ? 1 : 0 );
				this.genreColoring = (set) ? true : false;
			},

	});

}
