/**
 * EPG module
 */
tma.registerModule( epgpage );
epgpage.prototype = new tmaModule();
function epgpage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'epgpage';

	this.pageGroup = 'epg';

	var self = this;

	this.dependencies = {

			css: [
					"epg.css",
					"../timerspage/timers.css"
			      ],

			js: [
					"../../js/vendor/shortcut.js",
					"../genrecolours/genrecolours.js"
			     ],

			templates: [
		    		"config.html",
		    		"content.html",
		    		"navbar.html",
		    		"conflicts.html",
			]
	};

	/**
	 * Starttime of the epg-timespan in milliseconds rounded to minutes
	 *
	 * @type {Number}
	 */
	this.starttime = 0;

	/**
	 * Endtime of the epg-timespan in milliseconds rounded to minutes
	 *
	 * @type {Number}
	 */
	this.endtime = 0;

	this.channels = new JKarray();

	/**
	 * state of the EPG-grid
	 */
	var state = {
			curr_day:		'',
			scrollTop:		-1,
			scrollLeft:		-1,
			width:			0,
			height:			0,
			channelHeight:	0,
	};

	/**
	 * initial state (pseudo-)constant
	 */
	var EMPTY_STATE = $.extend({}, state);

	var $contentTmpl, $configTmpl, $stickyTmpl;
	var favHandler, conflictAddHandler;

	/**
	 * initialise module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			// select channels list
			tma.channels.fav_name = tma.prefs.favGroup;

			// load genre list
			tma.apiEpgGenre().done(function() {

				$('<div id="debug">').appendTo('BODY')
				.css({
					position: 'absolute',
					top: 0,
					right: 0,
				});

				self.showEPG();

				self.showSticky();

				// insert config menu item
				tma.call('addConfigItem', ['MainMenu', self]);

				self.insertConfig();

				// add shortcuts
				self.addShortcuts();

				$("#timestamp").hide(); // prevent vertical document scrollbar (vScroll in EPG-frame)

			});

		});

	};

	/**
	 * this page gets removed, so hide the modules configuration
	 */
	this.cleanup = function() {

		$('DIV#debug').remove();

		// remove event handling
		tma.channels.off(favHandler);
		tma.timers.conflicts.off(conflictAddHandler);
		favHandler = null;
		conflictAddHandler = null;

		// remove templates
		JK.removeTemplate( $contentTmpl );
		JK.removeTemplate( $configTmpl );
		JK.removeTemplate( $stickyTmpl );

		$(window).off('resize.epgpage');

		// remove shortcuts
		this.removeShortcuts();

		// show app status
		$("#timestamp").show();

	};

	this.addShortcuts = function() {

		shortcut.add("Left"		, function() { $('.epg-nav BUTTON.ic-left').click(); });
		shortcut.add("Right"	, function() { $('.epg-nav BUTTON.ic-right').click(); });
		shortcut.add("Page_up"	, function() { $('.epg-nav BUTTON.epg-nav-day-next').click(); });
		shortcut.add("Page_down", function() { $('.epg-nav BUTTON.epg-nav-day-prev').click(); });
		shortcut.add("p"		, function() { $('.epg-nav BUTTON.epg-nav-primetime').click(); });
		shortcut.add("t"		, function() { $('.epg-nav BUTTON.ic-clock').click(); });
		shortcut.add("Down"		, function() { $('.epg-container').scrollTop(state.scrollTop + state.height); });
		shortcut.add("Up"		, function() { $('.epg-container').scrollTop(state.scrollTop - state.height); });

	};

	this.removeShortcuts = function() {

		shortcut.remove("Left");
		shortcut.remove("Right");
		shortcut.remove("Page_up");
		shortcut.remove("Page_down");
		shortcut.remove("p");
		shortcut.remove("t");
		shortcut.remove("Down");
		shortcut.remove("Up");

	};

	/**
	 * callback of timer modification drag-Objects
	 * @this {jkqDragDrop}
	 */
	this.onTimerModified = function(left, top, width) {

		/** @type jQueryObject */
		var $source = this.$source;
		$source.css({top: 0});
		if ($source.is('[timerid]')) {
			var timer = tma.timers.find( $source.attr('timerid') );
			if (timer) {
				timer.modStarttime = tma.toPvrDate( Math.round( left		  / prefs.scale) * 60000 + self.starttime );
				timer.modEndtime   = tma.toPvrDate( Math.round((left + width) / prefs.scale) * 60000 + self.starttime );
				//timer.modD   = tma.toPvrDate( Math.round((left + width) / prefs.scale) * 60000 + self.starttime );
			}
		} else if ($source.is('[slot]')) {
			var recording = tma.status.recording[ parseInt($source.attr('slot')) ];
			recording.modEndtime   = tma.toPvrDate( Math.round((left + width) / prefs.scale) * 60000 + self.starttime );
		}

	};

	/**
	 * add timer bar event handlers
	 */
	this.addTimerBarHandler = function() {

		$contentTmpl

		.on('click', '.epg-mark', function() {

			if ($(this).is('[timerid]')) {
				// edit reservation
				var timer = tma.timers.find( $(this).attr('timerid') );
				if (timer) {

					// hope, that any module handles the 'editTimer' event
					tma.call( 'editTimer', [ timer ] ).done(function(popup) {
						self.removeShortcuts();
						$(popup).on('close', function() {
							self.addShortcuts();
						});
					});

				}
			}

		})

		.on('click', '.epg-mark-fn', function(e) {

			// set to active marker (display in front)
			if ( !$(e.target).is('BUTTON.ic-clock')) {
				// all elements, except BUTTON.ic-clock, should prevent opening
				// the timer edit window via $marker.click event
				e.stopPropagation();
			}

			$('.epg-mark-active').removeClass('epg-mark-active');
			$(this).parents('DIV.epg-mark').addClass('epg-mark-active');

		})

		.on('click', 'BUTTON.epg-mark-fn-jumptofirst', function(e) {

			// jump to the first occurance of this reservation
			e.stopPropagation();

			var day = parseInt($(this).attr('day'));

			$contentTmpl.scrollLeft($contentTmpl.scrollLeft() - day * prefs.scale * 24*60 );

		})

		.on('click', 'BUTTON.ic-save', function(e) {

			// edit modified timer or recording
			e.stopPropagation();

			var $marker = $(this).parents('DIV.epg-mark');
			if ($marker.is('[timerid]')) {

				/** @returns {TmaTimer} */
				var timer = tma.timers.find( $marker.attr('timerid') );
				if (timer) {

					tma.apiTimerUpdate( timer ).fail(function(xhr, errorType) {

						new jkqAlert({
							title:	tma.i18.error.dialog_title,
							text:	tma.i18.error.timer.create_modify
						}).confirm();

					});
				}

			} else if ($marker.is('[slot]')) {

				// current recording
				var recording = tma.status.recording[ parseInt($marker.attr('slot')) ];
				var start = new Date(recording.starttime);
				var end = new Date(recording.modEndtime);
				var newDuration = parseInt( (end.getTime() - start.getTime()) / 60000 );
				tma.apiFilesDuration( recording.slot , newDuration);

			}

		})

		.on('click', 'BUTTON.ic-stop', function(e) {

			// stop current recording
			e.stopPropagation();

			var $marker = $(this).parents('DIV.epg-mark');
			if ($marker.is('[slot]')) {
				new jkqDialog({
					title: self.i18.stoprec_dialog_title,
					text: self.i18.stoprec_dialog_text,
					autoSkipText: tma.i18.respond_auto_skip,
					autoSkipButton: 'epgpageStopRecording',
					buttons: {
						epgpageStopRecording:	tma.i18.respond_yes,
						jkqDialogCancel:		tma.i18.respond_cancel
					}
				}).confirm(function(button) {

					if (button == 'epgpageStopRecording') {

						tma.stopRecording( parseInt($marker.attr('slot')) );

					}
				});
			}

		})

		.on('click', 'BUTTON.ic-reset', function(e) {

			// reset modified reservation or recording
			e.stopPropagation();

			var $marker = $(this).parents('DIV.epg-mark');
			if ($marker.is('[timerid]')) {
				var timer = tma.timers.find( $marker.attr('timerid') );
				if (timer) {
					timer.modReset();
				}
			} else if ($marker.is('[slot]')) {
				var recording = tma.status.recording[ parseInt($marker.attr('slot')) ];
				recording.modReset();
			}

		})

		.on('click', 'BUTTON.ic-delete', function(e) {

			e.stopPropagation();

			var $marker = $(this).parents('DIV.epg-mark');
			if ($marker.is('[timerid]')) {
				// delete timer
				/** @returns {TmaTimer} */
				var timer = tma.timers.find( $marker.attr('timerid') );
				if (timer) {

					new jkqDialog({
						title: tma.i18.timer.delete_confirm_caption,
						text:  tma.i18.timer.delete_confirm_text.format(timer.getFilename(true)),
						autoSkipText: tma.i18.respond_auto_skip,
						autoSkipButton: 'tmaDeleteTimer',
						buttons: {
							tmaDeleteTimer:	 tma.i18.respond_yes,
							jkqDialogCancel: tma.i18.respond_cancel
						}
					}).confirm(function(button) {

						if (button == 'tmaDeleteTimer') {

							tma.apiDeleteTimer(timer).fail(function(xhr, errorType, error) {

								new jkqAlert({
									title:	tma.i18.error.dialog_title,
									text:	tma.i18.error.timer.del + '<br><br>' + error
								}).confirm();

							});

						}

					});

				}
			}

		})

		// drag marker start
		.on('mousedown touchstart', 'BUTTON.epg-mark-fn-resize-left', function(e) {

			e.stopPropagation();

			self.timerResizeL.start(e, $(this).parents('.epg-mark'));
			self.timerResizeL.onDragMove = self.onTimerModified;

		})

		// drag marker end
		.on('mousedown touchstart', 'BUTTON.epg-mark-fn-resize-right', function(e) {

			e.stopPropagation();

			self.timerResizeR.start(e, $(this).parents('.epg-mark'));
			self.timerResizeR.onDragMove = self.onTimerModified;

		})

		// drag marker
		.on('mousedown touchstart', '.epg-mark[is-recurring="false"] .epg-mark-fn', function(e) {

			if ( !$(e.target).is('BUTTON') ) {

				e.stopPropagation();

				self.timerMove.start(e, $(this).parents('.epg-mark'), 500);
				self.timerMove.onDragMove = self.onTimerModified;

			}

		});

	};

	/**
	 * create hours array for timeline
	 */
	this.createTimeline = function () {

		var tl_start = new Date( parseInt(this.starttime / 3600000) * 3600000 ).getTime();
		var tl_offset = tl_start;

		var hours = [];

		var tl_hours = parseInt( (this.endtime - tl_start + 3599999) / 3600000);
		for ( var i = 0; i < tl_hours; i++, tl_start += 3600000 ) {
			var hour = JK.model({
				starttime	: tl_start,
				date		: tma.date(this.i18.shortdate_fmt, tl_start),
				hour		: tma.date('G' , tl_start ),
				left		: (tl_start - self.starttime) / ( 24 * 60 * prefs.nrOfDays * 600),	// in percent
				width		: 100 / ( 24 * prefs.nrOfDays),										// in percent
			});
			hours.push( hour );
		}

		return hours;

	};

	/**
	 * update EPG data of visible channels in visible time range
	 */
	this.updateEpg = function() {

		// update current day
		if (this.nav) {
			this.nav.scrollPosTime = this.starttime + Math.round(state.scrollLeft * 60000 / prefs.scale);
		}

		// check visible channels
		var fromChannelNr	= parseInt( state.scrollTop / state.channelHeight );
		var toChannelNr		= parseInt( (state.scrollTop + state.height) / state.channelHeight + 0.99);

		// check toChannelNr max
		toChannelNr	= Math.min(toChannelNr, tma.channels.favChannels.length - 1);

		// start timestamp in minutes
		var start = parseInt(this.starttime / 60000);

		var timeLeft	= this.starttime + parseInt( state.scrollLeft				 * 60000 / prefs.scale);
		var timeRight	= this.starttime + parseInt((state.scrollLeft + state.width) * 60000 / prefs.scale);

		for( var chNr = fromChannelNr; chNr <= toChannelNr; chNr++ ) {

			/** @returns {TmaChannel} */
			var channel = tma.channels.favChannels[chNr];
			channel.getEvents( timeLeft, timeRight );

		}

	};

	/**
	 * renders the navigation bar
	 */
	this.showSticky = function() {

		this.nav = JK.model({
			currentDay		: 0,
			scrollPosTime	: this.starttime,
			days			: [],
		});
		this.nav.on('change', 'currentDay', function(e, val, nav, prop, oldVal) {
			nav.days[oldVal].isCurrent = false;
			nav.days[val].isCurrent = true;
		});
		this.nav.on('change', 'scrollPosTime', function(e, val) {
			// update current day
			var currDayDate = tma.date('Ymd', val );
			for( var day = 0; day < this.days.length; day ++) {
				if (currDayDate == this.days[day].date) {
					this.currentDay = day;
					break;
				}
			}
		});

		var date = this.starttime;
		var day = 0;
		while( date < this.endtime) {

			this.nav.days.push( JK.model({
				day			: day,
				isCurrent	: (day === 0),
				date		: tma.date('Ymd', date),
				date1		: tma.date(this.i18.shortestdate_fmt, date),
				date2		: tma.date(this.i18.shorterdate_fmt, date),
				date3		: tma.date(this.i18.shortdate_fmt, date)
			}) );

			day++;
			date += 24*60*60000;
		}

		// render template
		$stickyTmpl = JK.template(
				this.templates.navbar,
				{
					i18		: this.i18,
					prefs	: prefs,
					nav		: this.nav,
				}
		).appendTo( '.sticky-content' )

		.on('click', 'BUTTON.epg-nav-day', function() {

			var clickedDay = parseInt( $(this).attr('day') );

			var diff = (clickedDay - self.nav.currentDay) * prefs.scale * 24*60;
			if ($contentTmpl) $contentTmpl.scrollLeft(state.scrollLeft + diff);

		}).on('click', 'BUTTON.epg-nav-day-prev', function() {

			var diff = prefs.scale * 24*60;
			if ($contentTmpl) $contentTmpl.scrollLeft(state.scrollLeft - diff);

		}).on('click', 'BUTTON.epg-nav-day-next', function() {

			var diff = prefs.scale * 24*60;
			if ($contentTmpl) $contentTmpl.scrollLeft(state.scrollLeft + diff);

		}).on('click', 'BUTTON.epg-nav-primetime', function() {

			//var timeLeft = self.starttime + parseInt( state.scrollLeft * 60000 / prefs.scale);
			var dateStrLeft = tma.date('Y/m/d H:i:s', self.nav.scrollPosTime);
			var dateStrPrime = tma.date('Y/m/d '	, self.nav.scrollPosTime) + prefs.primetime + ':00';
			var dateLeft = new Date(dateStrLeft);
			var datePrime = new Date(dateStrPrime);

			var diff = parseInt(prefs.scale * (datePrime.getTime() - dateLeft.getTime()) / 60000);
			if ($contentTmpl) $contentTmpl.scrollLeft(state.scrollLeft + diff);

		}).on('click', 'BUTTON.ic-left', function() {

			if ($contentTmpl) $contentTmpl.scrollLeft(state.scrollLeft - state.width);

		}).on('click', 'BUTTON.ic-right', function() {

			if ($contentTmpl) $contentTmpl.scrollLeft(state.scrollLeft + state.width);

		}).on('click', 'BUTTON.ic-clock', function() {

			// toggle timermode
			$('.epg-container').toggleClass('epg-mode-edit-timer');

		}).on('click', 'BUTTON.tma-test', function() {
			// run function to test
			self.asyncCalculateRecSlots();
		});

		$(window).on('resize.epgpage', function() {

			var h = $('.epg-nav > BUTTON').outerHeight(true) + 10;
			$('.epg-nav-days').attr('width-mode', 3);
			var widthMode = 3;
			while ((widthMode > 1) && ($('.epg-nav-days').height() > h)) {
				$('.epg-nav-days').attr('width-mode', --widthMode);
			}

		}).resize();

	};

	this.blacklistFilter = function( channel ) {

		// return true, if logoid is not of analogue type
		return ( channel.logoid.substring(0,14) != "ffffffffffffff" );

	};

	/**
	 * load and display an EPG grid
	 *
	 * currently the EPG data is loaded for all channels of the selected favourites group (=> channels page)
	 */
	this.showEPG = function () {

		$.extend(state, EMPTY_STATE);

		this.starttime = new Date(tma.status.timestamp);
		this.starttime = Math.round(this.starttime.getTime() / 60000) * 60000;
		var endtime = this.starttime + prefs.nrOfDays * 24*60*60000;
		var enddate = new Date( tma.date('Y/m/d', endtime) );
		this.endtime = enddate.getTime();

		if (!TmaReservation.prototype.gridLeft) {
			TmaReservation.mixin({
				gridLeft: function() {
					"watch: {{starttime}}";
					return function() {
						return ( (new Date(this.starttime)).getTime() - self.starttime) / ( 24 * 60 * prefs.nrOfDays * 600);
					};
				},
				gridWidth: function() {
					"watch: {{starttime}},{{endtime}}";
					return function() {
						return ( this.timer.modDuration * 100) / ( 24 * 60 * prefs.nrOfDays);
					};
				},
			});
		}

		if (!TmaConflict.prototype.gridLeft) {
			TmaConflict.mixin({
				gridLeft: function() {
					"watch: {{starttime}}";
					return function() {
						return ( (new Date(this.starttime)).getTime() - self.starttime) / ( 24 * 60 * prefs.nrOfDays * 600);
					};
				},
				gridWidth: function() {
					"watch: {{starttime}},{{endtime}}";
					return function() {
						return ( (new Date(this.endtime)).getTime() - (new Date(this.starttime)).getTime()) / ( 24 * 60 * prefs.nrOfDays * 600);
					};
				},
			});
		}

		if (!TmaStatusRecording.prototype.gridWidth) {
			TmaStatusRecording.mixin({
				gridWidth: function() {
					"watch: {{modEndtime}}";
					return function() {
						return ( (new Date(this.modEndtime)).getTime() - self.starttime) / ( 24 * 60 * prefs.nrOfDays * 600);
					};
				},
			});
		}

		if (!TmaEpgEvent.prototype.gridLeft) {
			TmaEpgEvent.mixin({
				gridGetLeft: function() {
					return ( (new Date(this.starttime_local)).getTime() - self.starttime) / ( 24 * 60 * prefs.nrOfDays * 600);
				},
				gridLeft: function() {
					"watch: {{starttime_local}}";
					return function() {
						var l = this.gridGetLeft();
						if (l < 0) l = 0;
						return l;
					};
				},
				gridWidth: function() {
					"watch: {{duration}}";
					return function() {
						var l = this.gridGetLeft();
						var w = ( this.duration * 100) / ( 24 * 60 * prefs.nrOfDays);
						if (l < 0) w += l;
						return w;
					};
				},
			});
		}

		tma.channels.getFavChannels(
				this.channels,
				tma.channels.fav_name,
				this.blacklistFilter
		);

		// render template
		$contentTmpl = JK.template(
				this.templates.content,
				{
					i18:			this.i18,
					prefs:			prefs,
					hours:			this.createTimeline(),
					channels:		this.channels,
					timers:			tma.timers,
					recordings:		tma.status.recording
				}
		).appendTo( '#content' );

		if (!favHandler) {
			favHandler = tma.channels.on('change', 'fav_name', function() {
				// deferred update
				setTimeout(function() {
					self.updateEpg();
				}, 100);
			});
		}

		// update timers
		tma.timers.load();

		// iOS hack
		$contentTmpl.on('dragstart drop', /*'DIV',*/ function(e) {
			e.preventDefault();
		});

		// adjust vertical size (vScroll in EPG-frame)
		$(window).on('resize.epgpage', function() {

			var offs = $contentTmpl.offset();
			var maxH = /*$(document).scrollTop() +*/ $(window).height() - offs.top - 15;
			$contentTmpl.css('max-height', maxH);

			state.channelHeight = $('.epg-channel:first').outerHeight();
			state.scrollTop = $contentTmpl.scrollTop();
			state.scrollLeft = $contentTmpl.scrollLeft();

			state.width = $contentTmpl.width() - $('DIV.epg-head').outerWidth();
			state.height = $contentTmpl.height() - $('DIV.epg-timeline-frame').outerHeight();

			self.updateEpg();

		}).resize();

		// keep channel on left border
		// and timeline on top (scroll in EPG-container)
		$contentTmpl.scroll(function() {

			state.channelHeight = $('.epg-channel:first').outerHeight();
			state.scrollTop = $contentTmpl.scrollTop();
			$contentTmpl.find('DIV.epg-timeline').parent().css('top', state.scrollTop);

			state.scrollLeft = $contentTmpl.scrollLeft();
			$contentTmpl.find('DIV.epg-head').css('left', state.scrollLeft);

			self.updateEpg();

		}).scroll();

		// show event details
		$contentTmpl.on('click', 'DIV.epg-channel-prg', function(e) {

			var $event = $(this);

			// check, if any module handles the 'showEvent' event
			// and prevent reopening of the same info
			if (
					(tma.fn.showEvent) &&
					!$event.hasClass('jk-mark')
			) {

				var channel = tma.channels.channel( $(this).parents('.epg-channel').attr('channelid') );
				if ( channel ) {

					var eventid = parseInt( $(this).attr('eventid') );
					if ( eventid ) {

						var event = channel.epgEvents.find( function(event) {
							return event.eventid == eventid;
						});

						if ( event ) {

							tma.fn.showEvent( event ).done(function(popup) {
								self.removeShortcuts();
								popup.$relatedPrg = $event;
								$(popup).on('close', function() {
									this.$relatedPrg.removeClass('jk-mark');
									self.addShortcuts();
								});
							});

							$event.addClass('jk-mark');

						}
					}
				}
			}

		})

		.on('click', '.epg-head-channel[channelid]', function() {
			self.selectCurrentChannel( $(this).attr('channelid') );
		});

		this.addTimerBarHandler();

		// move content with mouse
		if ( !Modernizr.touch ) {
			var $ch_container = $contentTmpl.find('DIV.epg-channels');
			var dragndrop = new jkqDragDrop();
			dragndrop.onDragStart = function(e, drag_source) {
				this.drag_x_rel = - $contentTmpl.scrollLeft()	- e.pageX;
				this.drag_y_rel = - $contentTmpl.scrollTop()	- e.pageY;
			};
			dragndrop.dragmove = function(x, y) {
				$contentTmpl.scrollLeft(-x - this.drag_x_rel);
				$contentTmpl.scrollTop( -y - this.drag_y_rel);
			};

			// start drag
			$contentTmpl.on('mousedown', '.epg-channels', function(emd) {
				// move
				$ch_container.on('mousemove', function(emm) {
					if (	Math.sqrt(
							Math.pow(emm.pageX - emd.pageX, 2) +
							Math.pow(emm.pageY - emd.pageY, 2)
						) > 5
					) {
						$ch_container.off('mousemove');
						dragndrop.start(emd, $ch_container);
					}
				});
			}).on('mouseup', '.epg-channels', function() {
				$ch_container.off('mousemove');
			});
		}

		// show conflict list
		$('BUTTON.epg-show-conflicts').click( function() {

			self.showConflicts();

		});

		if (!conflictAddHandler) {
			conflictAddHandler = tma.timers.conflicts.on('add', '*', function() {

				self.showConflicts();

			});
		}

		// handle click on timeline timer bar
		$('.epg-timeline').on('click', '.epg-mark', function() {

			var channelid = $(this).attr('channelid');
			var $head = $('.epg-head-channel[channelid="' + channelid + '"]');
			var pos = $head.position();
			if (pos) {
				if ($contentTmpl) {
					$contentTmpl.animate({
						scrollTop: pos.top - parseInt(state.height / 3)
					}, 1000);
				}
			}

		});

	};

	/**
	 * selects a channel
	 *
	 * @param {String} channelid
	 */
	this.selectCurrentChannel = function (channelid) {

		var channel = tma.channels.channel( channelid );
		if ( channel ) {
			tma.apiChannelSet( channel.chtype, channel.chnum ).fail(function(xhr, errorType, error) {

				if (errorType == 'tma-error') {
					new jkqAlert({
						title:	tma.i18.error.dialog_title,
						text:	tma.i18.error.channels.set
					}).confirm();
				}

			});
		}

	};

	var popup = new jkqWindow();

	/**
	 * show conflict list in a window
	 */
	this.showConflicts = function() {

		if (!TmaConflict.prototype.epgStartdateFmt) {
			TmaConflict.mixin({
				epgStartdateFmt: function() {
					"watch: {{starttime}}";
					return tma.date( self.i18.timerdate_fmt, this.starttime );
				},
			});
		}

		popup.show();
		popup.setTitle( this.i18.conflicts_caption );

		popup.container.find('DIV.jkqWindowContent')
		.append( JK.template(
				this.templates.conflicts,
				{
					conflicts: tma.timers.conflicts,
				}
		) )

		.on('click', '.epg-conflict', function(e) {
			e.stopPropagation();

			var start = parseInt($(this).attr('start'));
			var channelid = $(this).attr('channelid');
			var $head = $('.epg-head-channel[channelid="' + channelid + '"]');
			var pos = $head.position();
			if (pos) {

				pos.left = Math.round( (start - self.starttime) * prefs.scale / 60000 );
				if ($contentTmpl) {
					$contentTmpl.animate({
						scrollTop: pos.top - parseInt(state.height / 3),
						scrollLeft: pos.left - parseInt(state.width / 3)
					}, 1000);

				}
			}

		});

		popup.adjustPos({
			hAlign: 'left',
			effect: 'slide-ltr'
		});

	};

	this.insertConfig = function () {

		$configTmpl = JK.template(
				this.templates.config,
				{
					i18:			this.i18,
					i18g:			tma.i18,	// global language def.
					prefs:			prefs,
					gPrefs:			tma.prefs,
					favgroups:		tma.favourites.groups,
				}
		).appendTo('DIV#config');

		/* favourite group */
		var $fav = $('DIV#config SELECT[name="favourites"]');
		$fav.val( tma.prefs.favGroup );
		$fav.change(function() {

			tma.channels.getFavChannels(
					self.channels,
					$(this).val(),
					self.blacklistFilter
			);
			self.updateEpg();

		});

		/**
		 * scale
		 */

		$('DIV#config DIV.cfg-scale *.less').click(function() {

			if (prefs.scale > 1) {
				prefs.setScale(prefs.scale - 1);
				self.updateEpg();
			}

		});
		$('DIV#config DIV.cfg-scale *.more').click(function() {

			prefs.setScale(prefs.scale + 1);

		});

		$('DIV#config INPUT[name="logotext"]')
		.click(function() {

			var val = $('DIV#config INPUT[name="logotext"]:checked').val();
			prefs.setDisplayMode( val );
			self.updateEpg();

		})
		.filter('[value="' +  prefs.displayMode + '"]' ).prop('checked', true);

		/**
		 * Primetime
		 */
		$('DIV#config INPUT[name="primetime"]').blur(function() {
			var val = $(this).val();
			if (val.match(/^\d{1,2}:\d{2}$/)) {
				prefs.setPrimetime( val );
			} else {
				$(this).val( prefs.primetime );
			}
		});

		tma.fn.getModule('genrecolours').createConfig(
				prefs.genreColoring,
				function(isEnabled) { // onEnableChange
					prefs.setGenreColoring(isEnabled);
				}
		).appendTo('DIV#config');

	};

	// init reservation resize

	// drag left end, to expand reservation
	this.timerResizeL = new jkqResizeLeft();

	// drag right end, to expand reservation
	this.timerResizeR = new jkqResizeRight();

	// move reservation
	this.timerMove = new jkqDragDrop();

	/* Settings */

	var prefs = JK.model({

			/**
			 * get the scale setting
			 *
			 * @return {Number}
			 */
			scale: parseInt(self.localSetting( 'scale', 7 )),

			/**
			 * set the scale setting
			 *
			 * @param {Number}	set		new scale setting
			 */
			setScale: function(set) {
				self.setLocalSetting( 'scale', parseInt(set) );
				this.scale = parseInt(set);
			},

			/**
			 * return the event-map width
			 */
			width: function() {
				"watch: {{scale}}, {{nrOfDays}}";
				return this.nrOfDays * 24*60 * this.scale + 80;
			},

			/**
			 * get the displayMode setting
			 *
			 * @return {Number}		1: small
			 * 						2: middle
			 * 						3: large logo & name
			 */
			displayMode: parseInt(self.localSetting( 'channel_display', 3 )),

			/**
			 * set the displayMode setting
			 *
			 * @param {Number}	set		new displayMode setting
			 * 							1: small
			 * 							2: middle
			 * 							3: display logo & name
			 */
			setDisplayMode: function(set) {
				if ((parseInt(set) >= 1) && (parseInt(set) <= 3)) {
					self.setLocalSetting( 'channel_display', parseInt(set) );
					this.displayMode = parseInt(set);
				}
			},

			/**
			 * get the Primetime setting
			 *
			 * @return {String}
			 */
			primetime: self.localSetting( 'primetime', '20:15' ),

			/**
			 * set the Primetime setting
			 *
			 * @param {String}	set		new Primetime
			 */
			setPrimetime: function(set) {
				if (set.match(/^\d{1,2}:\d{2}$/)) {
					self.setLocalSetting( 'primetime', set );
					this.primetime = set;
				}
			},

			/**
			 * get the number of visible days setting
			 *
			 * @return {Number}
			 */
			nrOfDays: parseInt( self.localSetting( 'nrofdays', 7 ) ),

			/**
			 * get the genre-coloring setting (*on/off)
			 *
			 * @return {Boolean}
			 */
			genreColoring: parseInt( self.localSetting( 'genre-coloring', 1 ) ) == 1,

			/**
			 * set the genre-coloring setting (on/off)
			 *
			 * @param {Boolean}	set		new genre-coloring setting
			 */
			setGenreColoring: function(set) {
				self.setLocalSetting( 'genre-coloring', (set) ? 1 : 0 );
				this.genreColoring = set;
			},

	});

}
