/**
 * epgsearch module
 */

/* TODO List
	DONE - Bug with block width when (event in progress) + (timer) + (recoding in progress). - This seems to be a problem when a recording-in-progress is truncated.
	DONE - Timers and 'in progress' on different lines.
	* Timers on other networks.
	* "Check my timers" feature.
	* Make genre colouring work.
	* Show new timer when creating timers from the results list.
	DONE - Delete saved searches.
	DONE - Warn if name already exists when saving a new search.
	DONE - In the load search list, change the button to a check-box "Show SmartEPG Searches".  Toggle SE searches based on this check-box state.
	DONE - Sort the saved search list.
*/


tma.registerModule( epgsearch );
epgsearch.prototype = new tmaModule();
function epgsearch() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'epgsearch';

	this.pageGroup = 'epg';

	var self = this;

	this.delayQuery = null;

	var queryStartTime = new Date(0);

	//var epgsource = "pvr";

	var PVRdirectory = "/mnt/hd/ProgramFiles/Settings/WebControl/public_html" + this.path();

	var indexFileName = PVRdirectory + "searches/index.json";

	var smartepgSearch = "/mnt/hd/ProgramFiles/Settings/SmartEPG_TMS/Search.txt";

	this.dependencies = {

			"css": [
			        "epgsearch.css",
			        "../../css/timepicker.css"
			        ],

			"js": [
					"../multichselect/multichselect.js",
					"../../js/jquery-ui-timepicker-addon-jk.js"
			       ],

	        "templates": [
	         		"criteria.html",
	           		"searches.html",
	               ]

	};

	var $contentTmpl;

	// define search criteria object
	var criteria = new TmaSearchCriteria();

	// define search result object
	var SearchResult = function() {

		// call super class constructor
		SearchResult._super.constructor.apply(this);

		this.error		= 0;
		this.searching	= undefined;
		this.progress	= 0;
		this.events		= [];
		this.count		= 0;

		this.init();

		// setup sort function
		this.events.sortBy('starttime_local');

		this.on('change', 'events.length', function(e, val) {
			this.count = val;
		});

		// remove handlers
		this.events.on('del', '*', function(e, nil, parent, ix, delEvent) {
			tma.timers.reservations.off( delEvent.wizOnReservationAdd );
			tma.timers.reservations.off( delEvent.wizOnReservationDel );
			tma.status.recordings.off( delEvent.wizOnRecordingAdd );
			tma.status.recordings.off( delEvent.wizOnRecordingDel );
		});

	};
	JK.inherit(SearchResult, JKobject);
	SearchResult.prototype.E_BUSY	= 1;
	SearchResult.prototype.E_FAILED	= 2;
	SearchResult.prototype.E_FATAL	= 4;

	SearchResult.prototype.reset = function() {
		this.error			= 0;
		this.searching		= false;
		this.progress		= 0;
		this.events.length	= 0;
	};

	/**
	 * table header formatting
	 * @returns {Function}
	 */
	SearchResult.mixin({
		headerFmt: function() {
			"watch: {{count}}";
			return function() {
				if (this.count === 0) {
					return self.i18.no_matches;
				} else if (this.count == 1) {
					return "" + this.count + " " + self.i18.results_found_single;
				} else {
					return "" + this.count + " " + self.i18.results_found_plural;
				}
			};
		},
	});

	var searchResult = new SearchResult();


	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Build the criteria window
	 */
	this.show = function() {

		this.init().done(function() {

			$('#timestamp').hide();

			//Load the timers
			tma.apiGetTimers();
			//Load the genres
			tma.apiEpgGenre();

			self.showContent();

		});

	};

	/**
	 * this page gets removed, so remove config class
	 */
	this.cleanup = function() {

		if (this.delayQuery) {
			window.clearTimeout(this.delayQuery);
			this.delayQuery = null;
		}

		// remove templates
		JK.removeTemplate( $contentTmpl );

		$('#timestamp').show();

	};

	/**
	 * prepare formatters (only once)
	 */
	this.setupFormatters = function() {

		if (!TmaEpgEvent.prototype.epgwizDateFmt) {
			TmaEpgEvent.mixin({

				epgwizDateFmt: function() {
					"watch: {{starttime_local}}";
					return function() {
						return tma.date(self.i18.timerdate_fmt, this.starttime_local);
					};
				},

				epgwizLeft: function() {
					"watch: {{wizMinStart}},{{wizMaxEnd}}";
					return function() {
						var evtMinStart = new Date(this.wizMinStart).getTime();
						var evtRange = new Date(this.wizMaxEnd).getTime() - evtMinStart;
						// relative left in percent
						if (evtRange) {
							return ( (new Date(this.starttime_local)).getTime() - evtMinStart) *100 / evtRange;
						} else {
							return 0;
						}
					};
				},

				epgwizWidth: function() {
					"watch: {{wizMinStart}},{{wizMaxEnd}}";
					return function() {
						var evtRange = new Date(this.wizMaxEnd).getTime() - new Date(this.wizMinStart).getTime();
						var duration = (new Date(this.endtime_local)).getTime() - (new Date(this.starttime_local)).getTime();
						// relative width in percent
						if (evtRange) {
							return duration * 100 / evtRange;
						} else {
							return 0;
						}
					};
				},

				epgwizCalcConstraints: function() {
					"watch: {{wizBars.length}}";
					var min = this.starttime_local;
					var max = this.endtime_local;
					for(var i = 0, l = this.wizBars.length; i < l; i++) {
						if (this.wizBars[i].relObject.starttime < min) {
							min = this.wizBars[i].relObject.starttime;
						}
						if (this.wizBars[i].relObject.endtime > max) {
							max = this.wizBars[i].relObject.endtime;
						}
					}
					this.wizMinStart = min;
					this.wizMaxEnd = max;
				},

			});
		}

		if ( !TmaSearchCriteria.prototype.epgwizStartdateFmt ) {

			TmaSearchCriteria.mixin({

				/**
				 * favourites group name formatter
				 */
				epgwizFavGroupTitle: function() {
					"watch: {{favgroup}}";
					return function() {
						switch (this.favgroup) {
							case ''			: return '';
							case '__tv'		: return "(" + tma.i18.favgroups.tv + ")";
							case '__radio'	: return "(" + tma.i18.favgroups.radio + ")";
							default			: return "(" + this.favgroup + ")";
						}
					};
				},

				/**
				 * start date formatter
				 */
				epgwizStartdateFmt: function() {
					"watch: {{startdate}}";
					return function() {
						if (this.startdate) {
							return tma.date(self.i18.timerdate_fmt, this.startdate);
						}
						return self.i18.not_limited;
					};
				},

				/**
				 * end date formatter
				 */
				epgwizEnddateFmt: function() {
					"watch: {{enddate}}";
					return function() {
						if (this.enddate) {
							return tma.date(self.i18.timerdate_fmt, this.enddate);
						}
						return self.i18.not_limited;
					};
				},

				/**
				 * start time formatter
				 */
				epgwizStarttimeFmt: function() {
					"watch: {{starttime}}";
					return function() {
						if (this.starttime) {
							var tempTime = new Date('2000/01/01 ' + this.starttime);
							return tma.date(self.i18.time_fmt, tempTime);
						}
						return self.i18.not_set;
					};
				},

				/**
				 * end time formatter
				 */
				epgwizEndtimeFmt: function() {
					"watch: {{endtime}}";
					return function() {
						if (this.endtime) {
							var tempTime = new Date('2000/01/01 ' + this.endtime);
							return tma.date(self.i18.time_fmt, tempTime);
						}
						return self.i18.not_set;
					};
				},

				/**
				 * minimum duration formatter
				 */
				epgwizMinlenFmt: function() {
					"watch: {{minlen}}";
					return function() {
						if (this.minlen) {
							var duration = tma.timeToMinutes( this.minlen );
							return tma.formatDuration( duration );
						}
						return self.i18.not_set;
					};
				},

				/**
				 * maximum duration formatter
				 */
				epgwizMaxlenFmt: function() {
					"watch: {{maxlen}}";
					return function() {
						if (this.maxlen) {
							var duration = tma.timeToMinutes( this.maxlen );
							return tma.formatDuration( duration );
						}
						return self.i18.not_set;
					};
				},

			} );

		}

	};

	/**
	 * Build the criteria window
	 */
	this.showContent = function() {

		// prepare formatters
		this.setupFormatters();

		/**
		 * render template
		 */

		$contentTmpl = JK.template(
			this.templates.criteria,
			{
				i18			: this.i18,
				i18g		: tma.i18,
				searchResult: searchResult,
				criteria	: criteria,
			}
		).appendTo('#content')

		/**
		 * add event handler
		 */

		// change search location option
		.on('click', '.epgwiz-search-location', function() {
			var prop = $(this).attr('search');
			$(this).parents('TR').ddtData(function() {
				this[prop] = !this[prop];
			});
		})

		// add text criteria
		.on('click', 'BUTTON.epgwiz-add-text', function() {
			criteria.addTextCriteria();
		})

		// delete text criteria
		.on('click', 'BUTTON.epgwiz-delete-text', function() {
			$(this).parents('TR').ddtData(function() {
				criteria.deleteTextCriteria( this );
			});
		})

		// select channels
		.on('click', '#epgwiz-channels', function() {
			//TODO - Remove analogue channel IDs.
			tma.fn.getModule('multichselect').showSelector( criteria );
		})

		// set weekday presets
		.on('click', '.epgwiz-preset-alldays', function() {
			criteria.setWeekdays( true, true, true, true, true, true, true);
		})
		.on('click', '.epgwiz-preset-weekdays', function() {
			criteria.setWeekdays( true, true, true, true, true, false, false);
		})
		.on('click', '.epgwiz-preset-weekends', function() {
			criteria.setWeekdays( false, false, false, false, false, true, true);
		})
		.on('click', '.epgwiz-preset-nodays', function() {
			criteria.setWeekdays( false, false, false, false, false, false, false);
		})

		// toggle weekday
		.on('click', '.epgwiz-weekday', function() {
			var day = $(this).attr('day');
			criteria.weekday[ day ] = !criteria.weekday[ day ];
		})

		/* reset icons */

		.on('click', '#epgwiz-reset-all', function() {
			criteria.reset();
			$('.epgwiz-results-table').hide();
		})
		.on('click', '#epgwiz-reset-channels', function() {
			criteria.favgroup = tma.localSetting( 'defaultfavourites', '' );
			criteria.channels.length = 0;
		})
		.on('click', '#epgwiz-reset-start-time', function() {
			criteria.starttime = '';
		})
		.on('click', '#epgwiz-reset-end-time', function() {
			criteria.endtime = '';
		})
		.on('click', '#epgwiz-reset-min-len', function() {
			criteria.minlen = '';
		})
		.on('click', '#epgwiz-reset-max-len', function() {
			criteria.maxlen = '';
		})
		.on('click', '#epgwiz-reset-start-date', function() {
			criteria.startdate = '';
		})
		.on('click', '#epgwiz-reset-end-date', function() {
			criteria.enddate = '';
		})

		//Setup text area help text
		.on('click', '#epgwiz-texthelp', function() {
			self.showModuleHelpText();
		})

		// load search definition
		.on('click', '#epgwiz-load-search', function() {
			self.loadSearchList();
		})

		// save search definition
		.on('click', '#epgwiz-save-search', function() {
			self.saveSearchDefinition();
		})

		// start search process
		.on('click', '#epgwiz-do-search', function() {
			$('.epgwiz-results-table').show();
			self.search();
		})

		// show event info popup
		.on('click', '.epgwiz-clm-name', function(e) {

			var $prg = $(this);
			var $event = $(this).parents( '.epgwiz-event' );

			// check, if any module handles the 'showEvent' event
			// and prevent reopening of the same info
			if (
					(tma.fn.showEvent) &&
					!$event.hasClass('jk-mark')
			) {

				// get the bound data of this list item
				$event.ddtData( function() {

					tma.fn.showEvent( this ).done(function(popup) {
						popup.$relatedPrg = $event;
						$(popup).on('close', function() {
							this.$relatedPrg.removeClass('jk-mark');
						});
					});
					$event.addClass('jk-mark');

				});

			}
		})

		.on('click', '.epgwiz-evt-reservation', function() {
			var timer = tma.timers.find( $(this).attr('timerid') );
			if (timer) {
				// hope, that any module handles the 'editTimer' event
				tma.call( 'editTimer', [ timer ] );
			}
		})

		;

		this.setupDateEvents( criteria );

	};

	this.setupDateEvents = function( criteria ) {

		// setup the datepickers
		var dtpOptions = {
				controlType		: 'slider',
				addSliderAccess	: true,
				sliderAccessArgs: { touchonly: false },
				firstDay		: 1,
				timeFormat		: "",
				dateFormat		: "yy/mm/dd",
				monthNames		: [],
				monthNamesShort	: [],
				dayNames		: [],
				dayNamesShort	: [],
				dayNamesMin		: [],
				timeText		: "",
				hourText		: this.i18.dtp_hour_text,
				minuteText		: this.i18.dtp_minute_text,
				closeText		: this.i18.dtp_close_text,
				currentText		: this.i18.dtp_current_text,
				timeOnlyTitle	: this.i18.dtp_time_only_title,
		};

		$.each(tma.i18.months, function(i, month) {
			dtpOptions.monthNames.push(month.name);
			dtpOptions.monthNamesShort.push(month.shortname);
		});

		$.each(tma.i18.days, function(i, day) {
			dtpOptions.dayNames.push(day.name);
			dtpOptions.dayNamesShort.push(day.shortname);
			dtpOptions.dayNamesMin.push(day.shortname.slice(0, 2));
		});

		//Setup the start date field
		$("#epgwiz-startdate").datetimepicker($.extend({}, dtpOptions, {
			onSelect: function (selectedDateTime) {
				if (selectedDateTime ) {
					criteria.startdate = selectedDateTime;
				}
			},
			beforeShowDay: function(date) {
				if (
						criteria.enddate &&
						(date > new Date(criteria.enddate))
				) {
					// disable days after enddate
					return [false, '', ''];
				}
				if (
						criteria.startdate &&
						criteria.enddate &&
						(date >= new Date(criteria.startdate)) &&
						(date <= new Date(criteria.enddate))
				) {
					// highlight selected range
					return [true, 'epgwiz-ui-state-highlight', ''];
				}
				return [true, '', ''];
			}
		}));

		// click on icon or text should trigger the dt-picker
		$("#epgwiz-startdate-text").click(function () {
			$("#epgwiz-startdate").datetimepicker('setDate', criteria.startdate || '').datetimepicker("show");
		});

		//Setup the end date field
		$("#epgwiz-enddate").datetimepicker($.extend({}, dtpOptions, {
			onSelect: function (selectedDateTime) {
				if ( selectedDateTime ) {
					criteria.enddate = selectedDateTime;
				}
			},
			beforeShowDay: function(date) {
				if (
						criteria.startdate &&
						(date < new Date(criteria.startdate))
				) {
					// disable days before startdate
					return [false, '', ''];
				}
				if (
						// highlight selected range
						criteria.startdate &&
						criteria.enddate &&
						(date >= new Date(criteria.startdate)) &&
						(date <= new Date(criteria.enddate))
				) {
					return [true, 'epgwiz-ui-state-highlight', ''];
				}
				return [true, '', ''];
			}
		}));

		// click on icon or text should trigger the dt-picker
		$("#epgwiz-enddate-text").click(function () {
			$("#epgwiz-enddate").datetimepicker('setDate', criteria.enddate || '').datetimepicker("show");
		});


		/**
		 * Reconfigure dtpOptions for time-only mode
		 */
		dtpOptions.timeFormat = "HH:mm";
		dtpOptions.timeText = this.i18.dtp_time_text;
		dtpOptions.dateFormat = "HH:mm";	//The date format needs to be here or a parsing error is generated
		dtpOptions.timeOnlyTitle = this.i18.dtp_time_text;

		//Setup the start time field
		$("#epgwiz-starttime").timepicker($.extend({}, dtpOptions, {
			onSelect: function (selectedDateTime) {
				criteria.starttime = selectedDateTime;
			}
		}));

		$("#epgwiz-starttime-text").click(function () {
			$("#epgwiz-starttime").timepicker('setTime', criteria.starttime || '00:00').timepicker("show");
		});


		//Setup the end time field
		$("#epgwiz-endtime").timepicker($.extend({}, dtpOptions, {
			onSelect: function (selectedDateTime) {
				criteria.endtime = selectedDateTime;
			}
		}));

		$("#epgwiz-endtime-text").click(function () {
			$("#epgwiz-endtime").timepicker('setTime', criteria.endtime || '00:00').timepicker("show");
		});


		/**
		 * Reconfigure dtpOptions for duration mode
		 */
		dtpOptions.timeOnlyTitle = this.i18.dtp_time_only_title;

		//Setup the minimum length field
		$("#epgwiz-minlen").timepicker($.extend({}, dtpOptions, {
			onSelect: function (selectedTime) {
				criteria.minlen = selectedTime;
			}
		}));

		$("#epgwiz-minlen-text").click(function () {
			$("#epgwiz-minlen").timepicker('setTime', criteria.minlen || '00:00').timepicker("show");
		});

		//Setup the maximum length field
		$("#epgwiz-maxlen").timepicker($.extend({}, dtpOptions, {
			onSelect: function (selectedTime) {
				criteria.maxlen = selectedTime;
			}
		}));

		$("#epgwiz-maxlen-text").click(function () {
			$("#epgwiz-maxlen").timepicker('setTime', criteria.maxlen || '00:00').timepicker("show");
		});


	};

	this.search = function() {

		var searchString = criteria.searchString();

		if ( typeof searchString == 'string' ) {

			var byteCount = searchString.length;

			searchResult.reset();
			searchResult.searching = true;
			this.initepgsearchTAP().done(function( response ) {

				// Refresh the timers list. This can be asynchronous because the search will take some time to run.
				tma.apiGetTimers();

				//Stop the timer from triggering again.
				if (self.delayQuery) {
					window.clearTimeout(self.delayQuery);
					self.delayQuery = null;
				}

				queryStartTime = new Date( response.timestamp );

				self.delayQuery = window.setTimeout(
					function() {

						if (response.error_code === 0) {
							self.executeQuery(searchString, response.port);
						} else {
							searchResult.error = searchResult.E_BUSY;
							new jkqAlert({
								title:	tma.i18.error.dialog_title,
								text:	self.i18.tap_busy
							}).confirm();
						}
					},
					750		//Wait 0.75 seconds before submitting the query to allow EPGapprentice time to start.
				);
			});

		} else {
			// searchString is a number => error in criteria validation

			/*  1 - No search fields selected
				2 - No search text provided
				3 - No weekdays specified
				4 - Inconsistent lengths
				5 - Inconsistent times
				6 - Inconsistent dates
				7 - Invalid search text */
			var errorFlag = searchString;

			new jkqAlert({
				title:	tma.i18.error.dialog_title,
				text:	self.i18.validation_errors[errorFlag]
			}).confirm();
		}

	};  //END search()

	this.MJDtoJSON = function(mjd){
		//From ETSI EN 300 468 V1.14.1 (2014-05)
		//To find Y, M, D from MJD

		//Y' = int [ (MJD - 15 078,2) / 365,25 ]
		var y1 = Math.floor((mjd - 15078.2) / 365.25);
		//M' = int { [ MJD - 14 956,1 - int (Y' × 365,25) ] / 30,6001 }
		var m1 = Math.floor((mjd - 14956.1 - (Math.floor(y1 * 365.25))) / 30.6001);
		//D = MJD - 14 956 - int (Y' × 365,25) - int (M' × 30,6001)
		var day = mjd - 14956 - (Math.floor(y1 * 365.25)) - (Math.floor(m1 * 30.6001));
		//If M' = 14 or M' = 15, then K = 1; else K = 0
		var k = 0;
		if((m1 == 14) || (m1 == 15)){k = 1;}
		//Y = Y' + K
		//if (y1 >= 100){y1 = y1 + 1900;}
		y1 = y1 + 1900;
		var year = y1 + k;
		//M = M' - 1 - K × 12
		var month = m1 - 1 - k * 12;
		//To find WD from MJD
		//WD = [ (MJD + 2) mod 7 ] + 1
		//var weekDay =  ((mjd + 2) % 7 ) + 1;

		var tDate = new Date(0);

		tDate.setFullYear(year);
		tDate.setMonth(month - 1);
		tDate.setDate(day);

		return tDate;

	};

	function EpgwizEventBar( event, relObject) {

		// call super class constructor
		EpgwizEventBar._super.constructor.apply(this);

		this.left = 0;
		this.width = 0;
		this.relObject = null;

		this.init();

		this.relObject = relObject;
		this.event = event;

		var self = this;
		var onWizMinStartChange = event.on('change', 'wizMinStart', function() {
			self.calcLeft();
			self.calcWidth();
		});
		var onWizMaxEndChange = event.on('change', 'wizMaxEnd', function() {
			self.calcLeft();
			self.calcWidth();
		});

		this.destruct = function() {
			this.event.off( onWizMinStartChange );
			this.event.off( onWizMaxEndChange );

			this.event = null;
		};

		this.calcLeft();
		this.calcWidth();

	}
	JK.inherit(EpgwizEventBar, JKobject);

	EpgwizEventBar.prototype.TYPE_RESERVATION	= 'res';
	EpgwizEventBar.prototype.TYPE_RECORDING		= 'rec';

	EpgwizEventBar.prototype.type = function() {
		if ( this.relObject instanceof TmaReservation ) {
			return this.TYPE_RESERVATION;
		} else if ( this.relObject instanceof TmaStatusRecording ) {
			return this.TYPE_RECORDING;
		}
	};

	/**
	 * are related object (e.g. reservation) and event on different channels?
	 * @returns {Boolean}
	 */
	EpgwizEventBar.prototype.other = function() {
		if ( this.relObject instanceof TmaReservation ) {
			return ( this.relObject.timer.logoid != this.event.logoid );
		} else if ( this.relObject instanceof TmaStatusRecording ) {
			return ( this.relObject.logoid != this.event.logoid );
		}
	};

	/**
	 * channel name of related object
	 * @returns {Boolean}
	 */
	EpgwizEventBar.prototype.chname = function() {
		if ( this.relObject instanceof TmaReservation ) {
			return this.relObject.timer.channel().servicename;
		} else if ( this.relObject instanceof TmaStatusRecording ) {
			return  this.relObject.channel().servicename;
		}
	};

	EpgwizEventBar.prototype.calcLeft = function() {
		var evtMinStart = new Date(this.event.wizMinStart).getTime();
		var evtRange = new Date(this.event.wizMaxEnd).getTime() - evtMinStart;
		// relative left in percent
		if (evtRange) {
			this.left = ( (new Date(this.relObject.starttime)).getTime() - evtMinStart) * 100 / evtRange;
		} else {
			this.left = 0;
		}
	};

	EpgwizEventBar.prototype.calcWidth = function() {
		var evtRange = new Date(this.event.wizMaxEnd).getTime() - new Date(this.event.wizMinStart).getTime();
		var duration = (new Date(this.relObject.endtime)).getTime() - (new Date(this.relObject.starttime)).getTime();
		// relative width in percent
		if (evtRange) {
			this.width = duration * 100 / evtRange;
		} else {
			this.width = 0;
		}
	};

	EpgwizEventBar.mixin({

		recalcLeft: function() {
			"watch: {{relObject.starttime}}";
			this.event.epgwizCalcConstraints();
			this.calcLeft();
		},

		recalcWidth: function() {
			"watch: {{relObject.starttime}}, {{relObject.endtime}}";
			this.event.epgwizCalcConstraints();
			this.calcWidth();
		},

	});

	/**
	 * evaluate the list of blockings, that touch this event
	 *
	 * @param {TmaEpgEvent}	event
	 */
	this.setupEventAffectingBlockings = function( event ) {
		var i = 0, len = 0;

		// create new property
		event.extendProp( 'wizBlockings', [] );

		var transponder = event.channel.transponder;

		// filter existing blocking
		for ( i = 0, len = transponder.blockings.length; i < len; i++ ) {
			var blocking = transponder.blockings[i];

			if ( event.isIntersecting( blocking ) ) {
				blocking.wizEvent = event;
				event.wizBlockings.push( blocking );
			}

		}

		// watch for added blockings and store handler-id in 'wizBlockingsOnAdd'
		event.wizBlockingsOnAdd = transponder.blockings.on('add', '*', function(e, blocking ) {
			if ( event.isIntersecting( blocking ) ) {
				blocking.wizEvent = event;
				event.wizBlockings.push( blocking );
			}
		});

		// watch for removed blockings and store handler-id in 'wizBlockingsOnDel'
		event.wizBlockingsOnDel = transponder.blockings.on('del', '*', function(e, nil, parent, index, delBlocking ) {
			var i = event.wizBlockings.indexOf( delBlocking );
			if ( i >= 0 ) {
				event.wizBlockings.splice(i, 1);
			}
		});

	};

	/**
	 * sort function for event.wizBars
	 * @param {EpgwizEventBar}	a
	 * @param {EpgwizEventBar}	b
	 */
	function eventBarSort( a, b ) {
		// recordings have the highest priority
		if ( a.type() == a.TYPE_RECORDING && b.type() != b.TYPE_RECORDING) {
			return -1;
		} else if ( a.type() != a.TYPE_RECORDING && b.type() == b.TYPE_RECORDING) {
			return 1;
		}
		// higher priority for bars on the same channel like event
		if ( !a.other() && b.other() ) {
			return -1;
		} else if ( a.other() && !b.other() ) {
			return 1;
		}
		// else sort on starttime
		if ( a.relObject.starttime < b.relObject.starttime ) {
			return -1;
		} else if ( a.relObject.starttime > b.relObject.starttime ) {
			return 1;
		}
		return 0;
	}

	/**
	 * setup search result event
	 *
	 * @param {TmaEpgEvent}	event
	 */
	this.setupEvent = function( event ) {

		// create new property
		event.extendProp( 'wizBars', [] );
		event.extendProp( 'wizMinStart'	, event.starttime_local );
		event.extendProp( 'wizMaxEnd'	, event.endtime_local );

		/**
		 * set sort function for wizBars
		 */
		event.wizBars.sort( eventBarSort );

	};

	/**
	 * add recording/reservation to events wizBars list, if its intersecting with event
	 *
	 * @param {TmaEpgEvent}	event
	 * @param {Object}		relObject	object with properties "starttime" and "endtime"
	 */
	function addRelatedObject( event, relObject ) {

		if ( event.isIntersecting( relObject ) ) {
			event.wizBars.add( new EpgwizEventBar( event, relObject ) );
		}

	}

	/**
	 * evaluate the list of reservations, that touch this event
	 *
	 * @param {TmaEpgEvent}	event
	 */
	this.setupEventAffectingReservations = function( event ) {
		var i = 0, len = 0;

		// filter existing reservations
		for ( i = 0, len = tma.timers.reservations.length; i < len; i++ ) {
			addRelatedObject( event, tma.timers.reservations[i] );
		}

		// watch for added reservations and store handler-id in 'wizOnReservationAdd'
		event.wizOnReservationAdd = tma.timers.reservations.on('add', '*', function(e, reservation ) {
			addRelatedObject( event, reservation );
		});

		// watch for removed reservations and store handler-id in 'wizOnReservationDel'
		event.wizOnReservationDel = tma.timers.reservations.on('del', '*', function(e, nil, parent, index, delReservation ) {
			var i = event.wizBars.findIndex( function( wizBar ) {
				return wizBar.relObject == delReservation;
			} );
			if ( i >= 0 ) {
				event.wizBars.splice(i, 1);
			}
		});

	};

	/**
	 * evaluate the list of recordings, that touch this event
	 *
	 * @param {TmaEpgEvent}	event
	 */
	this.setupEventAffectingRecordings = function( event ) {
		var i = 0, len = 0;

		// filter existing recordings
		for ( i = 0, len = tma.status.recordings.length; i < len; i++ ) {
			addRelatedObject( event, tma.status.recordings[i] );
		}

		// watch for added recordings and store handler-id in 'wizOnRecordingAdd'
		event.wizOnRecordingAdd = tma.status.recordings.on('add', '*', function(e, recording ) {
			addRelatedObject( event, recording );
		});

		// watch for removed recordings and store handler-id in 'wizOnRecordingDel'
		event.wizOnRecordingDel = tma.status.recordings.on('del', '*', function(e, nil, parent, index, delRecording ) {
			var i = event.wizBars.findIndex( function( wizBar ) {
				return wizBar.relObject == delRecording;
			} );
			if ( i >= 0 ) {
				event.wizBars.splice(i, 1);
			}
		});

	};

	this.initepgsearchTAP = function () {

		var params = {
				'function'	: 'epg',
				action		: 'search!',
				//tap			: this.TapId + '!',
				//epgsource	: epgsource,
				cache		: '1'		//If cache is set to '1', the TAP will not delete the EPG cache unless the timeout has elapsed
		};

		/**
		 * this construct allows us to switch to fail(),
		 * if the api returns an error_code <br>
		 * see: return deferred.promise().fail()
		 * http://stackoverflow.com/a/5112734
		 */
		var deferred = $.Deferred();

		$.getJSON('/api', params).done(function(data, status, xhr) {

			if (data.content === "epgsearch_ack") {
				deferred.resolve(data, status, xhr);
			} else {
				// switch to fail()
				deferred.reject(
						xhr,
						'tma-error',
						'epgsearch: ' + data.timestamp,
						'error code: ' + data.error_code
				);
			}

		}).fail(function(xhr, errorType, error) {
			deferred.reject(xhr, errorType, error);
		});

		return deferred.promise().fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in epgsearch.initepgsearchTAP()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: '',
				}
			});

		});

	};

	this.calcDuration = function(t1, t2) {

		return (t1.getTime() - t2.getTime()) / 60000;

	};//END calcDuration()

	this.executeQuery = function(queryText, port) {

		var url = 'ws://' + window.location.hostname + ':' + port + '/epgsearch';

		if ('WebSocket' in window) {

			var socket = new WebSocket(url);

			socket.onopen = function(msg) {
				if(socket.readyState != WebSocket.OPEN)
				{alert("Socket not ready to send");}
				socket.send(queryText);
			};

			socket.onmessage = function(msg) {

				var magic = msg.data.slice(0, 12);

				if(magic == "@@@@FATAL@@@") {
					searchResult.error = searchResult.E_FATAL;
					new jkqAlert({
						title:	tma.i18.error.dialog_title,
						text:	self.i18.fatal_error
					}).confirm();
				} else if (magic == "@@PROGRESS@@") {
					searchResult.progress = msg.data.slice(13);
				} else {
					// Filter out optional whitespaces and comma at the beginning
					var match = msg.data.match(/^[\s,]*({[\s\S]+})[\s]*$/);//, '');
					if (match) {
						// got an event
						try {
							var event = new TmaEpgEvent( JSON.parse(match[1]) );
							// ignore events in the past
							if ( event.startDate() >= queryStartTime ) {
								// Got a future event
								self.setupEvent( event );
								self.setupEventAffectingBlockings( event );
								self.setupEventAffectingReservations( event );
								self.setupEventAffectingRecordings( event );
								searchResult.events.add( event );
							}
						} catch (e) {
							if (console) {
								console.log('websocket.onmessage: adding received event FAILED!');
								console.log(e);
							}
						}
					}
				}

			};

			socket.onclose = function(msg) {

				//1000 = normal close, 1006 = socket closed without closing WS frame.
				if (msg.code != 1000 && msg.code != 1006) {
					alert("WebSocket closed with code " + msg.code.toString());
				}

				socket = null;
				searchResult.searching = false;
			};

			socket.onerror = function(msg) {
				//TODO - A lot of these error occur because TMAapprentice is not yet listening to the port.
				//TODO - Build a retry loop that tries every 500ms for 5 seconds before displaying a fatal error message.
				searchResult.error = searchResult.E_FAILAD;
				new jkqAlert({
					title:	tma.i18.error.dialog_title,
					text:	self.i18.search_failed
				}).confirm();
				tma.log("Error: " + msg.data);
				//console.log("Error: " + msg.code); //The code is not available for this event.
			};

		} else {
			alert(this.i18.no_websockets);
		}

	};//END executeQuery()

	this.loadSearchFields = function(searchDefinition) {

		criteria.reset();
		criteria.texts.length = 0;
		criteria.extend( searchDefinition );

		if ( criteria.texts.length === 0 ) {
			criteria.addTextCriteria();
		}

	};

	/**
	 * find a search definition in index file
	 *
	 * @param {String}		searchName	Name of the definition to search
	 * @param {Function}	callback	function( {Boolean} isExisting, {Object} indexObject )
	 */
	this.findSearchFileName = function(searchName, callback) {

		// Take the search name, lookup the index file and return the file name.
		// If the search name is not found, return a serial number to be used as the name.

		$.getJSON(indexFileName).done( function( indexObjects ){

			for (var i = 0, len = indexObjects.length; i < len; i++ ) {
				if (indexObjects[i].searchName == searchName) {
					callback( true, indexObjects[i]);
					return;
				}
			}
			// index object not found => create a new one
			callback(false, {
				fileName: tma.date('YmdHis'),
				searchName: searchName
			});

		}).fail(function(xhr, errorType, error) {

			if(xhr.status == 404) {
				// Index file not found => create a new index object
				callback(false, {
					fileName: tma.date('YmdHis'),
					searchName: searchName
				});
			} else {
				// something went wrong
				tma.log({
					title: errorType + " in epgsearch.loadSearchDefinition()",
					details: {
						error: error,
						xhrReadyState: xhr.readyState,
						xhrStatus: xhr.status,
					}
				});
				//callback(null);
			}
		});

	}; //END findSearchFileName

	this.updateSearchIndex = function(searchObject, deleteFlag) {

		$.getJSON(indexFileName).done( function(fileIndexObject){

			// index file exists

			if (deleteFlag) {
				// search object should be deleted
				$.each(fileIndexObject, function(i, indexObject){

					if (
							(searchObject.searchName == indexObject.searchName) &&
							(searchObject.fileName == indexObject.fileName)
					) {
						// object found
						fileIndexObject.splice(i, 1);
						return false; // = break;
					}
				});

			} else {
				// add search object to index
				fileIndexObject.push(searchObject);
			}

			// save index file
			tma.apiSaveFile( indexFileName, JSON.stringify(fileIndexObject, null, '\t') );

		}).fail(function(xhr, errorType, error) {

			if(xhr.status == 404) {
				// create a new index file with searchObject as first content
				tma.apiSaveFile( indexFileName, JSON.stringify( [ searchObject ], null, '\t') );
			} else {
				// something went wrong
				tma.log({
					title: errorType + " in epgsearch.updateSearchIndex()",
					details: {
						error: error,
						xhrReadyState: xhr.readyState,
						xhrStatus: xhr.status,
					}
				});
				return 0;
			}
		});

	}; //END updateSearchIndex


	this.loadSearchDefinition = function(searchName) {

		//Load the search definition from file and then pass it to the loadSearchFields() function.

		self.findSearchFileName(searchName, function(isExisting, loadFile){

			if ( isExisting ) {

				var searchFileName = PVRdirectory + "searches/" + loadFile.fileName + "_search.json";

				$.getJSON(searchFileName).done( function(searchDefinition){

					self.loadSearchFields(searchDefinition);

				}).fail( function(xhr, errorType, error) {

					if (xhr.status == 404) {

						new jkqAlert( {
							title:	tma.i18.error.dialog_title,
							text:	self.i18.saved_search_file + " " + searchName + " " + self.i18.not_found
						} ).confirm();

					} else {

						tma.log({
							title: errorType + " in epgsearch.loadSearchDefinition()",
							details: {
								error: error,
								xhrReadyState: xhr.readyState,
								xhrStatus: xhr.status,
							}
						});
					}
				});

			} else {

				new jkqAlert( {
					title:	tma.i18.error.dialog_title,
					text:	self.i18.saved_search_def + " " + self.i18.not_found
				} ).confirm();

			}

		});

	};//END loadSearchDefinition()

	/**
	 * delete a search definition
	 *
	 * @returns {jQueryDeferred}
	 */
	this.deleteSearchDefinition = function(fileObject) {

		var deferred = $.Deferred();

		var s_name = $(this).parents('LI').attr('value');
		var s_file = $(this).parents('LI').attr('filename');

		new jkqDialog({
			title: self.i18.delete_hdr,
			text: self.i18.delete_def + " '" + fileObject.searchName + "'",
			buttons: {
				ok:				 tma.i18.respond_yes,
				jkqDialogCancel: tma.i18.respond_cancel
			}
		}).confirm(function(button) {

			//Try to delete the search file first
			var params =
				"file="	+ fileObject.fileName + "_search.json\n" +
				"dir="	+ PVRdirectory + "searches\n";

			$.post('/api?function=files&action=delete!', params, null, 'json').done(function(data) {

				if ((data.mpg_status === 0) && (data.inf_status === 0) && (data.nav_status === 0)) {
					//Delete the index record here
					self.updateSearchIndex(fileObject, true);
					deferred.resolve();
					//criteria.reset();
				} else {
					new jkqAlert({
						title: tma.i18.error.dialog_title,
						text: "Error deleting definition file '" + fileObject.searchName + "' '" + fileObject.fileName + "'"
					}).confirm();
					deferred.reject();
				}

			}).fail( function() {
				deferred.reject();
			});

		}).cancel( function() {
			deferred.reject();
		} );

		return deferred.promise();

	};//END deleteSearchDefinition()

	this.saveSearchDefinition = function() {

		// query search definition name
		new jkqInputDialog({
			title: this.i18.save,
			text: this.i18.save_long,
			inputValue: criteria.name
		}).confirm(function() {

			var searchName = this.value();

			if ( !searchName ) {

				// name is missing
				new jkqAlert( {
					title:	tma.i18.error.dialog_title,
					text:	"Please enter a search name"
				} ).confirm();

			} else {

				self.findSearchFileName(searchName, function(isExisting, saveFile){

					var searchFileName = PVRdirectory + "searches/" + saveFile.fileName + "_search.json";

					if ( isExisting ) {

						new jkqDialog({
							title: self.i18.replace,
							text: self.i18.replace_existing + " '" + searchName + "'",
							buttons: {
								ok:				 tma.i18.respond_yes,
								jkqDialogCancel: tma.i18.respond_cancel
							}
						}).confirm(function(button) {

							criteria.save( searchFileName, saveFile.searchName );

						});

					} else {
						criteria.save( searchFileName, saveFile.searchName );
						self.updateSearchIndex(saveFile, false);
					}

				});

			}

		});

	};//END saveSearchDefinition()

	// help text cache
	var helpText = '';

	/**
	 * reads the module help text
	 */
	this.getModuleHelpText = function() {

		var deferred = $.Deferred();

		if (!helpText) {

			// i18.module.helptext is a filename
			var filename = this.path() + 'lng/' + this.i18.module.helptext;

			// load help text
			$.get(filename).done(function(data) {

				helpText = data;
				deferred.resolve(helpText);

			});

		} else {

			deferred.resolve(helpText);

		}

		return deferred.promise();

	};

	/**
	 * displays the module help text
	 */
	this.showModuleHelpText = function() {

		if ( this.i18.module.helptext ) {

			this.getModuleHelpText().done(function(info) {

				var infoWindow = new jkqWindow();
				infoWindow.width = 580;

				infoWindow.showModal();
				infoWindow.setTitle( self.i18.module.help_title );
				infoWindow.closeOnOutsideClick();

				infoWindow.container.find('DIV.jkqWindowContent')
				.append(
						'<div class="epgwiz-help">' +
						info +
						'</div>'
				);

				infoWindow.adjustPos();

			});

		}

	};

	this.loadSearchList = function() {

		var seSearchDefinition = {
			name:		'',
			searchName:	'',
			minlen:		'',
			maxlen:		'',
			startdate:	'',
			enddate:	'',
			starttime:	'',
			endtime:	'',

			favgroup:	'',

			weekday:	{1: false, 2: false, 3: false, 4: false, 5: false, 6: false, 7: false},

			texts:		[],

			channels:	[],
		};

		var webcSearches = JK.model( [] );
		webcSearches.sortBy('searchName');

		var seSearches = JK.model( [] );
		seSearches.sortBy('searchName');


		var popup = new jkqWindow();
		popup.width = 320;

		popup.showModal();
		popup.setTitle( this.i18.module.search_title );
		popup.closeOnOutsideClick();

		var $content = popup.container.find('DIV.jkqWindowContent');

		var $template = JK.template(
				this.templates.searches,
				{
					i18			: this.i18,
					webcSearches: webcSearches,
					seSearches	: seSearches,
				}
		).appendTo($content)

		// load webc search definition
		.on('click', '.epgwiz-webc-searches .epgwiz-search-item', function() {
			$(this).ddtData( function() {
				self.loadSearchDefinition( this.searchName );
			});
			popup.close();
		})

		// load webc search definition
		.on('click', '.epgwiz-se-searches .epgwiz-search-item', function() {
			$(this).ddtData( function() {
				self.loadSearchFields( this );
			});
			popup.close();
		})

		// delete search definition
		.on('click', '.epgwiz-webc-searches .epgwiz-saved-search-delete', function(e) {
			e.stopPropagation();
			$(this).parents( '.epgwiz-search-item' ).ddtData( function() {
				self.deleteSearchDefinition( this ).done( function() {
					popup.close();
				} );
			});
		})

		// toggle WebControl/SmartEPG list
		.on('click', '.epgwiz-se-label', function() {
			$(this).parents('.epgwiz-saved-search-list').toggleClass('epgwiz-display-se-searches');
		})

		;

		// load search index file
		$.getJSON(indexFileName, function(savedSearchList){
			$.each(savedSearchList, function(i, searchObject){
				webcSearches.add( searchObject );
			});
		});

		// load SE searches
		$.get(smartepgSearch, function(seDefinition){

			var properties = seDefinition.split('\n');
			var seSearch = $.extend(true, {}, seSearchDefinition);
			$.each( properties, function(index, value) {

				var match = value.match(/^([^:]+):\s+(.*)$/);
				var tLen = 0;
				if (match) {

					switch (match[1]) {

						case "Name":
							seSearch.name		= 'SmartEPG_' + match[2];
							seSearch.searchName	= match[2];
							break;

						case "ChannelID":
							seSearch.channels.push({logoid: match[2]});
							break;

						case "StartDate":
							tLen = parseInt(match[2], 16);
							if (tLen > 0) {
								seSearch.startdate = self.MJDtoJSON(tLen);
							}
							break;

						case "EndDate":
							tLen = parseInt(match[2], 16);
							if (tLen > 0) {
								seSearch.enddate = self.MJDtoJSON(tLen);
							}
							break;

						case "StartTime":
							tLen = parseInt(match[2]);
							if (tLen == 6144) tLen = 0;
							if (tLen > 0) {
								seSearch.starttime = ("00" + ((tLen & 0xff00) >> 8).toString()).slice(-2) + ":" + ("00" + (tLen & 0x00ff).toString()).slice(-2);
							}
							break;

						case "EndTime":
							tLen = parseInt(match[2]);
							if (tLen == 6144) tLen = 0;
							if (tLen > 0) {
								seSearch.endtime = ("00" + ((tLen & 0xff00) >> 8).toString()).slice(-2) + ":" + ("00" + (tLen & 0x00ff).toString()).slice(-2);
							}
							break;

						case "MinLength":
							tLen = parseInt(match[2]);
							if (tLen > 0) {
								seSearch.minlen = ("00" + (Math.floor(tLen / 60)).toString()).slice(-2) + ":" + ("00" + (tLen % 60).toString()).slice(-2);
							}
							break;

						case "MaxLength":
							tLen = parseInt(match[2]);
							if (tLen > 0) {
								seSearch.maxlen = ("00" + (Math.floor(tLen / 60)).toString()).slice(-2) + ":" + ("00" + (tLen % 60).toString()).slice(-2);
							}
							break;

						case "WeekDays":
							$(match[2].split('')).each( function (index, character) {
								seSearch.weekday[parseInt(character)] = true;
							});
							break;

						case "Search":

							var searchFields = match[2].split(';');

							var tT = false;
							var tD = false;
							var tE = false;

							switch(searchFields[0]) {
								case "B": tT = true;  tD = true;  tE = true; 	break;
								case "T": tT = true;  tD = false; tE = false; 	break;
								case "D": tT = false; tD = true;  tE = false; 	break;
								case "A": tT = true;  tD = true;  tE = true; 	break;
								case "E": tT = false; tD = false; tE = true; 	break;
							}

							var tX = searchFields[1].trim();
							tX = tX.replace(/^\*/,'');	//Trim leading * if one exists
							tX = tX.replace(/\*$/,'');	//trim trailing * if one exists

							var p = parseInt(searchFields[2]);
							var points = Math.trunc( (p + Math.sign(p)*9) / 10) * 10;
							if ( Math.abs( points ) > 99 ) {
								points = Math.sign( points ) * 99;
							}

							seSearch.texts.push({
								text:	tX,
								title:	tT,
								desc:	tD,
								ext:	tE,
								points:	points.toString(),
							});
							break;
					}

				} else if ( value.match(/^-+$/) ) {
					seSearches.add( seSearch );

					// next object
					seSearch = $.extend(true, {}, seSearchDefinition);
				}

			});

		});

		$(popup).on( 'close', function() {
			JK.removeTemplate( $template, true );
		} );

		popup.adjustPos();

	};

}
