/**
 * EPG event info modul
 */
tma.registerModule( eventinfo );
eventinfo.prototype = new tmaModule();
function eventinfo() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'eventinfo';

	var self = this;

	this.dependencies = {

			css: [
			      "eventinfo.css"
			      ],

			js: [
			     "../genrecolours/genrecolours.js"
			     ],

			templates: [
					     "content.html",
			]
	};

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'showEvent' );
		tma.fn.addFeature( this, 'findEvent' );

		return $.Deferred().resolve();

	};

	/**
	 * Show EPG event popup
	 *
	 * @param {tmaFeature}	f
	 * @param {TmaEpgEvent}	event		tma EPG event object
	 */
	this.showEvent = function (f, event) {

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();


		this.init().done(function() {

			// load genre list
			tma.apiEpgGenre().done(function() {

				var popup = self.createPopup(event);

				// return popup instance
				f.stop(popup);

			});

		});

	};

	/**
	 * Show EPG event popup by EventID in time range
	 *
	 * @param {tmaFeature}	f
	 * @param {TmaEpgEvent}	options		Object containing eventid, channel, starttime, endtime
	 */
	this.findEvent = function (f, options) {

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			// load genre list
			tma.apiEpgGenre().done(function() {

				options.channel.getEvents( options.starttime, options.endtime ).done(function(events) {
					for (var i = 0; i < events.length; i++) {
						if (events[i].eventid == options.eventid) {

							var popup = self.createPopup(events[i], options.channel);

							// return popup instance
							f.stop(popup);

							break;
						}
					}
				});

			});

		});

	};

	/**
	 * create window
	 *
	 * @param {TmaEpgEvent}	event		tma EPG event object
	 * @returns {jkqWindow}
	 */
	this.createPopup = function(event) {

		var popup = new jkqWindow();
		popup.width = 500;

		popup.show();
		popup.setTitle( event.channel.servicename );
		popup.closeOnOutsideClick();

		var $content = popup.container.find('DIV.jkqWindowContent');

		if (!TmaEpgEvent.prototype.evtinfoStartdate) {
			TmaEpgEvent.mixin({
				evtinfoStartdate: function() {
					"watch: {{starttime_local}}";
					return tma.date( tma.i18.eventstart_fmt, this.starttime_local);
				}
			});
		}

		var $template = JK.template(
				this.templates.content,
				{
					i18		: this.i18,
					event	: event,
					prefs	: prefs,
				}
		).appendTo($content);

		// is "edittimer" available
		if (tma.fn.editTimer) {

			// edit the timer of this event
			$template.on('click', 'DIV.eventinfo-top-bar BUTTON.ic-clock', function() {

				tma.fn.editTimer( event.reservation.timer );

			});

			// create a new timer for this event
			$template.on('click', 'DIV.eventinfo-top-bar BUTTON.ic-rec', function() {

				var event_start 	= event.startDate().getTime();
				var event_duration	= parseInt( (event.endDate().getTime() - event_start) / 60000);

				// create a new timer
				tma.fn.editTimer(
						new TmaTimer({
							index		: -1,		// new timer
							isrec		: 1,		// record
							starttime	: tma.toPvrDate(event_start - (prefs.prePadding * 60000) ),
							duration	: event_duration + prefs.prePadding + prefs.postPadding,
							filename	: event.name + tma.config.recextension,
							logoid		: event.channel.logoid,
//							chname		: event.channel.servicename,
							eventid1	: event.eventid,
							eventid2	: event.eventid,
//							servicetype	: event.channel.chtype,
//							serviceindex: event.channel.chnum,
//							serviceid	: event.serviceid,
						})
				);

			});
			// handle modified timer
			$(tma).on('tmaTimerModified.eventinfo' + popup.ctx.id, function() {
				popup.close();
			});

			// handle added timer
			$(tma).on('tmaTimerAdded.eventinfo' + popup.ctx.id, function(timer) {
				popup.close();
			});

			// handle deleted timer
			$(tma).on('tmaTimerDeleted.eventinfo' + popup.ctx.id, function() {
				popup.close();
			});

			$(popup).on('close', function() {
				// remove timer handlers on close
				$(tma).off('tmaTimerModified.eventinfo' + this.ctx.id);
				$(tma).off('tmaTimerAdded.eventinfo' + this.ctx.id);
				$(tma).off('tmaTimerDeleted.eventinfo' + this.ctx.id);
			});

		}

		/* config */

		$template.on('click', 'BUTTON.ic-gear', function() {
			$template.find('DIV.eventinfo-config').slideToggle();
		});

		// config pre-padding
		$template.on('click', '.eventinfo-config .pretime BUTTON.less', function() {

			if (prefs.prePadding > 0) {
				prefs.setPrePadding( prefs.prePadding - 1 );
			}

		});

		$template.on('click', '.eventinfo-config .pretime BUTTON.more', function() {

			prefs.setPrePadding( prefs.prePadding + 1 );

		});

		// config post-padding
		$template.on('click', '.eventinfo-config .posttime BUTTON.less', function() {

			if (prefs.postPadding > 0) {
				prefs.setPostPadding( prefs.postPadding - 1 );
			}

		});

		$template.on('click', '.eventinfo-config .posttime BUTTON.more', function() {

			prefs.setPostPadding( prefs.postPadding + 1 );

		});

		// Genre colouring
		tma.fn.getModule('genrecolours').createConfig(
				prefs.genreColoring,
				function(isEnabled) { // onEnableChange
					prefs.setGenreColoring(isEnabled);
				}
		).appendTo($template.find('.eventinfo-config'));


		$(popup).on( 'close', function() {
			JK.removeTemplate( $template, true );
		} );

		popup.adjustPos({
			hAlign:	'right',
			effect:	'slide-rtl'
		});

		return popup;

	};

	/* settings */

	var prefs = JK.model({

			/**
			 * get the pre-padding time in minutes
			 *
			 * @return {Number}
			 */
			prePadding: parseInt(self.setting('pretime', 0)),

			/**
			 * set the pre-padding time in minutes
			 *
			 * @param {Number}	set
			 */
			setPrePadding: function(set) {
				self.setSetting( 'pretime', parseInt(set) );
				this.prePadding = parseInt(set);
			},

			/**
			 * get the post-padding time in minutes
			 *
			 * @return {Number}
			 */
			postPadding: parseInt(self.setting('posttime', 0)),

			/**
			 * set the post-padding time in minutes
			 *
			 * @param {Number}	set
			 */
			setPostPadding: function(set) {
				self.setSetting( 'posttime', parseInt(set) );
				this.postPadding = parseInt(set);
			},

			/**
			 * genre-coloring (*on|off)
			 *
			 * @return {Boolean}
			 */
			genreColoring: parseInt( self.localSetting( 'genre-coloring', 1 ) ) == 1,

			/**
			 * set the genre-coloring (on/off)
			 *
			 * @param {Boolean}	set		new genre-coloring setting
			 */
			setGenreColoring: function(set) {
				self.setLocalSetting( 'genre-coloring', (set) ? 1 : 0 );
				this.genreColoring = (set) ? true : false;
			},

	});

}
