/**
 * recordings module
 */
tma.registerModule( recordingspage );
recordingspage.prototype = new tmaModule();
function recordingspage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'recordingspage';

	this.pageGroup = 'files';

	this.dependencies = {

			"css": [
			        "recordings.css",
			        ],

           "templates": [
                         "config.html",
                         "filelist.html",
                         "morefn.html",
                         "head.html",
         				 "mobile.html"
                         ]

	};

	var self = this;

	var $contentTmpl, $configTmpl, $stickyTmpl, $moreFnTmpl;
	var filesAddHandler, updateTimestampHandler, changePathSectionHandler, recordingsSelectedHandler;

	/**
	 * setup module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );
		tma.fn.addFeature( this, 'playFile' );
		tma.fn.addFeature( this, 'startTap' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * display the content
	 *
	 * @returns jQueryDeferred
	 */
	this.show = function() {

		return this.init().done(function() {

			// Settings
			var sortColumn = self.localSetting('sort_clm', prefs.sortColumn);
			if (prefs.sort[sortColumn]) {
				prefs.sortColumn = sortColumn;
			}
			prefs.sortDir 		= self.localSetting('sort_dir', prefs.sortDir);
			prefs.displayMode	= parseInt(self.localSetting( 'channel_display', 3 ));
			prefs.mobileLayout	= self.localSetting( 'mobilelayout', $(window).innerWidth() < 599 );
			prefs.setColumnCfg( self.localSetting( 'columns', null ) );

			self.showSticky();

			self.showFiles();

			if (!tma.files.directory) {
				// initial load
				tma.apiGetFiles( "/DataFiles" );
			}
			self.updateRunningTaps();

			tma.call('addConfigItem', ['MainMenu', self]);

			self.insertConfig();

		});

	};

	/**
	 * refresh the files, obtained from the PVR in the browser window
	 */
	this.refresh = function() {

		JK.removeTemplate( $contentTmpl );
		JK.removeTemplate( $stickyTmpl );
		tma.apiGetFiles(tma.files.directory).done(function() {
			self.showSticky();
			self.showFiles();
		});


	};

	/**
	 * this page gets removed
	 */
	this.cleanup = function() {

		// remove templates
		JK.removeTemplate($stickyTmpl);
		JK.removeTemplate($moreFnTmpl);
		JK.removeTemplate($contentTmpl);
		JK.removeTemplate($configTmpl);

		// remove handlers
		tma.files.files.off(filesAddHandler);
		tma.status.off(updateTimestampHandler);
		tma.files.off(changePathSectionHandler);
		tma.files.off(recordingsSelectedHandler);
		filesAddHandler = null;
		updateTimestampHandler = null;
		changePathSectionHandler = null;
		recordingsSelectedHandler = null;

	};

	this.updateRunningTaps = function() {

		if (
				(tma.files.directory == '/mnt/hd/ProgramFiles') ||
				(tma.files.directory == '/mnt/hd/ProgramFiles/AutoStart')
		) {
			// refresh running taps
			tma.apiGetTaps();
		}

	};

	this.showSticky = function() {

		function updateBreadcrumbs(e, directory, obj) {

			obj.breadcrumbs.length = 0;
			var path = directory.replace(/^\//, '').split('/');
			var url = '';
			var bpath = [];
			for( var i = 0; i < path.length; i++) {

				url += '/' + path[i];
				obj.breadcrumbs.push({
					folder	: path[i],
					path	: (i > 1) ? url : '',
				});

			}

		}

		if (!tma.files.breadcrumbs) {

			tma.files.extendProp('breadcrumbs', []);

			tma.files.on('change', 'directory', updateBreadcrumbs);

			// init breadcrumbs
			updateBreadcrumbs('change', tma.files.directory, tma.files );

		}

		// render template
		$stickyTmpl = JK.template(
				this.templates.head,
				{
					i18		: this.i18,
					prefs	: prefs,
					status	: tma.status,
					files	: tma.files,
				}
		).appendTo( '.sticky-content' )

		.on('click', 'BUTTON[path]', function(e) {
			tma.apiGetFiles( $(this).attr('path') );
		})

		.on('click', '.files-head-breadcrumbs A', function(e) {
			e.preventDefault();
			tma.apiGetFiles( $(this).attr('href') );
		})

		// add event listener to delete multiple files
		.on('click', '.files-multisel-delete', function() {

			var files = [];
			$('.files-file[inode]').has('.files-file-select:checked').each(function() {
				/** @returns {TmaFile} */
				var file = tma.files.find( 'inode', parseInt( $(this).attr( 'inode' ) ) );
				if (file) {
					files.push( file );
				}
			});
			self.deleteFiles(null, tma.files.directory, files);

		})

		// add event listener to move multiple files
		.on('click', '.files-multisel-move', function() {

			// open a directory selector...
			if (tma.featureExist('selectDirectory')) {

				tma.fn.selectDirectory( '/mnt', tma.files.directory, function(selectedPath) {

					$('.files-file[inode]').has('.files-file-select:checked').each(function() {
						/** @returns {TmaFile} */
						var file = tma.files.find( 'inode', parseInt( $(this).attr( 'inode' ) ) );
						if (file) {
							self.moveFile(null, tma.files.directory, file, selectedPath).done( function() {
								tma.files.recordingsSelected--;
							} );
						}
					});

				});
			}

		})

		// set sort event handler
		.on('click', '.files-head-sort-item', function(e) {
			var sortColumn = $(this).attr('clm-name');
			var sortDir = $(this).attr('sort-dir');
			if (prefs.sort[sortColumn]) {
				prefs.sortColumn = sortColumn;
				prefs.sortDir = sortDir;
				tma.files.files.sortBy(prefs.sort[prefs.sortColumn].key, prefs.sortDir);
			}
			$(this).parents( '.files-head-sort-box' ).removeClass('files-head-sort-box-open');
		})
		.on('touchstart', '.files-head-sort-item', function(e) {
			// don't propagate a tap on an item up to box-opener
			e.stopPropagation();
		})

		// open sort options
		.on('touchstart', '.files-head-sort', function(e) {
			$('.files-head-sort-box').toggleClass('files-head-sort-box-open');
		})
		// hover effect with mouse
		.on('mouseenter', '.files-head-sort', function(e) {
			$('.files-head-sort-box').addClass('files-head-sort-box-open');
		})
		.on('mouseleave', '.files-head-sort', function(e) {
			$('.files-head-sort-box').removeClass('files-head-sort-box-open');
		})

		;

	};

	/**
	 * display the file list of the current files object
	 */
	this.showFiles = function () {

		// sort table cells in prefs.columnCfg order
		function rowSort(a,b) {
			var colA = $(a).attr('clm-name');
			var colB = $(b).attr('clm-name');
			var indexA = prefs.columnCfg.findIndex( function(clm) { return (clm.name == colA); } );
			var indexB = prefs.columnCfg.findIndex( function(clm) { return (clm.name == colB); } );
			return indexA - indexB;
		}

		if (!TmaFile.prototype.recordingsFiledateFmt) {
			// add formater
			TmaFile.mixin({

				recordingsFiledateFmt: function() {
					if ( this.isFile() ) {
						return tma.date( self.i18.recdate_fmt, this.sortdate );
					} else {
						return '';
					}
				},

				filesShortDateFmt: function() {
					if ( this.isFile() ) {
						return tma.date( self.i18.shortdate_fmt, this.sortdate );
					} else {
						return '';
					}
				},

				filesStartTimeFmt: function() {
					if ( this.isFile() ) {
						return tma.date( "H:i", this.sortdate );
					} else {
						return '';
					}
				},

			});

			// add selection counter
			tma.files.extend({recordingsSelected: 0});

			// if directory changes, reset selection counter
			tma.files.on('change', 'directory', function(e, val, obj) {
				obj.recordingsSelected = 0;
			});

		}

		// sort files
		tma.files.sort(prefs.sort[prefs.sortColumn].key, prefs.sortDir);

		var templ = prefs.mobileLayout ? this.templates.mobile : this.templates.filelist;

		$contentTmpl = JK.template( templ, {
			files			: tma.files,
			i18				: this.i18,
			prefs			: prefs,
		}).appendTo('DIV#content');

		// sort table columns in user defined order
		$('.files TR').each(function() {
			$(this).find('TD,TH').sort(rowSort).appendTo(this);
		});

		// hide deselected columns
		$.each(prefs.columnCfg, function() {
			if (!this.display) $contentTmpl.find( ( 'TD[clm-name="{0}"],TH[clm-name="{0}"]' ).format( this.name ) ).hide();
		});

		// set sort event handler
		$contentTmpl.on('click', '.files-sort', function(e) {
			e.preventDefault();

			var column = $(this).parent().attr('clm-name');
			if (prefs.sort[column]) {
				// is this column is currently sorted?
				if (prefs.sort[column].sorted) {
					// yes => change sort direction
					prefs.sortDir = (prefs.sortDir == 'asc') ? 'desc' : 'asc';
				} else {
					// no => change sortColumn
					prefs.sortColumn = column;
				}
				tma.files.sort(prefs.sort[column].key, prefs.sortDir);
			}
		});

		// add event listener for folders to change directories
		$contentTmpl.on('click', 'TR[attr="224"]', function() {

			var file = tma.files.find( 'inode', parseInt($(this).attr( 'inode' )) );
			if (file) {
				var directory = tma.files.directory + '/' + file.name;
				tma.apiGetFiles(directory);
			}
		});

		// prevent display of rec-infos on checkbox click
		$contentTmpl.on('click', '.files-clm-name INPUT', function(e) {
			e.stopPropagation();
		});

		// file selection
		$contentTmpl.on('click', '.files-file-select', function() {

			var $row = $(this).parents('TR[inode]');
			if ( $(this).is(':checked') ) {
				$row.addClass( 'jk-mark' );
			} else {
				$row.removeClass( 'jk-mark' );
			}

			var $selected = $('.files-file-select').filter(':checked');
			tma.files.recordingsSelected = $selected.length;

		});

		if (!recordingsSelectedHandler) {
			recordingsSelectedHandler = tma.files.on('change', 'recordingsSelected', function(e, val) {

				if (val > 0) {
					// at least one file is selected
					if ( $('.files-head-breadcrumbs').is(':visible') ) {
						// animated replacement from breadcrumbs to multiselect-actions
						$('.files-head-breadcrumbs').slideToggle();
						$('.files-head-multisel-actions').slideToggle();
						$('.files-head-paths').fadeTo(400, 0, function(){
							$('.files-head-paths').css("visibility", "hidden");
						});
					}
				} else {
					// no file is selected
					if ( $('.files-head-breadcrumbs').is(':hidden') ) {
						// animated replacement from multiselect-actions to breadcrumbs
						$('.files-head-breadcrumbs').slideToggle();
						$('.files-head-multisel-actions').slideToggle();
						$('.files-head-paths').css("visibility", "visible");
						$('.files-head-paths').fadeTo(400, 1);
					}
				}

			});
		}

		// add event listener to show infos
		$contentTmpl.on('click', '.files-clm-name', function(e) {

			var $row = $(this).parents('TR[inode]');
			var file = tma.files.find( 'inode', parseInt($row.attr( 'inode' )) );
			if (file) {

				// check, if any module handles the 'showRecInf' event
				// and prevent reopening of the same info
				if (
						(file.infpresent) &&
						(tma.fn.showRecInf) &&
						!$row.hasClass('jk-mark')
				) {


					tma.fn.showRecInf( file ).done(function(popup) {

						popup.inode = file.inode;
						$(popup).on('close', function() {
							$('TR[inode="' + this.inode.toString() + '"]').removeClass('jk-mark');
						});

					});

					$row.addClass('jk-mark');

				} else if (file.isTextfile() ) {

					// open a text file in a new window
					window.open(tma.files.directory + '/' + file.name, '_blank');

				}
			}

		});

		// close "more functions" and remove row highlighting
		var closeMorefn = function() {
			JK.removeTemplate($moreFnTmpl, false);
			$('.files-fn-box').fadeOut(function() {
				$(this).parents('TR').removeClass('jk-mark');
				$(this).remove();
			});
			$moreFnTmpl = null;
		};

		// add event listener to show "more functions"
		$contentTmpl.on('click', '.files-fn-open', function() {

			var $row = $(this).parents('TR[inode]');
			if ( !$row.is('.jk-mark') ) {

				// remove previous moreFn's
				closeMorefn();

				$row.addClass('jk-mark');

				$moreFnTmpl = JK.template(
						self.templates.morefn,
						{
							i18: self.i18
						}
				).appendTo( $(this).parent() );

				// test, if panel fits the screen
				var scrollTop	= $(document).scrollTop();
				var windowH = $(window).height();
				var h = $moreFnTmpl.outerHeight();
				var top = $moreFnTmpl.offset().top;

				if ( top + h > scrollTop + windowH ) {
					$moreFnTmpl.addClass('files-fn-dropup');
				}

				$moreFnTmpl.css({
					display: 'none',
					visibility: 'visible'
				}).fadeIn();

				/** @returns {TmaFile} */
				var file = tma.files.find( 'inode', parseInt($row.attr( 'inode' )) );
				if (file) {
					tma.call('fileCreateMoreFn', [$moreFnTmpl, file]);
				}

			}

		});

		// add event listener to close moreFn
		$contentTmpl.on('click', 'BUTTON.files-close-fn-panel', function() {
			closeMorefn();
		});

		// add event listener to stop playback or tap
		$contentTmpl.on('click', 'BUTTON.files-stop', function() {

			var $row = $(this).parents('TR');
			/** @returns {TmaFile} */
			var file = tma.files.find( 'inode', parseInt($row.attr( 'inode' )) );
			if (file) {

				if (file.isPlaying) {

					tma.stopPlayback();

				} else if (file.isRunningTap) {

					// find running TAP
					var tap = tma.taps.find(file.tap.tapid, tma.files.directory + '/' + file.name);
					if (tap) {
						tma.apiStopTap(tap);
					}

				}

			}

		});

		// add event listener to stop recording
		$contentTmpl.on('click', 'BUTTON.files-stop-rec', function() {

			var $row = $(this).parents('TR');
			/** @returns {TmaFile} */
			var file = tma.files.find( 'inode', parseInt($row.attr( 'inode' )) );
			if (file && file.recording) {
				tma.stopRecording(file.recording.slot);
			}

		});

		// add event listener to play files
		$contentTmpl.on('click', 'BUTTON.files-play', function() {

			var $row = $(this).parents('TR');
			/** @returns {TmaFile} */
			var file = tma.files.find( 'inode', parseInt($row.attr( 'inode' )) );
			if (file && file.isPlayable()) {

				// start playback
				var startposition = 0;
				if (file.infpresent) {
					startposition = file.infdata.bookmarks.startposition || 0;
				}

				self.playFile(null, tma.files.directory, file, startposition);

			}

		});

		// add event listener to delete files
		$contentTmpl.on('click', 'BUTTON.files-delete', function() {

			var $row = $(this).parents('TR');
			/** @returns {TmaFile} */
			var file = tma.files.find( 'inode', parseInt($row.attr( 'inode' )) );
			if (file) {
				self.deleteFile(null, tma.files.directory, file)
				.on( 'close', function() {
					closeMorefn();
				} );
			} else {
				closeMorefn();
			}

		});

		// add event listener to rename files
		$contentTmpl.on('click', 'BUTTON.files-rename', function() {

			var $row = $(this).parents('TR');
			/** @returns {TmaFile} */
			var file = tma.files.find( 'inode', parseInt($row.attr( 'inode' )) );
			if (file) {
				self.renameFile(null, tma.files.directory, file)
				.on( 'close', function() {
					closeMorefn();
				} );
			} else {
				closeMorefn();
			}

		});

		// add event listener to start TAPs
		$contentTmpl.on('click', 'BUTTON.files-start-tap', function() {

			var $row = $(this).parents('TR');
			/** @returns {TmaFile} */
			var file = tma.files.find( 'inode', parseInt($row.attr( 'inode' )) );
			if (file && file.tappresent) {
				// start TAP
				self.startTap(null, tma.files.directory, file);
			}
			closeMorefn();

		});

		// add event listener to move files
		$contentTmpl.on('click', 'BUTTON.files-move', function() {

			var $row = $(this).parents('TR');
			/** @returns {TmaFile} */
			var file = tma.files.find( 'inode', parseInt($row.attr( 'inode' )) );
			if (file && tma.featureExist('selectDirectory')) {
				// open a directory selector...
				tma.fn.selectDirectory( '/mnt', tma.files.directory, function(selectedPath) {
					self.moveFile(null, tma.files.directory, file, selectedPath);
				}).done(function( popup ) {
					popup.on('close', function() {
						closeMorefn();
					});
				});
			} else {
				closeMorefn();
			}

		});

		var $tbody = $contentTmpl.find('TABLE.sortable-columns TBODY');
		if ($tbody.length) {
			// sort columns after TBODY is re-attached
			if (!filesAddHandler) {
				filesAddHandler = tma.files.on('attach', 'files', function(e, file) {

					// sort table columns in user defined order
					$tbody.find('TR').each(function() {
						$(this).find('TD').sort(rowSort).appendTo(this);
					});

					// hide deselected columns
					$.each(prefs.columnCfg, function() {
						if (!this.display) $tbody.find( ( 'TD[clm-name="{0}"]' ).format( this.name ) ).hide();
					});
				});
			}
		}

		if (!updateTimestampHandler) {
			updateTimestampHandler = tma.status.on('change', 'timestamp', function() {
				self.updateRunningTaps();
			});
		}

		if (!changePathSectionHandler) {
			changePathSectionHandler = tma.files.on('change', 'pathSection', function() {
				self.updateRunningTaps();
			});
		}

	};

	/**
	 * displays an alert-box with a file operation error message
	 *
	 * @param {String} fileAction	Name of the file operation
	 * @param {Object} data			API result data
	 */
	this.fileError = function(fileAction, data) {

		var errmsg = '';
		if ( (data.mpg_status !== 0) && (data.mpg_errno) ) {
			errmsg += '<i>{recext}</i>: {mpg_error}<br>';
		}
		if ( (data.inf_status !== 0) && (data.inf_errno) ) {
			errmsg += '<i>inf</i>: {inf_error}<br>';
		}
		if ( (data.nav_status !== 0) && (data.nav_errno) ) {
			errmsg += '<i>nav</i>: {mpg_error}<br>';
		}

		new jkqAlert({
			title: tma.i18.error.dialog_title,
			text: (
					'{message}' +
					'<span style="display: block; text-align: left; margin-top: 0.5em">' +
						errmsg +
					'</span>'
			).format({
				message:	fileAction,
				recext:		tma.config.recextension,
				mpg_error:	(
						tma.i18.error.files.errno[data.mpg_errno] ?
								tma.i18.error.files.errno[data.mpg_errno] :
									'errno = ' + data.mpg_errno
				),
				inf_error:	(
						tma.i18.error.files.errno[data.inf_errno] ?
								tma.i18.error.files.errno[data.inf_errno] :
									'errno = ' + data.nav_errno
				),
				nav_error:	(
						tma.i18.error.files.errno[data.nav_errno] ?
								tma.i18.error.files.errno[data.nav_errno] :
									'errno = ' + data.inf_errno
				),
			})
		}).confirm();

	};

	/**
	 * Feature handler: rename a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 * @param {String}		newDirectory
	 * @returns jQueryDeferred
	 */
	this.moveFile = function (f, directory, file, newDirectory) {

		var deferred = $.Deferred();

		if (tma.files.directory != newDirectory) {

			tma.apiRenameFile(
					directory, file.name, file.name_hex,
					newDirectory, file.name
			).done(function(data) {

				if (
						(data.mpg_status !== 0) ||
						(data.inf_status !== 0) ||
						(data.nav_status !== 0)
				) {

					self.fileError(tma.i18.error.files.move, data);
					deferred.reject();

				} else {

					deferred.resolve();

				}

			}).fail(function() {
				deferred.reject();
			});

		} else {

			deferred.reject();

		}

		return deferred.promise();

	};

	/**
	 * Feature handler: delete a list of files
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {Array}		files				array of tma file objects
	 */
	this.deleteFiles = function(f, directory, files) {

		function deleteResponseCheck( data ) {

			if (
					(data.mpg_status !== 0) ||
					(data.inf_status !== 0) ||
					(data.nav_status !== 0)
			) {
				self.fileError(tma.i18.error.files.del, data);
			} else {
				tma.files.recordingsSelected--;
			}

		}

		new jkqDialog({
			title: self.i18.delete_confirm_caption,
			text: self.i18.delete_multiple_confirm,
			buttons: {
				ok:				 tma.i18.respond_yes,
				jkqDialogCancel: tma.i18.respond_cancel
			}
		}).confirm(function(button) {

			if (button == 'ok') {

				for( var i = 0; i < files.length; i++) {
					tma.apiDeleteFile(directory, files[i].name, files[i].name_hex).done( deleteResponseCheck );
				}

			}

		});

	};

	/**
	 * Feature handler: delete a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 * @returns {jkqInputDialog}
	 */
	this.deleteFile = function(f, directory, file) {

		return new jkqDialog({
			title: self.i18.delete_confirm_caption,
			text: self.i18.delete_confirm.format( file.filenameNoExt() ),
			buttons: {
				ok:				 tma.i18.respond_yes,
				jkqDialogCancel: tma.i18.respond_cancel
			}
		}).confirm(function(button) {

			if (button == 'ok') {

				tma.apiDeleteFile(directory, file.name, file.name_hex).done(function(data) {

					if (
							(data.mpg_status !== 0) ||
							(data.inf_status !== 0) ||
							(data.nav_status !== 0)
					) {

						self.fileError(tma.i18.error.files.del, data);

					}

				});

			}

		});

	};

	/**
	 * Feature handler: rename a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 * @returns {jkqInputDialog}
	 */
	this.renameFile = function(f, directory, file) {

		return new jkqInputDialog({
			title: self.i18.rename_dialog_caption,
			text: self.i18.rename_dialog_text.format( file.filenameFmt().call(file) ),
			inputValue: file.filenameFmt().call(file).replace(/"/, '&quot;')
		}).confirm(function() {

			tma.apiRenameFile(
					directory, file.name, file.name_hex,
					directory, this.value() + ((file.infpresent) ? tma.config.recextension : '')
			).done(function(data) {

				if (
						(data.mpg_status !== 0) ||
						(data.inf_status !== 0) ||
						(data.nav_status !== 0)
				) {

					self.fileError(tma.i18.error.files.rename, data);

				}

			});

		});

	};

	/**
	 * Feature handler: play a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 * @param {Number}		startposition
	 */
	this.playFile = function (f, directory, file, startposition) {

		tma.apiPlayFile(directory, file.name, file.name_hex, startposition).done(function(playObject) {

			if (playObject.status !== 0) {

				new jkqAlert({
					title:	tma.i18.error.dialog_title,
					text:	tma.i18.error.files.play
				}).confirm();

			}

		});

	};

	/**
	 * Feature handler: start a TAP
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file
	 */
	this.startTap = function (f, directory, file, startposition) {

		tma.apiStartTap(directory + '/' + file.name).done(function(result) {

			if (result.return_code !== 1 || result.error !== 0) {

				new jkqAlert({
					title:	tma.i18.error.dialog_title,
					text:	tma.i18.error.files.tap_start
				}).confirm();

			}

		}).fail(function() {

			new jkqAlert({
				title:	tma.i18.error.dialog_title,
				text:	tma.i18.error.files.tap_start
			}).confirm();

		});

	};

	this.insertConfig = function () {

		// setup column config module
		var columnconfig = tma.fn.getModule('columnconfig');
		columnconfig.init().done(function() {

			// prepare template data (mixed with data, required by column config)
			var data = $.extend(
					{
						i18:			self.i18,
						i18g:			tma.i18,	// global language def.
						prefs:			prefs,
					},
					columnconfig.getTemplateData( prefs.columnCfg,	self.i18.columns )
			);

			$configTmpl = JK.template(
					self.templates.config,
					data,
					{ columnconfig: columnconfig.templates.content }
			).appendTo('DIV#config');

			// add column config event handlers
			columnconfig.addEventHandler(
					$configTmpl,
					'.files TABLE',
					function(cfg) {
						self.setLocalSetting('columns', cfg );
					}
			);

			// logo/text
			$('DIV#config INPUT[name="logotext"]')
			.click(function() {
				prefs.setDisplayMode( $('DIV#config INPUT[name="logotext"]:checked').val() );
			})
			.filter('[value="' +  prefs.displayMode + '"]' ).attr('checked', true);

			// mobile layout
			$('DIV#config INPUT[name="mobile"]')
			.click(function() {
				prefs.setMobileLayout( $(this).is(':checked') );
				JK.removeTemplate( $contentTmpl );
				self.showFiles();
			});

		});

	};

	/* Settings */

	var prefs = JK.model({

		/**
		 * current sorted column
		 *
		 * @type {String}
		 */
		sortColumn: 'filedate',

		/**
		 * displayMode setting
		 *
		 * @type {string}	asc:  sort ascending
		 * 					desc: sort descending
		 */
		sortDir: 'asc',

		sort: {
			filename:		{ sorted: false, key: 'filename'},
			filedate:		{ sorted: true , key: 'sortdate'},
			filesize:		{ sorted: false, key: 'size'},
			duration:		{ sorted: false, key: 'infdata.header.durationmin'},
		},

		/**
		 * displayMode setting
		 *
		 * @type {Number}		1: display logo
		 * 						2: display name
		 * 						3: display both
		 */
		displayMode: 3,

		/**
		 * set the displayMode setting
		 *
		 * @param {Number}	set		new displayMode setting
		 * 							1: display logo
		 * 							2: display name
		 * 							3: display both
		 */
		setDisplayMode: function(set) {
			var val = parseInt(set);
			if ((val >= 1) && (val <= 3)) {
				self.setLocalSetting( 'channel_display', val );
				this.displayMode = val;
			}
		},

		columnCfg: [
					{ name: 'channel'	, display: true },
					{ name: 'filename'	, display: true },
					{ name: 'filesize'	, display: true },
					{ name: 'filedate'	, display: true },
					{ name: 'duration'	, display: true },
					{ name: 'filefn'	, display: true }
			],

		setColumnCfg: function(cfg) {
			if (cfg) {
				// sort
				this.columnCfg.sort( function(a, b) {
					return Object.keys(cfg).indexOf(a.name) - Object.keys(cfg).indexOf(b.name);
				});
				// set display
				for( var i = 0; i < this.columnCfg.length; i++) {
					if (cfg[this.columnCfg[i].name] !== undefined) {
						this.columnCfg[i].display = cfg[this.columnCfg[i].name];
					}
				}
			}
		},

		/**
		 * mobile layout setting
		 */
		mobileLayout: false,

		/**
		 * set the mobile layout setting
		 *
		 * @param {Boolean}	set
		 */
		setMobileLayout: function(set) {
			var val = set === true;
			self.setLocalSetting( 'mobilelayout', val);
			this.mobileLayout = val;
		},

	});

	prefs.on('change', 'sortColumn', function(e, val, obj, prop, oldVal) {
		self.setLocalSetting( 'sort_clm', val );
		if (oldVal && prefs.sort[oldVal]) {
			prefs.sort[oldVal].sorted	= false;
		}
		if (val && prefs.sort[val])	{
			prefs.sort[val].sorted		= true;
		}
	});

	prefs.on('change', 'sortDir', function(e, val) {
		self.setLocalSetting( 'sort_dir', val );
	});

}
