/**
 * edit timer popup module
 */
tma.registerModule( timeredit );
timeredit.prototype = new tmaModule();
function timeredit() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'timeredit';

	var self = this;

	this.dependencies = {
			css: [
				"timeredit.css",
				"../../css/timepicker.css"
			],

			js: [
				"../../js/jquery-ui-timepicker-addon-jk.js"
			],

			templates: [
				"content.html"
			]
	};

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'editTimer' );

		return $.Deferred().resolve();

	};

	/**
	 * display the reservation form
	 *
	 * @param {tmaFeature}	f
	 * @param {TmaTimer}	timer		tma timer object
	 * @returns {jQueryDeferred}
	 */
	this.editTimer = function(f, timer) {

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			// set defaults
			if	(!timer.filename) {
				timer.modFilename	= self.i18.default_filename;// + tma.config.recextension;
			}
			// The default start time is in 5 minutes
			if	(!timer.starttime) {
				timer.modStarttime = tma.toPvrDate(new Date(new Date() - tma.config.timeOffset + 300000));
			}
			// The default duration is 2 hours
			if	(!timer.duration) {
				timer.modDuration	= 120;
			}
			// The default channel is the first channel in the channel list
			if (!timer.logoid && tma.channels.channels.length) {
				timer.modLogoid = tma.channels.channels[0].logoid;
			}

			// init the window
			var popup = new jkqWindow();
			popup.showModal();
			popup.setTitle(
					(timer.index < 0) ? self.i18.caption_new : self.i18.caption_edit
			);

			var $content = popup.container.find('DIV.jkqWindowContent');

			// prepare Formatters (only once)
			if (!TmaTimer.prototype.timereditStarttime) {
				TmaTimer.mixin({
					timereditStarttime: function() {
						"watch: {{modStarttime}}";
						return tma.date(self.i18.timerdate_fmt, this.modStarttime);
					},
					timereditEndtime: function() {
						"watch: {{modEndtime}}";
						return tma.date(self.i18.timerdate_fmt, this.modEndtime);
					},
				});
			}

			$contentTmpl = JK.template(
					self.templates.content,
					{
						i18				: self.i18,
						timer			: timer,
						frequencies		: tma.i18.frequency,
						channels		: tma.channels.channels,
					}
			).appendTo($content);

			//Start time toolbar
			$('DIV#starttoolbar > BUTTON').button({text: true}).click(function() {
				var amount = parseInt( $(this).text() );
				timer.modStarttime = tma.toPvrDate( new Date(timer.modStarttime).getTime() + amount * 60000 );
			});

			//Duration time toolbar
			$('DIV#durtoolbar > BUTTON').button({text: true}).click(function() {
				var amount = parseInt( $(this).text() );
				timer.modDuration += amount;
			});

			//End time toolbar
			$('DIV#endtoolbar > BUTTON').button({text: true}).click(function() {
				var amount = parseInt( $(this).text() );
				timer.modDuration += amount;
			});

			popup.adjustPos({
				hAlign: 'right',
				effect: 'slide-rtl'
			});

			/**
			 * this could be an alternative time selection
			 * instead of slider access
			 * set controlType: dtpSpinner in dtpOptions
			var dtpSpinner =  {
					create: function(tp_inst, obj, unit, val, min, max, step){
						$('<input class="ui-timepicker-input" value="'+val+'" style="width:50%">')
						.appendTo(obj)
						.spinner({
							min: min,
							max: max,
							step: step,
							change: function(e,ui){ // key events
								// don't call if api was used and not key press
								if(e.originalEvent !== undefined)
									tp_inst._onTimeChange();
								tp_inst._onSelectHandler();
							},
							spin: function(e,ui){ // spin events
								tp_inst.control.value(tp_inst, obj, unit, ui.value);
								tp_inst._onTimeChange();
								tp_inst._onSelectHandler();
							}
						});
						return obj;
					},
					options: function(tp_inst, obj, unit, opts, val){
						if(typeof(opts) == 'string' && val !== undefined)
							return obj.find('.ui-timepicker-input').spinner(opts, val);
						return obj.find('.ui-timepicker-input').spinner(opts);
					},
					value: function(tp_inst, obj, unit, val){
						if(val !== undefined)
							return obj.find('.ui-timepicker-input').spinner('value', val);
						return obj.find('.ui-timepicker-input').spinner('value');
					}
			};
			 */

			// setup the datepickers
			var dtpOptions = {
					//controlType: dtpSpinner,
					controlType: 'slider',
					addSliderAccess: true,
					sliderAccessArgs: { touchonly: false },
					firstDay: 1,
					timeFormat: "HH:mm:ss",
					pickerTimeFormat: "HH:mm",
					showSecond: false,
					dateFormat: "yy/mm/dd",
					monthNames: [],
					monthNamesShort: [],
					dayNames: [],
					dayNamesShort: [],
					dayNamesMin: [],
					timeText: self.i18.dtp_time_text,
					hourText: self.i18.dtp_hour_text,
					minuteText: self.i18.dtp_minute_text,
					closeText: self.i18.dtp_close_text,
					currentText: self.i18.dtp_current_text,
					timeOnlyTitle: self.i18.dtp_time_only_title,
			};

			$.each(tma.i18.months, function(i, month) {
				dtpOptions.monthNames.push(month.name);
				dtpOptions.monthNamesShort.push(month.shortname);
			});

			$.each(tma.i18.days, function(i, day) {
				dtpOptions.dayNames.push(day.name);
				dtpOptions.dayNamesShort.push(day.shortname);
				dtpOptions.dayNamesMin.push(day.shortname.slice(0, 2));
			});

			//Setup the start date field
			$("#timerstart").datetimepicker($.extend({}, dtpOptions, {
				onSelect: function (selectedDateTime) {
					timer.modStarttime = selectedDateTime;
				}
			}));

			$("#timerstarticon,#timerstarttext").click(function () {
				$("#timerstart").datetimepicker("show");
			}).css( 'cursor', 'pointer' );

			//Setup the duration field
			$("#timerduration").timepicker($.extend({}, dtpOptions, {
				timeFormat: "H:mm",
				onSelect: function (selectedDateTime) {
					// calculate minutes from H:mm
					var hm = selectedDateTime.split(':');
					timer.modDuration = parseInt(hm[0]) * 60 + parseInt(hm[1]);
				},
			}));

			$("#timerdurationicon,#timerdurationtext").click(function () {
				var durFmt = tma.formatDuration( timer.modDuration, 0 );
				$("#timerduration").timepicker('setTime', durFmt).timepicker("show");
			}).css( 'cursor', 'pointer' );

			// Setup the end date field
			$("#timerend").datetimepicker($.extend({}, dtpOptions, {
				onSelect: function (selectedDateTime) {
					timer.modEndtime = selectedDateTime;
				}
			}));

			$("#timerendicon,#timerendtext").click(function () {
				$("#timerend").datetimepicker("show");
			}).css( 'cursor', 'pointer' );

			$('#timertypediv').buttonset();
			var isRecChangeCallback = timer.on('change', 'modIsrec', function() {
				$('#timertypediv').buttonset('refresh');
			});

			// update the isrec data on selection change
			$('#timertypediv INPUT[name="timertype"]').change(function() {
				timer.modIsrec = parseInt($(this).val());
			});

			// update the channel data on selection change
			$('#timerservice').change(function() {

				var channel = tma.channels.channel( $(this).val() );
				if (channel) {
					timer.modLogoid		= channel.logoid;
				}

			});

			$('#timerfrequency').change(function() {
				timer.modReservationtype = parseInt($(this).val());
			});

			// apply filename
			$("#timername").on('input', function() {
				timer.modFilename = $(this).val();// + tma.config.recextension;
			});

			//Make the abort button hide the timer edit window
			$('#timerbuttonabort').click(function() {
				popup.close();
			});

			//Make the reset button reset the timer to its initial settings
			$('#timerbuttonreset').click(function() {
				timer.modReset();
			});

			//Make the create/update button do its thing
			$('#timerbuttonupdate').click(function(e) {

				e.stopPropagation();

				var dlgOptions;

				if (timer.index < 0) {
					dlgOptions = {
							title: self.i18.create_dialog_title,
							text: self.i18.create_dialog_text,
							autoSkipText: tma.i18.respond_auto_skip,
							autoSkipButton: 'tmaCreateTimer',
							buttons: {
								tmaCreateTimer:	 tma.i18.respond_yes,
								jkqDialogCancel: tma.i18.respond_cancel
							}
					};
				} else {
					dlgOptions = {
							title: self.i18.modify_dialog_title,
							text: self.i18.modify_dialog_text,
							autoSkipText: tma.i18.respond_auto_skip,
							autoSkipButton: 'tmaModifyTimer',
							buttons: {
								tmaModifyTimer:	 tma.i18.respond_yes,
								jkqDialogCancel: tma.i18.respond_cancel
							}
					};
				}

				new jkqDialog(dlgOptions).confirm(function(button) {

					if (button == dlgOptions.autoSkipButton) {

						return tma.apiTimerUpdate(timer).fail(function(xhr, errorType) {

								new jkqAlert({
									title:	tma.i18.error.dialog_title,
									text:	tma.i18.error.timer.create_modify
								}).confirm();

						}).done(function() {
							popup.close();
						});

					}
				});

			});

			// add event listener to delete timers
			$('#timerbuttondelete').click(function() {

				new jkqDialog({
					title: tma.i18.timer.delete_confirm_caption,
					text:  tma.i18.timer.delete_confirm_text.format(timer.getFilename(true)),
					autoSkipText: tma.i18.respond_auto_skip,
					autoSkipButton: 'tmaDeleteTimer',
					buttons: {
						tmaDeleteTimer:	 tma.i18.respond_yes,
						jkqDialogCancel: tma.i18.respond_cancel
					}
				}).confirm(function(button) {

					if (button == 'tmaDeleteTimer') {

						tma.apiDeleteTimer(timer).done(function() {

							popup.close();

						}).fail(function(xhr, errorType, error) {

							new jkqAlert({
								title:	tma.i18.error.dialog_title,
								text:	tma.i18.error.timer.del + '<br><br>' + error
							}).confirm();

						});

					}

				});

			});

			$(popup).on( 'close', function() {
				JK.removeTemplate( $contentTmpl, true );
				timer.off(isRecChangeCallback);
				timer.modReset();
			} );

			/**
			 * this method handled the feature, so
			 * stop the feature chain
			 * and return this popup instance
			 */
			f.stop(popup);

		});

	};

}
