/**
 * timers module
 */
tma.registerModule( timerspage );
timerspage.prototype = new tmaModule();
function timerspage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'timerspage';

	this.pageGroup = 'timers';

	this.dependencies = {

			css: [
			      "timers.css",
			],
			templates: [
				"timerlist.html",
				"config.html",
				"head.html",
				"mobile.html"
			]
	};

	var self = this;

	var $contentTmpl, $configTmpl, $stickyTmpl;
	var timersAddHandler;

	/**
	 * initialise module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the timers, obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			// Settings
			prefs.sortColumn	= self.localSetting('sort_clm', prefs.sortColumn);
			prefs.sortDir 		= self.localSetting('sort_dir', prefs.sortDir);
			prefs.displayMode	= parseInt(self.localSetting( 'channel_display', 3 ));
			prefs.typeFilter	= parseInt(self.localSetting( 'timersfilter', 3 )) || 3;
			prefs.mobileLayout	= self.localSetting( 'mobilelayout', $(window).innerWidth() < 599 );
			prefs.setColumnCfg( self.localSetting( 'columns', null ) );

//			$('#content').html('<img src="css/img/wait.gif">');

			tma.apiGetTimers().done(function() {

				self.showSticky();

				self.showTimers();

				self.insertConfig();

				tma.call('addConfigItem', ['MainMenu', self]);

			}).fail(function(xhr, errorType, error) {

				$("#timestamp").text( tma.i18.error.ajax.format(errorType, error) );
				// remove pending image
//				$('#content').empty();

			});

		});

	};

	/**
	 * refresh the timers, obtained from the PVR in the browser window
	 */
	this.refresh = function() {

		JK.removeTemplate( $contentTmpl );
		tma.apiGetTimers();
		self.showTimers();

	};

	/**
	 * this page gets removed, so hide the channels config
	 */
	this.cleanup = function() {

		// remove templates
		JK.removeTemplate($stickyTmpl);
		JK.removeTemplate($contentTmpl);
		JK.removeTemplate($configTmpl);

		// remove handlers
		tma.timers.timers.off(timersAddHandler);
		timersAddHandler = null;

	};

	this.showSticky = function() {

		// render template
		$stickyTmpl = JK.template(
				this.templates.head,
				{
					i18:	this.i18,
					prefs:	prefs,
				}
		).appendTo( '.sticky-content' )

		// add a timer
		.on('click', 'BUTTON.timers-add-timer', function(e) {
			// stop event propagation, needed for 'closeOnOutsideClick'
			e.stopPropagation();

			tma.call('editTimer', [ new TmaTimer({
				isrec:		1,		// record
			})  ] );
		})

		// Timer filter
		.on('click', '.timers-filter-rec', function() {
			prefs.setTypeFilter( prefs.typeFilter ^ 1 );
		})
		.on('click', '.timers-filter-play', function() {
			prefs.setTypeFilter( prefs.typeFilter ^ 2 );
		})

		// set sort event handler
		.on('click', '.timers-head-sort-item', function(e) {
			var sortColumn = $(this).attr('clm-name');
			var sortDir = $(this).attr('sort-dir');
			if (prefs.sort[sortColumn]) {
				prefs.sortColumn = sortColumn;
				prefs.sortDir = sortDir;
				tma.timers.timers.sortBy(prefs.sort[prefs.sortColumn].key, prefs.sortDir);
			}
			$(this).parents( '.timers-head-sort-box' ).removeClass('timers-head-sort-box-open');
		})
		.on('touchstart', '.timers-head-sort-item', function(e) {
			// don't propagate a tap on an item up to box-opener
			e.stopPropagation();
		})

		// open sort options
		.on('touchstart', '.timers-head-sort', function(e) {
			$('.timers-head-sort-box').toggleClass('timers-head-sort-box-open');
		})
		// hover effect with mouse
		.on('mouseenter', '.timers-head-sort', function(e) {
			$('.timers-head-sort-box').addClass('timers-head-sort-box-open');
		})
		.on('mouseleave', '.timers-head-sort', function(e) {
			$('.timers-head-sort-box').removeClass('timers-head-sort-box-open');
		})

		;

	};

	this.showTimers = function () {

		// sort table cells in prefs.columnCfg order
		function rowSort(a,b) {
			var colA = $(a).attr('clm-name');
			var colB = $(b).attr('clm-name');
			var indexA = prefs.columnCfg.findIndex( function(clm) { return (clm.name == colA); } );
			var indexB = prefs.columnCfg.findIndex( function(clm) { return (clm.name == colB); } );
			return indexA - indexB;
		}

		// delete all child nodes of content
//		$('#content').empty();

		$("#timestamp").text( new Date(tma.timers.timestamp).toLocaleString() );

		// sort timers
		tma.timers.timers.sortBy(prefs.sort[prefs.sortColumn].key, prefs.sortDir);

		// prepare Formatters (only once)
		if (!TmaTimer.prototype.timerspageStarttime) {
			TmaTimer.mixin({
				timerspageStarttime: function() {
					"watch: {{starttime}}";
					return tma.date(self.i18.timerdate_fmt, this.starttime);
				},
				timerspageEndtime: function() {
					"watch: {{endtime}}";
					return tma.date(self.i18.timerdate_fmt, this.endtime);
				},
				timerspageShortDate: function(){
					"watch: {{starttime}}";
					return tma.date(self.i18.shortdate_fmt, this.starttime);
				},
				timerspageDay: function() {
					"watch: {{starttime}} {{timestamp}}";
					return function() {
						var now = new Date(this.timestamp);
						now.setHours( 0, 0, 0, 0 );
						var date = new Date(this.starttime);
						date.setHours( 0, 0, 0, 0 );
						return (date.getTime() - now.getTime()) / (24*60*60*1000);
					};
				},
			});
		}

		var templ = prefs.mobileLayout ? this.templates.mobile : this.templates.timerlist;

		$contentTmpl = JK.template( templ, {
			timers			: tma.timers.timers,
			i18				: this.i18,
			prefs			: prefs,
		}).appendTo('DIV#content');

		// sort table columns in user defined order
		$('.timers TR').each(function() {
			$(this).find('TD,TH').sort(rowSort).appendTo(this);
		});

		// hide deselected columns
		$.each(prefs.columnCfg, function() {
			if (!this.display) $contentTmpl.find( ( 'TD[clm-name="{0}"],TH[clm-name="{0}"]' ).format( this.name ) ).hide();
		});

		// set sort event handler
		$contentTmpl.on('click', '.timers-sort', function(e) {
			e.preventDefault();

			var sortColumn = $(this).parent().attr('clm-name');
			if (prefs.sort[sortColumn]) {
				// is this column is currently sorted?
				if (prefs.sort[sortColumn].sorted) {
					// yes => change sort direction
					prefs.sortDir = (prefs.sortDir == 'asc') ? 'desc' : 'asc';
				} else {
					// no => change sortColumn
					prefs.sortColumn = sortColumn;
				}
				tma.timers.timers.sortBy(prefs.sort[prefs.sortColumn].key, prefs.sortDir);
			}
		});

		// add event listener to delete timers
		$contentTmpl.on('click', 'BUTTON.delete-timer', function() {

			var timer = tma.timers.find( $(this).parents('TR').attr('timerid') );
			if ( timer ) {
				self.deleteTimer( timer );
			}

		});

		// add event listener to edit timers
		$contentTmpl.on('click', 'BUTTON.edit-timer', function(e) {

			var $row = $(this).parents('.timer');
			var timer = tma.timers.find( $row.attr('timerid') );
			if ( timer ) {
				tma.call( 'editTimer', [ timer ] ).done(function(popup) {

					popup.relTimer = timer;
					$(popup).on('close', function() {

						var closedPopup = this;
						// remove current mark
						$('.timers .timer[timerid="' + closedPopup.relTimer.id + '"]')
						.removeClass('jk-mark');

					});

					$row.addClass('jk-mark');

				});
			}

		});

		// open more-fn
		$contentTmpl.on('touchend', '.timers-fn-open', function(e) {
			$('.timers-fn .morefn').hide();
			$(this).siblings( '.morefn' ).show();
		});
		// close more-fn
		$contentTmpl.on('touchend', '.timers-fn-close', function(e) {
			$(this).parents( '.morefn' ).hide();
		});

		if (tma.fn.findEvent) {

			// add event listener to view timers event
			$contentTmpl.on('click', '.timer .timers-name', function(e) {

				var $row = $(this).parents('.timer');
				var timer = tma.timers.find( $row.attr('timerid') );
				if (
						timer &&
						!$row.hasClass('jk-mark')
				) {
					var options = {
						channel: 	tma.channels.channel(timer.logoid),
						eventid:	timer.eventid2,
						starttime:	timer.starttime,
						endtime:	timer.endtime,
					};
					if ( options.channel ) {
						tma.call( 'findEvent', [ options ] ).done(function(popup) {

							popup.relTimerId = timer.id;
							$(popup).on('close', function() {

								var closedPopup = this;
								// remove current mark
								$( '.timer[timerid="' + closedPopup.relTimerId + '"]' ).removeClass('jk-mark');

							});

							$row.addClass('jk-mark');

						});
					}
				}

			});

		}

		// set reference to TBODY,
		// if TBODY is detached while adding rows (not yet)
		var $tbody = $contentTmpl.find('TABLE.sortable-columns TBODY');
		if ($tbody.length) {
			// add handler to TmaTimers object, to receive event after template operation
			if (!timersAddHandler) {
				timersAddHandler = tma.timers.on('add', 'timers.*', function(e, timer) {
					var $tr = $tbody.find('TR[timerid="' + timer.id + '"]');
					if ( $tr.length ) {
						// sort columns
						$tr.find('TD').sort(rowSort).appendTo($tr);

						// hide deselected columns
						$.each(prefs.columnCfg, function() {
							if (!this.display) $tr.find( ( 'TD[clm-name="{0}"]' ).format( this.name ) ).hide();
						});
					}
				});
			}
		}

	};

	/**
	 * deletes a reservation
	 *
	 * @param {TmaTimer}	timer
	 */
	this.deleteTimer = function(timer) {

		new jkqDialog({
			title: tma.i18.timer.delete_confirm_caption,
			text:  tma.i18.timer.delete_confirm_text.format( timer.getFilename(true) ),
			autoSkipText: tma.i18.respond_auto_skip,
			autoSkipButton: 'tmaDeleteTimer',
			buttons: {
				tmaDeleteTimer:	 tma.i18.respond_yes,
				jkqDialogCancel: tma.i18.respond_cancel
			}
		}).confirm(function(button) {

			if (button == 'tmaDeleteTimer') {

				tma.apiDeleteTimer(timer).fail(function(xhr, errorType, error) {

					new jkqAlert({
						title:	tma.i18.error.dialog_title,
						text:	tma.i18.error.timer.del + '<br><br>' + error
					}).confirm();

				});

			}

		});

	};

	this.insertConfig = function () {

		// setup column config module
		var columnconfig = tma.fn.getModule('columnconfig');
		columnconfig.init().done(function() {

			// prepare template data (mixed with data, required by column config)
			var data = $.extend(
					{
						i18:			self.i18,
						i18g:			tma.i18,	// global language def.
						prefs:			prefs,
					},
					columnconfig.getTemplateData( prefs.columnCfg,	self.i18.columns )
			);

			$configTmpl = JK.template(
					self.templates.config,
					data,
					{ columnconfig: columnconfig.templates.content }
			).appendTo('DIV#config');

			// add column config event handlers
			columnconfig.addEventHandler(
					$configTmpl,
					'.timers TABLE',
					function(cfg) {
						self.setLocalSetting('columns', cfg );
					}
			);

			// logo/text
			$('DIV#config INPUT[name="logotext"]')
			.click(function() {
				prefs.setDisplayMode( $('DIV#config INPUT[name="logotext"]:checked').val() );
			})
			.filter('[value="' +  prefs.displayMode + '"]' ).attr('checked', true);

			// mobile layout
			$('DIV#config INPUT[name="mobile"]')
			.click(function() {
				prefs.setMobileLayout( $(this).is(':checked') );
				JK.removeTemplate( $contentTmpl );
				self.showTimers();
			});

		});

	};

	/* Settings */

	var prefs = JK.model({

		/**
		 * default sor column setting
		 *
		 * @type {String}
		 */
		sortColumn: 'starttime',

		/**
		 * displayMode setting
		 *
		 * @type {string}	asc:  sort ascending
		 * 					desc: sort descending
		 */
		sortDir: 'asc',

		sort: {
			filename	: { sorted: false, key: 'filename'},
			starttime	: { sorted: true , key: 'starttime'},
			endtime		: { sorted: false, key: 'endtime'},
			duration	: { sorted: false, key: 'duration'},
			channel		: { sorted: false, key: 'chname'},
			frequency	: { sorted: false, key: 'reservationtype'},
		},

		/**
		 * displayMode setting
		 *
		 * @type {Number}		1: display logo
		 * 						2: display name
		 * 						3: display both
		 */
		displayMode: 3,

		/**
		 * set the displayMode setting
		 *
		 * @param {Number}	set		new displayMode setting
		 * 							1: display logo
		 * 							2: display name
		 * 							3: display both
		 */
		setDisplayMode: function(set) {
			var val = parseInt(set);
			if ((val >= 1) && (val <= 3)) {
				self.setLocalSetting( 'channel_display', val );
				this.displayMode = val;
			}
		},

		/**
		 * timer filter setting
		 *
		 * @type {Number}			0: display all
		 * 							1: display rec timers
		 * 							2: display play timers
		 * 							3: display all
		 */
		typeFilter: 3,

		/**
		 * set the timer filter setting
		 *
		 * @param {Number}	set		0: display all
		 * 							1: display rec timers
		 * 							2: display play timers
		 * 							3: display all
		 */
		setTypeFilter: function(set) {
			var val = parseInt(set);
			if ((val >= 0) && (val <= 3)) {
				self.setLocalSetting( 'timersfilter', val );
				this.typeFilter = val;
			}
		},

		columnCfg: [
					{ name: 'timertype'	, display: true },
					{ name: 'filename'	, display: true },
					{ name: 'starttime'	, display: true },
					{ name: 'endtime'	, display: true },
					{ name: 'duration'	, display: true },
					{ name: 'channel'	, display: true },
					{ name: 'frequency'	, display: true },
					{ name: 'timerfn'	, display: true }
			],

		setColumnCfg: function(cfg) {
			if (cfg) {
				// sort
				this.columnCfg.sort( function(a, b) {
					return Object.keys(cfg).indexOf(a.name) - Object.keys(cfg).indexOf(b.name);
				});
				// set display
				for( var i = 0; i < this.columnCfg.length; i++) {
					if (cfg[this.columnCfg[i].name] !== undefined) {
						this.columnCfg[i].display = cfg[this.columnCfg[i].name];
					}
				}
			}
		},

		/**
		 * mobile layout setting
		 */
		mobileLayout: false,

		/**
		 * set the mobile layout setting
		 *
		 * @param {Boolean}	set
		 */
		setMobileLayout: function(set) {
			var val = set === true;
			self.setLocalSetting( 'mobilelayout', val);
			this.mobileLayout = val;
		},

	});

	prefs.on('change', 'sortColumn', function(e, val, obj, prop, oldVal) {
		self.setLocalSetting( 'sort_clm', val );
		if (oldVal && prefs.sort[oldVal]) {
			prefs.sort[oldVal].sorted	= false;
		}
		if (val && prefs.sort[val])	{
			prefs.sort[val].sorted		= true;
		}
	});

	prefs.on('change', 'sortDir', function(e, val) {
		self.setLocalSetting( 'sort_dir', val );
	});

}
